Package: TPP2D
Title: Detection of ligand-protein interactions from 2D thermal
        profiles (DLPTP)
Version: 1.28.0
Authors@R: c(
    person("Nils", "Kurzawa", email = "nilskurzawa@gmail.com", 
        role = c("aut", "cre")),
    person("Holger", "Franken", role = c("aut")),
    person("Simon", "Anders", role = c("aut")),
    person("Wolfgang", "Huber", role = c("aut")),
    person("Mikhail M.", "Savitski", role = c("aut")))
Description: Detection of ligand-protein interactions from 2D thermal 
    profiles (DLPTP), Performs an FDR-controlled analysis of 2D-TPP 
    experiments by functional analysis of dose-response curves across 
    temperatures.
License: GPL-3
Encoding: UTF-8
VignetteBuilder: knitr
LazyData: false
biocViews: Software, Proteomics, DataImport
BugReports: https://support.bioconductor.org/
URL: http://bioconductor.org/packages/TPP2D
RoxygenNote: 7.3.3
Depends: R (>= 3.6.0), stats, utils, dplyr, methods
Imports: ggplot2, tidyr, foreach, doParallel, openxlsx, stringr, RCurl,
        parallel, MASS, BiocParallel, limma
Suggests: knitr, testthat, rmarkdown, BiocStyle
git_url: https://git.bioconductor.org/packages/TPP2D
git_branch: RELEASE_3_23
git_last_commit: 42f758f
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:46:44 UTC; biocbuild
Author: Nils Kurzawa [aut, cre],
  Holger Franken [aut],
  Simon Anders [aut],
  Wolfgang Huber [aut],
  Mikhail M. Savitski [aut]
Maintainer: Nils Kurzawa <nilskurzawa@gmail.com>
Built: R 4.6.0; ; 2026-04-29 05:04:11 UTC; unix

# Generated by roxygen2: do not edit by hand

export(TPP_importCheckConfigTable)
export(annotateDataList)
export(bootstrapNull)
export(bootstrapNullAlternativeModel)
export(bootstrapNullAlternativeModelFast)
export(competeModels)
export(computeFStatFromParams)
export(computeFstat)
export(configWide2Long)
export(filterOutContaminants)
export(findHits)
export(fitAndEvalDataset)
export(fitH0Model)
export(fitH1Model)
export(getFDR)
export(getModelParamsDf)
export(getPEC504Temperature)
export(getPvalues)
export(gg_qq)
export(import2dDataset)
export(import2dMain)
export(plot2dTppFcHeatmap)
export(plot2dTppFit)
export(plot2dTppProfile)
export(plot2dTppRelProfile)
export(plot2dTppVolcano)
export(recomputeSignalFromRatios)
export(renameColumns)
export(resolveAmbiguousProteinNames)
export(runTPP2D)
export(tpcaResult)
exportClasses(tpp2dExperiment)
import(BiocParallel)
import(dplyr)
import(ggplot2)
importFrom(MASS,lda)
importFrom(RCurl,url.exists)
importFrom(doParallel,registerDoParallel)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(limma,squeezeVar)
importFrom(methods,is)
importFrom(methods,new)
importFrom(openxlsx,read.xlsx)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(stats,approx)
importFrom(stats,coef)
importFrom(stats,density)
importFrom(stats,fft)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,optim)
importFrom(stats,p.adjust)
importFrom(stats,predict)
importFrom(stats,residuals)
importFrom(stats,sd)
importFrom(stringr,str_to_title)
importFrom(tidyr,gather)
importFrom(tidyr,spread)
importFrom(utils,head)
importFrom(utils,read.delim)
importFrom(utils,read.table)

# TPP2D

[![codecov](https://codecov.io/gh/nkurzaw/TPP2D/branch/master/graph/badge.svg)](https://codecov.io/gh/nkurzaw/TPP2D)


> Detection of ligand-protein interactions from thermal proteome profiles (DLPTP) by FDR-controlled analysis of 2D-TPP experiments

This package contains functions to analyze 2D-thermal proteome profiles using the DLPTP (detection of ligand-protein interactions from thermal proteome profiles) approach. This is done by fitting two nested models (null and alternative) to the thermal profile of each protein and asking whether the alternative model explains the variance in the data significantly better than the null.

<img align="right" src="inst/tpp2DHexSticker.png" height="300"/>

## Installation

### Installation from Bioconductor (recommended)

The `TPP2D` package has been accepted into Bioconductor! Therefore, the most reliable way to install it is via BiocManager:

```{R}
if (!requireNamespace("BiocManager", quietly = TRUE))
install.packages("BiocManager")
BiocManager::install("TPP2D")
```

### Installation of the development version

```{R}
BiocManager::install(“nkurzaw/TPP2D”)
```

## Getting started

The easiest way of learning how to use the `TPP2D` package is to browse it's vignette:
```{R}
library(TPP2D)
browseVignettes("TPP2D")
```

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----getPackage, eval=FALSE---------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
# install.packages("BiocManager")
# BiocManager::install("TPP2D")

## ----eval = FALSE-------------------------------------------------------------
# BiocManager::install("nkurzaw/TPP2D")

## ----Load, message=FALSE------------------------------------------------------
library(TPP2D)

## ----echo=FALSE, fig.cap="Experimental 2D-TPP workflow."----------------------
knitr::include_graphics("tpp_2d_schematic.jpg")

## ----message=FALSE, warning=FALSE---------------------------------------------
library(dplyr)
library(TPP2D)

## -----------------------------------------------------------------------------
data("config_tab")
data("raw_dat_list")

config_tab

## ----warning=FALSE------------------------------------------------------------
import_df <- import2dDataset(
    configTable = config_tab,
    data = raw_dat_list,
    idVar = "protein_id",
    intensityStr = "signal_sum_",
    fcStr = "rel_fc_",
    nonZeroCols = "qusm",
    geneNameVar = "gene_name",
    addCol = NULL,
    qualColName = "qupm",
    naStrs = c("NA", "n/d", "NaN"),
    concFactor = 1e6,
    medianNormalizeFC = TRUE,
    filterContaminants = TRUE)

recomp_sig_df <- recomputeSignalFromRatios(import_df)

# resolve ambiguous protein names
preproc_df <- resolveAmbiguousProteinNames(recomp_sig_df)
# alternatively one could choose to run
# preproc_df <- resolveAmbiguousProteinNames(
#     recomp_sig_df, includeIsoforms = TRUE)

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(tibble(
    column = colnames(recomp_sig_df),
    description = 
            c("protein identifier",
            "number of unique quantified peptides",
            "number of unique spectra",
            "gene name",
            "temperature incubated at",
            "experiment identifier",
            "TMT label",
            "RefCol",
            "treatment concentration",
            "raw reporter ion intensity sum",
            paste("raw relative fold change compared to",
                    "vehicle condition at the same temperature"),
            "log10 treatment concentration",
            "median normalized fold change",
            "recomputed reporter ion intensity",
            "recomputed log2 reporter ion intensity"),
    required = 
            c("Yes",
            "No",
            "No",
            "Yes",
            "Yes",
            "No",
            "No",
            "No",
            "No",
            "No",
            "No",
            "Yes",
            "No",
            "No",
            "Yes"))
)

## -----------------------------------------------------------------------------
model_params_df <- getModelParamsDf(
    df = preproc_df)

## -----------------------------------------------------------------------------
fstat_df <- computeFStatFromParams(model_params_df)

## -----------------------------------------------------------------------------
set.seed(12, kind = "L'Ecuyer-CMRG")
null_model <- bootstrapNullAlternativeModel(
    df = preproc_df,
    params_df = model_params_df,
    B = 2)

## ----warning=FALSE------------------------------------------------------------
fdr_tab <- getFDR( 
    df_out = fstat_df,
    df_null = null_model)

hits <- findHits(
    fdr_df = fdr_tab,
    alpha = 0.1)

hits %>% 
    dplyr::select(clustername, nObs, F_statistic, FDR)

## -----------------------------------------------------------------------------
plot2dTppVolcano(fdr_df = fdr_tab, hits_df = hits)

## -----------------------------------------------------------------------------
plot2dTppFit(recomp_sig_df, "tp1", 
             model_type = "H0",
             dot_size = 1,
             fit_color = "darkorange")

## -----------------------------------------------------------------------------
plot2dTppFit(recomp_sig_df, "tp1", 
             model_type = "H1",
             dot_size = 1,
             fit_color = "darkgreen")

## -----------------------------------------------------------------------------
plot2dTppFcHeatmap(recomp_sig_df, "tp1", 
                   drug_name = "drug X")

## -----------------------------------------------------------------------------
plot2dTppFcHeatmap(recomp_sig_df, "tp3", 
                   drug_name = "drug X")

## -----------------------------------------------------------------------------
sessionInfo()

#' Recompute robust signal intensities based on 
#' bootstrapped TMT channel ratios
#' 
#' @param df tidy data_frame retrieved after 
#' import of a 2D-TPP dataset
#' 
#' @return A data_frame with recomputed signal 
#' intensities (columname: value) and log2 
#' transformed signal intensities (columnanme: log2_value) 
#' that more reliably reflect relative ratios 
#' between the TMT channels
#' 
#' @export
#'
#' @examples
#' 
#' data("simulated_cell_extract_df")
#' recomputeSignalFromRatios(simulated_cell_extract_df)
#' 
#' @import dplyr
recomputeSignalFromRatios <- function(df){
  clustername <- log2_value <- raw_value <- rel_value <- 
    representative <- sum_raw <- sum_rel <- temperature <- 
    value <- NULL
  
  df_recomp <- df %>%
    group_by(representative, clustername, temperature) %>%
    mutate(sum_raw = sum(raw_value, na.rm = TRUE),
           sum_rel = sum(rel_value, na.rm = TRUE)) %>%
    mutate(value = (rel_value/sum_rel)*sum_raw) %>%
    ungroup() %>%
    dplyr::select(-sum_raw, -sum_rel) %>%
    mutate(log2_value = log2(value)) %>%
    filter(!is.na(log2_value))
  
  return(df_recomp)
}

#' Resolve ambiguous protein names
#' 
#' @param df tidy data_frame retrieved after 
#' import of a 2D-TPP dataset
#' @param includeIsoforms logical indicating
#' whether protein isoform should be kept for
#' analysis
#' 
#' @return data frame with resolved protein name
#' ambiguity
#' 
#' @examples 
#' tst_df <- bind_rows(tibble(representative = rep(1:3, each = 3), 
#'                            clustername = rep(letters[1:3], each = 3)), 
#'                     tibble(representative = rep(c(4, 5), c(3, 2)), 
#'                            clustername = rep(c("a", "b"), c(3, 2))))
#'                            
#' resolveAmbiguousProteinNames(tst_df)
#' @export
resolveAmbiguousProteinNames <- function(df, includeIsoforms = FALSE){
    representative <- clustername <- nObs <- NULL
    lookUpDf <- df %>% 
        group_by(representative, clustername) %>% 
        summarize(nObs = n()) %>% 
        ungroup() 
    
    if(includeIsoforms){
        df_fil <- df %>% 
            rowwise() %>% 
            mutate(clustername = 
                       paste(representative,
                             clustername, sep = "_")) %>% 
            ungroup
    }else{
        lookUpDf <- lookUpDf %>% 
            group_by(clustername) %>% 
            filter(nObs == max(nObs)) %>% 
            ungroup %>% 
            filter(!duplicated(clustername)) 
        
        df_fil <- df %>% 
            filter(representative %in% 
                       lookUpDf$representative) %>% 
          mutate(protein_id = paste(
            representative,
            clustername,
            sep = "_"
          )) %>% 
          mutate(representative = clustername)
    }
    return(df_fil)
}

## Tests for resolveAmbiguousProteinNames()
## library(TPP2D); library(testthat); source("setup-2dtpp-dataset.R"); source("test-recompute-signal.R")
context("resolve ambiguous protein names")

tst_df <- bind_rows(
    simulated_cell_extract_df,
    filter(simulated_cell_extract_df, 
           clustername == "tp3",
           temperature == 62) %>% 
        mutate(representative = "99")
)

test_that("resolveAmbiguousProteinNames works as expected", {
    out_df <- resolveAmbiguousProteinNames(tst_df)
    expect_equal(nrow(filter(out_df, clustername == "tp3")), 50)
})

test_that(paste("resolveAmbiguousProteinNames works as expected", 
                "with includeIsoforms = FALSE"), {
    out_df <- resolveAmbiguousProteinNames(
        tst_df, includeIsoforms = TRUE)
    expect_equal(nrow(filter(out_df, clustername == "20_tp3")), 50)
})

## Tests for recomputeSignalFromRatios()
## library(TPP2D); library(testthat); source("setup-2dtpp-dataset.R"); source("test-compute-F-stat-from-params.R")
context("compute F stat from params")

test_that("computeFStatFromParams works as expected", {
    params_df <- getModelParamsDf(simulated_cell_extract_df %>%
                                      filter(representative %in% 1:3))
    fstat_df <- computeFStatFromParams(params_df)
    expect_equal(round(fstat_df$F_statistic, 2), c(1.23, 0.36, 0.24), tolerance = 0.015)
})

## Tests for import2dDataset()
## library(TPP2D); library(testthat); source("setup-raw-2dtpp-dataset.R"); source("test-import-raw_data.R")
context("import data")

test_that("import2dDataset works as expected", {
  import_df <- import2dDataset(configTable = config_tab, 
                               data = raw_dat_list,
                               idVar = "protein_id",
                               intensityStr = "signal_sum_",
                               fcStr = "rel_fc_",
                               nonZeroCols = "qusm",
                               geneNameVar = "gene_name",
                               addCol = NULL,
                               qualColName = "qupm",
                               naStrs = c("NA", "n/d", "NaN"),
                               concFactor = 1e6,
                               medianNormalizeFC = TRUE,
                               filterContaminants = TRUE)
  recomp_sig_df <- data.frame(recomputeSignalFromRatios(import_df))
  expect_equivalent(data.frame(simulated_cell_extract_df), recomp_sig_df)
})

## Tests for recomputeSignalFromRatios()
## library(TPP2D); library(testthat); source("setup-2dtpp-dataset.R"); source("test-compete-models.R")
context("compete models")

test_that("competeModels works as expected", {
  cm_df <- competeModels(simulated_cell_extract_df %>% 
                           filter(clustername == "tp1"))
  expect_identical(round(cm_df$F_statistic, -2), 100)
})

test_that("competeModels throws error", {
  expect_error(competeModels(simulated_cell_extract_df %>% 
                               dplyr::select(-log2_value)))
})

## Tests for recomputeSignalFromRatios()
## library(TPP2D); library(testthat); source("setup-2dtpp-dataset.R"); source("test-recompute-signal.R")
context("recompute signal from ratios")

test_that("recomputeSignalFromRatios works as expected", {
  out <- data.frame(recomputeSignalFromRatios(simulated_cell_extract_df))
  expect_equivalent(data.frame(simulated_cell_extract_df), out)
})

#' Run complete TPP2D analysis
#'
#' @param df tidy data_frame retrieved after import of a 2D-TPP 
#' dataset, potential filtering and addition of a column "nObs"
#' containing the number of observations per protein
#' @param configTable character string of a file path to a config table
#' @param data possible list of datasets from different MS runs 
#' corresponding to a 2D-TPP dataset, circumvents loading datasets 
#' referencend in config table, default is NULL
#' @param idVar character string indicating which data column provides the 
#'   unique identifiers for each protein.
#' @param intensityStr character string indicating which columns contain 
#'   raw intensities measurements
#' @param fcStr character string indicating which columns contain the actual 
#'   fold change values. Those column names containing the suffix \code{fcStr} 
#'   will be regarded as containing fold change values.
#' @param naStrs character vector indicating missing values in the data table. 
#'   When reading data from file, this value will be passed on to the argument 
#'   \code{na.strings} in function \code{read.delim}.
#' @param qualColName character string indicating which column can be used for 
#'   additional quality criteria when deciding between different non-unique 
#'   protein identifiers.
#' @param medianNormalizeFC perform median normalization (default: TRUE).
#' @param addCol character string indicating additional column to import
#' @param filterContaminants logical variable indicating whether data 
#' should be filtered to exclude contaminants (default: TRUE).
#' @param recomputeSignalRatios logical variable indicaiting whether 
#' signals should be recomputed from relative fold changes, recommended
#' if Isobarquant was used for protein quantification
#' @param minObs number of minimal observations per protein to include it
#' in the analysis
#' @param independentFiltering logical variable indicating whether
#' independent filtering should be performed based on minimal
#' fold changes per protein profile
#' @param fcThres numeric value of minimal fold change 
#' (or inverse fold change) a protein has to show to be kept 
#' upon independent filtering
#' @param nonZeroCols column like default qssm that should be imported and
#' requested to be non-zero in analyzed data
#' @param geneNameVar character string of the column name that describes
#' the gene name of a given protein in the raw data files
#' @param concFactor numeric value that indicates how concentrations need to 
#' be adjusted to yield total unit e.g. default mmol - 1e6
#' @param maxit maximal number of iterations the optimization
#' should be given, default is set to 500
#' @param optim_fun_h0 optimization function that should be used
#' for fitting the H0 model
#' @param optim_fun_h1 optimization function that should be used
#' for fitting the H1 model
#' @param optim_fun_h1_2 optional additional optimization function 
#' that will be run with paramters retrieved from optim_fun_h1 and 
#' should be used for fitting the H1 model with the trimmed sum
#' model, default is NULL
#' @param gr_fun_h0 optional gradient function for optim_fun_h0,
#' default is NULL
#' @param gr_fun_h1 optional gradient function for optim_fun_h1,
#' default is NULL
#' @param gr_fun_h1_2 optional gradient function for optim_fun_h1_2,
#' default is NULL
#' @param slopEC50 logical flag indicating whether the h1 model is
#' fitted with a linear model describing the shift od the pEC50 over 
#' temperatures
#' @param BPPARAM = BiocParallel::SerialParam(progressbar = TRUE),
#' @param B numeric value indicating number of rounds of bootstraps
#' that should be performed to estimate the null distribution
#' @param byMsExp logical indicating whether bootstrapping should be
#' performed within MS experiments
#' @param alpha FDR level that should be controlled
#' 
#' @return a tpp2dExperiment object
#' 
#' @examples 
#' data("simulated_cell_extract_df")
#' runTPP2D(df = simulated_cell_extract_df %>% 
#'    filter(representative %in% 1:3),
#'    B = 1)
#' 
#' @export
#' 
#' @importFrom methods new
#' 
runTPP2D <- function(df = NULL,
                     configTable = NULL, 
                     data = NULL,
                     idVar = "protein_id",
                     intensityStr = "signal_sum_",
                     fcStr = "rel_fc_",
                     nonZeroCols = "qusm",
                     geneNameVar = "gene_name",
                     addCol = NULL,
                     qualColName = "qupm",
                     naStrs = c("NA", "n/d", "NaN"),
                     concFactor = 1e6,
                     medianNormalizeFC = TRUE,
                     filterContaminants = TRUE,
                     recomputeSignalRatios = FALSE,
                     minObs = 20,
                     independentFiltering = FALSE,
                     fcThres = 1.5,
                     optim_fun_h0 = .min_RSS_h0,
                     optim_fun_h1 = .min_RSS_h1_slope_pEC50,
                     optim_fun_h1_2 = NULL,
                     gr_fun_h0 = NULL,
                     gr_fun_h1 = NULL,
                     gr_fun_h1_2 = NULL,
                     slopEC50 = TRUE,
                     maxit = 750,
                     BPPARAM = BiocParallel::SerialParam(progressbar = TRUE),
                     B = 20,
                     byMsExp = TRUE,
                     alpha = 0.1){
    
    if(is.null(df) & !is.null(configTable)){
        message("Importing data")
        import_df <- import2dDataset(configTable = configTable, 
                                     data = data,
                                     idVar = idVar,
                                     intensityStr = intensityStr,
                                     fcStr = fcStr,
                                     nonZeroCols = nonZeroCols,
                                     geneNameVar = geneNameVar,
                                     addCol = addCol,
                                     qualColName = qualColName,
                                     naStrs = naStrs,
                                     concFactor = concFactor,
                                     medianNormalizeFC = medianNormalizeFC,
                                     filterContaminants = filterContaminants)
        
        cfgTab <- TPP_importCheckConfigTable(
            infoTable = configTable, type = "2D")
        .checkDfColumns(import_df)
        df <- import_df
    }else if(!is.null(df)){
        cfgTab <- data.frame()
    }else if(is.null(configTable)){
        stop(paste("Either a data.frame (df) or a configTable",
                   "need to be supplied in order to perform",
                   "the analysis!\n"))
    }
    
    if(identical(optim_fun_h1, .min_RSS_h1_slope_pEC50)){
        slopEC50 = TRUE
    }else{
        slopEC50 = FALSE
    }
    
    if(recomputeSignalRatios){
        df <- recomputeSignalFromRatios(df)
    }
    
    if(independentFiltering){
        message(paste("Independent Filtering: removing proteins without", 
                      "any values crossing the set threshold.\n"))
        df <- .independentFilter(df, fcThres = fcThres) 
    }
    
    message("Computing null and alternative model parameters\n")
    
    params_df <- getModelParamsDf(df,
                                  optim_fun_h0 = optim_fun_h0,
                                  optim_fun_h1 = optim_fun_h1,
                                  optim_fun_h1_2 = optim_fun_h1_2,
                                  gr_fun_h0 = gr_fun_h0,
                                  gr_fun_h1 = gr_fun_h1,
                                  gr_fun_h1_2 = gr_fun_h1_2,
                                  slopEC50 = slopEC50,
                                  maxit = maxit,
                                  qualColName = qualColName)
    
    message("Computing F statistics\n")
    
    fstat_df <- computeFStatFromParams(params_df)
    
    message("Bootstrapping null distribution\n")
    
    null_df <- bootstrapNullAlternativeModel(df, 
                 params_df,
                 maxit = maxit,
                 minObs = minObs,
                 optim_fun_h0 = optim_fun_h0,
                 optim_fun_h1 = optim_fun_h1,
                 optim_fun_h1_2 = optim_fun_h1_2,
                 gr_fun_h0 = gr_fun_h0,
                 gr_fun_h1 = gr_fun_h1,
                 gr_fun_h1_2 = gr_fun_h1_2,
                 BPPARAM = BPPARAM,
                 B = B,
                 byMsExp = byMsExp)
    
    message("Computing FDR\n")
    
    fdr_df <- getFDR(df_out = fstat_df,
                     df_null = null_df)
    hits_df <- findHits(fdr_df = fdr_df,
                        alpha = alpha)
    
    tpp2dObj <- new("tpp2dExperiment",
                    configTable = cfgTab,
                    idVar = idVar,
                    intensityStr = intensityStr,
                    fcStr = fcStr,
                    nonZeroCols = nonZeroCols,
                    geneNameVar = geneNameVar,
                    qualColName = qualColName,
                    naStrs = naStrs,
                    concFactor = concFactor,
                    medianNormalizeFC = medianNormalizeFC,
                    filterContaminants = filterContaminants,
                    minObs = minObs,
                    independentFiltering = independentFiltering,
                    fcThres = fcThres,
                    optim_fun_h0 = optim_fun_h0,
                    optim_fun_h1 = optim_fun_h1,
                    slopEC50 = slopEC50,
                    maxit = maxit,
                    BPPARAM = class(BPPARAM)[1],
                    B = B,
                    byMsExp = byMsExp,
                    alpha = alpha,
                    tidyDataTable = df,
                    modelParamsDf = params_df,
                    resultTable = fstat_df,
                    bootstrapNullDf = null_df,
                    hitTable = hits_df)
    return(tpp2dObj)
}

#' @import dplyr
.removeAmbiguousMeasurements <- function(in_df, 
                                         qualColName = "qupm"){
    representative <- temperature <- conc <- value <- 
        raw_value <- NULL
    
    out_df <- in_df %>% 
        group_by(representative, temperature, conc) %>% 
        filter_(paste(qualColName, " == max(", qualColName, ")",
                      collapse = "")) %>% 
        filter(raw_value == max(raw_value)) %>% 
        group_by(representative) %>% 
        filter_(paste("any(", qualColName, " > 1)", collapse = "")) %>% 
        arrange(temperature, conc) %>% 
        mutate(temp_id = dense_rank(temperature),  
               conc_id = dense_rank(conc)) %>% 
        ungroup
    
    return(out_df)
}

#' @import dplyr
#' @importFrom stats sd
.detectOutliers <- function(in_df){
    representative <- temp_id <- conc_id <- NULL
    
    out_df <- bind_rows(lapply(unique(in_df$representative), function(repr){
        temp_repr <- filter(in_df, representative == repr)
        outlier_score <- 
            lapply(seq_len(nrow(temp_repr)), function(i){
                temp2 <- filter(temp_repr, temp_id %in% 
                                    c(temp_repr$temp_id[i] - 1, 
                                      temp_repr$temp_id[i], 
                                      temp_repr$temp_id[i] + 1), 
                                conc_id %in% 
                                    c(temp_repr$conc_id[i] - 1, 
                                      temp_repr$conc_id[i], 
                                      temp_repr$conc_id[i] + 1), 
                                !(temp_id == temp_repr$temp_id[i] & 
                                      conc_id == temp_repr$conc_id[i]))
                factor_sd1 <- abs(log2(filter(temp_repr, temp_id == temp_repr$temp_id[i] & 
                                                  conc_id == temp_repr$conc_id[i])$rel_value) - 
                                      mean(log2(temp2$rel_value), na.rm = TRUE))/ 
                    sd(log2(temp2$rel_value), na.rm = TRUE)
                factor_sd2 <- abs(log2(filter(temp_repr, temp_id == temp_repr$temp_id[i] & 
                                                  conc_id == temp_repr$conc_id[i])$rel_value) - 
                                      mean(log2(temp2$rel_value), na.rm = TRUE))/ 
                    sd(log2(temp_repr$rel_value), na.rm = TRUE)
                temp3 <- filter(temp_repr, temp_id %in% 
                                    c(temp_repr$temp_id[i] - 1, 
                                      temp_repr$temp_id[i], 
                                      temp_repr$temp_id[i] + 1), 
                                conc_id %in% 
                                    c(temp_repr$conc_id[i] - 1, 
                                      temp_repr$conc_id[i], 
                                      temp_repr$conc_id[i] + 1))
                return(list("factor1" = factor_sd1,
                            "factor2" = factor_sd2,
                            "shrinked_value" = mean(temp3$rel_value, na.rm = TRUE),
                            "conc_edge" = (temp_repr$conc_id[i] == max(temp_repr$conc_id))))
            })
        if(!is.null(outlier_score)){
            temp_df <- temp_repr 
            temp_df$outlier_score_local = 
                sapply(outlier_score, function(x) x[["factor1"]])
            temp_df$outlier_score_global =
                sapply(outlier_score, function(x) x[["factor2"]])
            temp_df$shrinked_value = 
                sapply(outlier_score, function(x) x[["shrinked_value"]])
            temp_df$conc_edge = 
                sapply(outlier_score, function(x) x[["conc_edge"]])
        }else{
            temp_df <- temp_repr %>% 
                mutate(outlier_score_local = NA,
                       outlier_score_global = NA,
                       shrinked_value = NA,
                       conc_edge = NA)
        }
        
        return(temp_df)
    }))
    
    return(out_df)
}

#' Fit H0 model and evaluate fit statistics
#' 
#' @param df tidy data_frame retrieved after import of a 2D-TPP 
#' dataset, potential filtering and addition of a column "nObs"
#' containing the number of observations per protein
#' @param maxit maximal number of iterations the optimization
#' should be given, default is set to 500
#' @param optim_fun optimization function that should be used
#' for fitting the H0 model
#' @param gr_fun optional gradient function for optim_fun,
#' default is NULL
#' 
#' @return data frame with H0 model characteristics for each
#' protein
#' 
#' @export
#'
#' @examples
#' 
#' data("simulated_cell_extract_df")
#' temp_df <- simulated_cell_extract_df %>% 
#'   filter(clustername %in% paste0("protein", 1:5)) %>% 
#'   group_by(representative) %>% 
#'   mutate(nObs = n()) %>% 
#'   ungroup 
#'   
#' fitH0Model(temp_df)
#' 
#' @import dplyr
#' @importFrom stats optim
fitH0Model <- function(df, 
                       maxit = 500,
                       optim_fun = .min_RSS_h0,
                       gr_fun = NULL){
  
  representative <- clustername <- nObs <- 
    temperature <- . <- NULL
  
  h0_df <- df %>%
    group_by(representative, clustername, nObs) %>%
    mutate(temp_i = dense_rank(temperature)) %>%
    do({
      unique_temp <- unique(.$temperature)
      len_temp <- length(unique_temp)
      start_par = vapply(unique_temp, function(x)
        mean(filter(., temperature == x)$log2_value), 1)
      h0_model = try(optim(par = start_par,
                       fn = optim_fun,
                       len_temp = len_temp,
                       data = .,
                       method = "L-BFGS-B",
                       gr = gr_fun,
                       control = list(maxit = maxit)))
      .eval_optim_result(h0_model, hypothesis = "H0",
                        data = .)
    }) %>%
    group_by(representative, clustername) %>%
    ungroup()
  
  return(h0_df)
}

.fitEvalH1 <- function(df_fil, unique_temp, len_temp,
                      optim_fun = .min_RSS_h1_slopeEC50, 
                      optim_fun_2 = NULL,
                      gr_fun = NULL,
                      gr_fun_2 = NULL,
                      ec50_lower_limit = NULL,
                      ec50_upper_limit= NULL,
                      slopEC50 = TRUE, maxit = 500){
  .min_RSS_h1_slopeEC50 <- NULL
  if(is.null(ec50_lower_limit)){
    ec50_lower_limit <- min(unique(df_fil$log_conc)[
      which(is.finite(unique(df_fil$log_conc)))])
  }
  if(is.null(ec50_upper_limit)){
    ec50_upper_limit <- max(unique(df_fil$log_conc)[
      which(is.finite(unique(df_fil$log_conc)))])
  }
  start_par = .getStartParameters(
    df = df_fil, 
    unique_temp = unique_temp, 
    len_temp = len_temp, 
    slopEC50 = slopEC50)
  opt_limits <- .getOptLimits(
    ec50Limits = c(ec50_lower_limit, ec50_upper_limit),
    len_temp = len_temp, 
    slopEC50 = slopEC50)
  lower = opt_limits$lower 
  upper = opt_limits$upper
  h1_model = try(optim(par = start_par,
                       fn = optim_fun,
                       gr = gr_fun,
                       len_temp = len_temp,
                       data = df_fil,
                       method = "L-BFGS-B",
                       upper = upper,
                       lower = lower,
                       control = list(maxit = maxit)))
  if(!is.null(optim_fun_2) & 
     !is(h1_model, "try-error")){
    h1_model = try(optim(par = h1_model$par,
                         fn = optim_fun_2,
                         gr = gr_fun_2,
                         len_temp = len_temp,
                         data = df_fil,
                         method = "L-BFGS-B",
                         upper = upper,
                         lower = lower,
                         control = list(maxit = maxit)))
  }
  return(h1_model)
}

#' Fit H1 model and evaluate fit statistics
#' 
#' @param df tidy data_frame retrieved after import of a 2D-TPP 
#' dataset, potential filtering and addition of a column "nObs"
#' containing the number of observations per protein
#' @param maxit maximal number of iterations the optimization
#' should be given, default is set to 500
#' @param optim_fun optimization function that should be used
#' for fitting the H0 model
#' @param optim_fun_2 optional secound optimization function for
#' fitting the H1 model that should be used based on the fitted 
#' parameters of the optimizationfor based on optim_fun
#' @param gr_fun optional gradient function for optim_fun,
#' default is NULL
#' @param gr_fun_2 optional gradient function for optim_fun_2,
#' default is NULL
#' @param ec50_lower_limit lower limit of ec50 parameter
#' @param ec50_upper_limit lower limit of ec50 parameter
#' @param slopEC50 logical flag indicating whether the h1 model is
#' fitted with a linear model describing the shift od the pEC50 over 
#' temperatures
#' 
#' @return data frame with H1 model characteristics for each
#' protein
#' 
#' @export
#'
#' @examples
#' 
#' data("simulated_cell_extract_df")
#' temp_df <- simulated_cell_extract_df %>% 
#'   filter(clustername %in% paste0("protein", 1:5)) %>% 
#'   group_by(representative) %>% 
#'   mutate(nObs = n()) %>% 
#'   ungroup
#'   
#' fitH1Model(temp_df)
#' 
#' @import dplyr
#' @importFrom stats optim
#' @importFrom methods is
fitH1Model <- function(df, 
                       maxit = 500,
                       optim_fun = .min_RSS_h1_slope_pEC50,
                       optim_fun_2 = NULL,
                       gr_fun = NULL,
                       gr_fun_2 = NULL,
                       ec50_lower_limit = NULL,
                       ec50_upper_limit = NULL,
                       slopEC50 = TRUE){
  
  representative <- clustername <- nObs <- 
    temperature <- . <- NULL
  
  if(is.null(ec50_lower_limit)){
    ec50_lower_limit <- min(unique(df$log_conc)[
      which(is.finite(unique(df$log_conc)))])
  }
  if(is.null(ec50_upper_limit)){
    ec50_upper_limit <- max(unique(df$log_conc)[
      which(is.finite(unique(df$log_conc)))])
  }
  
  h1_df <- df %>%
    group_by(representative, clustername, nObs) %>%
    mutate(temp_i = dense_rank(temperature)) %>%
    do({
      unique_temp <- unique(.$temperature)
      len_temp <- length(unique_temp)
      h1_model <- .fitEvalH1(df_fil = .,
                            unique_temp = unique_temp, 
                            len_temp = len_temp,
                            optim_fun = optim_fun, 
                            optim_fun_2 = optim_fun_2,
                            gr_fun = gr_fun,
                            gr_fun_2 = gr_fun_2,
                            slopEC50 = slopEC50, 
                            maxit = maxit)
      .eval_optim_result(h1_model, hypothesis = "H1",
                        data = ., len_temp = len_temp,
                        slopEC50 = slopEC50)
    }) %>%
    group_by(representative, clustername) %>%
    ungroup()
  
  return(h1_df)
}

#' @importFrom methods is
#' @importFrom stats fft
#' @importFrom MASS lda
.eval_optim_result <- function(optim_result, hypothesis = "H1",
                              data, len_temp = NULL,
                              slopEC50 = TRUE){
  # evaluate optimization results for H0 or H1 models 
  spec <- NULL
  if(!is(optim_result, "try-error")){
    if(hypothesis == "H1"){
      pEC50 <-  -optim_result$par[1]
      if(!slopEC50){
        slope <-  optim_result$par[2]
      }else{
        pEC50_slope <- optim_result$par[2]
        slope <- optim_result$par[3]
      }
      rss <- optim_result$value
      nCoeffs <- length(optim_result$par)
      fitStats <- 
        data.frame(rss = rss, nCoeffs = nCoeffs,
                   pEC50 = pEC50, slope = slope)
      if(slopEC50){
        fitStats$pEC50_slope <- pEC50_slope
      }
      if(!is.null(len_temp)){
        if(!slopEC50){
            alpha <- optim_result$par[(4 + len_temp):(3 + len_temp*2)]
        }else{
            alpha <- optim_result$par[(5 + len_temp):(4 + len_temp*2)]
        }
        # alpha_fft_df <- data.frame(freq = seq_len(length(alpha)),
        #                            strength = Mod(fft(alpha)))
        # alpha_fft_fit <- lm(strength ~ poly(freq, 2), 
        #                     data = alpha_fft_df[-1,])
        # freq <- Mod(fft(c(alpha, rep(0, 12-length(alpha)))))
        # alpha_fft_df <- tibble(freq) %>% 
        #     mutate(spec = paste0("T", 1:n())) %>% 
        #     arrange(spec) %>% 
        #     spread(spec, freq)
        # lda_pred <- predict(lda_model, alpha_fft_df)
        # if(lda_pred$class == "yes"){
        #     fitStats$detected_effect <- "carry-over"
        # }else 
        if(alpha[1] > (max(alpha[-1])/3)){
            fitStats$detected_effect <- "expression/solubility"
        }else{
            fitStats$detected_effect <- "stability"
        }
      }
      names(fitStats) <- paste0(names(fitStats), hypothesis)
      return(fitStats)
    }else{
      rss = optim_result$value
      nCoeffs = length(optim_result$par)
      fitStats <- data.frame(rss = rss,  nCoeffs = nCoeffs)
      names(fitStats) <- paste0(names(fitStats), hypothesis)
      return(fitStats)
    }
  }else{
    fitStats <- 
      data.frame(rss = NA,nCoeffs = NA,
                 pEC50 = NA, slope = NA)
    names(fitStats) <- paste0(names(fitStats), hypothesis)
    return(fitStats)
  }
}

#' Compute F statistic from H1 and H0 model characteristics
#'
#' @param h0_df data frame with H0 model characteristics for each
#' protein
#' @param h1_df data frame with H1 model characteristics for each
#' protein
#' 
#' @return data frame with H0 and H1 model characteristics for each
#' protein and respectively computed F statistics
#'
#'
#' @examples
#' data("simulated_cell_extract_df")
#' temp_df <- simulated_cell_extract_df %>% 
#'   filter(clustername %in% paste0("protein", 1:20)) %>% 
#'   group_by(representative) %>% 
#'   mutate(nObs = n()) %>% 
#'   ungroup 
#'   
#' h0_df <- fitH0Model(temp_df)
#' h1_df <- fitH1Model(temp_df)
#'   
#' computeFstat(h0_df, h1_df)
#' 
#' @export
#' 
#' @import dplyr
computeFstat <- function(h0_df, h1_df){
  
  nCoeffsH1 <- nCoeffsH0 <- nObs <- rssH0 <- 
    rssH1 <- df2 <- df1 <- NULL
  
  sum_df <- 
    left_join(h0_df, h1_df, by = c("representative",
                                   "clustername", "nObs")) %>%
    ungroup() %>%
    mutate(df1 = nCoeffsH1 - nCoeffsH0, df2 = nObs - nCoeffsH1) %>%
    mutate(F_statistic = ((rssH0 - rssH1) / rssH1) * (df2/df1))
  
  return(sum_df)
}

#' Fit H0 and H1 model to 2D thermal profiles of proteins
#' and compute F statistic
#'
#' @param df tidy data_frame retrieved after import of a 2D-TPP 
#' dataset, potential filtering and addition of a column "nObs"
#' containing the number of observations per protein
#' @param maxit maximal number of iterations the optimization
#' should be given, default is set to 500
#' @param optim_fun_h0 optimization function that should be used
#' for fitting the H0 model
#' @param optim_fun_h1 optimization function that should be used
#' for fitting the H1 model
#' @param optim_fun_h1_2 optional additional optimization function 
#' that will be run with paramters retrieved from optim_fun_h1 and 
#' should be used for fitting the H1 model with the trimmed sum
#' model, default is NULL
#' @param gr_fun_h0 optional gradient function for optim_fun_h0,
#' default is NULL
#' @param gr_fun_h1 optional gradient function for optim_fun_h1,
#' default is NULL
#' @param gr_fun_h1_2 optional gradient function for optim_fun_h1_2,
#' default is NULL
#' @param ec50_lower_limit lower limit of ec50 parameter
#' @param ec50_upper_limit lower limit of ec50 parameter
#' @param slopEC50 logical flag indicating whether the h1 model is
#' fitted with a linear model describing the shift od the pEC50 over 
#' temperatures
#' 
#' @return data frame with H0 and H1 model characteristics for each
#' protein and respectively computed F statistics
#' 
#' @examples 
#' data("simulated_cell_extract_df")
#' temp_df <- simulated_cell_extract_df %>% 
#'   group_by(representative) %>% 
#'   mutate(nObs = n()) %>% 
#'   ungroup 
#' fitAndEvalDataset(temp_df)  
#' 
#' @export
fitAndEvalDataset <- function(df, maxit = 500,
                              optim_fun_h0 = .min_RSS_h0,
                              optim_fun_h1 = .min_RSS_h1_slope_pEC50,
                              optim_fun_h1_2 = NULL,
                              gr_fun_h0 = NULL,
                              gr_fun_h1 = NULL,
                              gr_fun_h1_2 = NULL,
                              ec50_lower_limit = NULL,
                              ec50_upper_limit = NULL,
                              slopEC50 = TRUE){
  
  h0_df <- fitH0Model(df = df,
                      maxit = maxit,
                      optim_fun = optim_fun_h0,
                      gr_fun = gr_fun_h0)
  
  h1_df <- fitH1Model(df = df,
                      maxit = maxit,
                      optim_fun = optim_fun_h1,
                      optim_fun_2 = optim_fun_h1_2,
                      gr_fun = gr_fun_h1,
                      gr_fun_2 = gr_fun_h1_2,
                      ec50_lower_limit = ec50_lower_limit,
                      ec50_upper_limit = ec50_upper_limit,
                      slopEC50 = slopEC50)
  
  sum_df <- computeFstat(h0_df, h1_df)
  
  return(sum_df)
}


.minObsFilter <- function(df, minObs = 20){
  # Filter data frame for a minimal number of observations
  # per protein
  representative <- clustername <- rel_value <- 
    nObs <- NULL
  
  df_fil <- df %>%
    group_by(representative, clustername) %>%
    mutate(nObs = n()) %>%
    filter(nObs >= minObs) %>%
    ungroup()
  
  return(df_fil)
}

.independentFilter <- function(df, fcThres = 1.5){
  # Filter data frame independently based on maximal 
  # fold change per protein
  representative <- clustername <- rel_value <- NULL
  
  df_fil <- df %>%
    group_by(representative, clustername) %>%
    filter(any(rel_value > fcThres) | any(rel_value < 1/fcThres)) %>%
    ungroup
  
  return(df_fil)
}


.getEC50Limits <- function(df){
  log_conc <- NULL
  
  ec50_lower_limit <- min(unique(df$log_conc)[
    which(is.finite(unique(df$log_conc)))])
  ec50_upper_limit <- max(unique(df$log_conc)[
    which(is.finite(unique(df$log_conc)))])
  
  return(c(ec50_lower_limit, 
           ec50_upper_limit))
}

#' Compete H0 and H1 models per protein and obtain F statistic
#' 
#' @param df tidy data frame retrieved after import of a 2D-TPP 
#' dataset, potential filtering and addition of a column "nObs"
#' containing the number of observations per protein
#' @param fcThres numeric value of minimal fold change 
#' (or inverse fold change) a protein has to show to be kept 
#' upon independent filtering
#' @param independentFiltering boolean flag indicating whether
#' independent filtering should be performed based on minimal
#' fold changes per protein profile
#' @param minObs numeric value of minimal number of observations
#' that should be required per protein
#' @param maxit maximal number of iterations the optimization
#' should be given, default is set to 500
#' @param optim_fun_h0 optimization function that should be used
#' for fitting the H0 model
#' @param optim_fun_h1 optimization function that should be used
#' for fitting the H1 model
#' @param optim_fun_h1_2 optional additional optimization function 
#' that will be run with paramters retrieved from optim_fun_h1 and 
#' should be used for fitting the H1 model with the trimmed sum
#' model, default is NULL
#' @param gr_fun_h0 optional gradient function for optim_fun_h0,
#' default is NULL
#' @param gr_fun_h1 optional gradient function for optim_fun_h1,
#' default is NULL
#' @param gr_fun_h1_2 optional gradient function for optim_fun_h1_2,
#' default is NULL
#' 
#' @return data frame summarising the fit characteristics of H0 and
#' H1 models and therof resulting computed F statistics per protein
#' 
#' @examples 
#' data("simulated_cell_extract_df")
#' temp_df <- simulated_cell_extract_df %>% 
#'   filter(clustername %in% paste0("protein", 1:10)) %>% 
#'   group_by(representative) %>% 
#'   mutate(nObs = n()) %>% 
#'   ungroup 
#' competeModels(temp_df)  
#' 
#' @export
competeModels <- function(df, fcThres = 1.5,
                          independentFiltering = FALSE,
                          minObs = 20,
                          optim_fun_h0 = .min_RSS_h0,
                          optim_fun_h1 = .min_RSS_h1_slope_pEC50,
                          optim_fun_h1_2 = NULL,
                          gr_fun_h0 = NULL,
                          gr_fun_h1 = NULL,
                          gr_fun_h1_2 = NULL,
                          maxit = 750){
  
  .checkDfColumns(df)
  
  if(identical(optim_fun_h1, .min_RSS_h1_slope_pEC50)){
    slopEC50 = TRUE
  }else{
    slopEC50 = FALSE
  }
  
  ec50_limits <- .getEC50Limits(df)
  
  df_fil <- .minObsFilter(df, minObs = minObs)
  
  if(independentFiltering){
    message(paste("Independent Filtering: removing proteins without", 
            "any values crossing the set threshold."))
    df_fil <- .independentFilter(df_fil, fcThres = fcThres) 
  }

  sum_df <- fitAndEvalDataset(
    df_fil,
    maxit = maxit,
    optim_fun_h0 = optim_fun_h0,
    optim_fun_h1 = optim_fun_h1,
    optim_fun_h1_2 = optim_fun_h1_2,
    ec50_lower_limit = ec50_limits[1],
    ec50_upper_limit = ec50_limits[2],
    slopEC50 = slopEC50)
  
  return(sum_df)
}

#' Compute F statistics from paramter data frame
#' 
#' @param params_df data frame listing all null and alternative
#' model parameters as obtained by 'getModelParamsDf'
#' 
#' @return data frame of all proteins and computed F statistics
#' and parameters that were used for the computation
#' 
#' @examples
#' data("simulated_cell_extract_df")
#' params_df <- getModelParamsDf(simulated_cell_extract_df)
#' computeFStatFromParams(params_df)
#'  
#' @export
#' 
#' @import dplyr
computeFStatFromParams <- function(params_df){
    nCoeffsH0 <- nCoeffsH1 <- nObs <- rssH0 <- rssH1 <- df1 <- 
        df2 <- representative <- clustername <- min_qupm <- 
        max_qupm <- pEC50H1 <- slopeH1 <- pEC50_slopeH1 <- 
        detected_effectH1 <- F_statistic <- NULL
    
    fstat_df <- params_df %>%
        mutate(df1 = nCoeffsH1 - nCoeffsH0, df2 = nObs - nCoeffsH1) %>%
        mutate(F_statistic = ((rssH0 - rssH1) / rssH1) * (df2/df1)) %>% 
        dplyr::select(representative, clustername, nObs, 
                      min_qupm, max_qupm,
                      nCoeffsH0, nCoeffsH1, rssH0, rssH1,
                      pEC50H1, slopeH1, pEC50_slopeH1, 
                      detected_effectH1,
                      df1, df2, F_statistic)
    return(fstat_df)
}

#' Get pEC50 for a protein of interest at a specific temperatures
#' (optimally the melting point of the protein)
#' 
#' @param fstat_df data frame as obtained after calling 
#' \code{getModelParamsDf}, containing fitted null and 
#' alternative model parameters for each protein
#' @param protein character string referring to the protein 
#' of interest
#' @param temperaturePEC50 temperature (numeric) at which pEC50
#' should be inferred
#' 
#' @return numeric value specifying the pEC50 for the 
#' indicated protein and temperature
#' 
#' @examples 
#' data("simulated_cell_extract_df")
#' 
#' model_params_df <- getModelParamsDf(
#'    df = filter(simulated_cell_extract_df, 
#'            clustername == "tp1"))
#' 
#' getPEC504Temperature(
#'     fstat_df = model_params_df, 
#'     protein = "tp1", 
#'     temperaturePEC50 = 60)
#' @import dplyr
#' @export
getPEC504Temperature <- function(fstat_df, protein, 
                                 temperaturePEC50 = 60){
    clustername <- NULL
    fstat_fil <- filter(fstat_df, clustername == protein)
    pEC50 <- fstat_fil$pEC50H1 - 
        (temperaturePEC50 *fstat_fil$pEC50_slopeH1)
    return(pEC50)
}

## Tests for getModelParamsDf()
## library(TPP2D); library(testthat); source("setup-2dtpp-dataset.R"); source("test-getModelParamsDf.R")
context("getModelParamsDf")

params_df <- getModelParamsDf(.minObsFilter(simulated_cell_extract_df) %>% 
                                  filter(representative %in% 1:3))

test_that("getModelParamsDf: rss behave as expected", {
    expect_true(all(params_df$rssH0 - params_df$rssH1 > 0))
})

test_that("getModelParamsDf: length of residuals corresponds to obs.", {
    expect_true(all(params_df$nObs == sapply(params_df$residualsH0, length)))
    expect_true(all(params_df$nObs == sapply(params_df$residualsH1, length)))
})

## Tests for recomputeSignalFromRatios()
## library(TPP2D); library(testthat); source("setup-2dtpp-dataset.R"); source("test-fits.R")
context("fit models")

test_that("fitH0Model works as expected", {
  temp_df <- simulated_cell_extract_df %>% 
    filter(clustername == "tp1") %>% 
    mutate(nObs = n()) 
  fit_df <- fitH0Model(temp_df)
  expect_identical(round(fit_df$rssH0, 2), 4.83)
})

test_that("fitH1Model works as expected", {
  temp_df <- simulated_cell_extract_df %>% 
    filter(clustername == "tp1") %>% 
    mutate(nObs = n()) 
  fit_df <- fitH1Model(temp_df)
  expect_identical(round(fit_df$rssH1, 1), 0.1)
})

test_that("computeFstat works as expected", {
  temp_df <- simulated_cell_extract_df %>% 
    filter(clustername == "tp1") %>% 
    mutate(nObs = n()) 
  h0_df <- fitH0Model(temp_df)
  h1_df <- fitH1Model(temp_df)
  f_df <- computeFstat(h0_df, h1_df)
  expect_identical(round(f_df$F_statistic, -2), 100)
})

test_that("fitAndEvalDataset works as expected", {
  temp_df <- simulated_cell_extract_df %>% 
    filter(clustername == "tp1") %>% 
    mutate(nObs = n()) 
  f_df <- fitAndEvalDataset(temp_df)
  expect_identical(round(f_df$F_statistic, -2), 100)
})

.TPP_importFct_CheckDataFormat <- function (files, dataframes, expNames){
    # internal function copied from TPP package to avoid 
    # import of non-exported package functions
    . <- NULL
    isDF <- !is.null(dataframes)
    isF <- !is.null(files)
    isBoth <- isDF & isF
    isNone <- !(isDF | isF)
    if (isBoth) {
        stop("Data import function received a",
             " filename AND a dataframe object. \n",
             "Please specify only one.")
    }
    else if (isNone) {
        stop("Data import function requires a", 
             " filename or a dataframe object. \n",
             "Please specify one.")
    }
    if (isDF) {
        isClassList <- is.list(dataframes) && !is.data.frame(dataframes)
        isClassDF <- is.data.frame(dataframes)
        if (isClassList) {
            classesInList <- dataframes %>% 
            vapply(. %>% inherits(., "data.frame"), TRUE)
            if (!all(classesInList)) {
                stop(paste("Argument 'dataframes' contains", 
                           "elements that are not of type", 
                           "'data.frame' at the following positions: "), 
                     which(!classesInList) %>% paste(collapse = ", "), ".")
            }
          }
          else if (isClassDF) {
              dataframes <- list(dataframes)
              names(dataframes) <- expNames
          }
          else {
              stop("Argument 'dataframes' must be either an object of class \n
                   'data.frame', or a list of such objects.")
          }
    }
    if (isF) {
        files <- as.character(files)
        names(files) <- expNames
    }
    return(list(files = files, dataframes = dataframes))
}

#' @importFrom utils read.delim
#' @importFrom RCurl url.exists
.TPP_importFct_readFiles <- function (files, naStrs){
  # internal function copied from TPP package to avoid 
  # import of non-exported package functions
  expNames <- names(files)
  data <- vector("list", length(files))
  names(data) <- expNames
  for (expName in expNames) {
    fTmp <- files[[expName]]
    if (file.exists(fTmp) || url.exists(fTmp)) {
      data[[expName]] <- read.delim(fTmp, as.is = TRUE, 
                                    na.strings = naStrs, quote = "")
    }
    else {
      stop("File ", fTmp, " could not be found.")
    }
  }
  return(data)
}

#' @import dplyr
.TPP_importFct_removeDuplicates <- function(inDF, refColName, 
                                           nonNAColNames, qualColName){
  # internal function copied from TPP package to avoid 
  # import of non-exported package functions
  message("Removing duplicate identifiers using quality column '", 
          qualColName, "'...")
  nonUniques <- unique(as_tibble(inDF)[duplicated(inDF[[refColName]]), 
                           refColName])
  retDF <- subset(inDF, !(get(refColName) %in% nonUniques))
  if(nrow(nonUniques)){
      for (nU in nonUniques) {
          tmpDF <- subset(inDF, get(refColName) == nU)
          nonNArows <- NULL
          for (r in seq_len(nrow(tmpDF))) {
              if (any(!is.na(tmpDF[r, nonNAColNames]))) {
                  nonNArows <- c(nonNArows, r)
              }
          }
          if (length(nonNArows) > 1) {
              if (is.null(qualColName)) {
                  useRow <- 1
              }
              else {
                  qualVals <- tmpDF[nonNArows, qualColName]
                  useRow <- match(max(qualVals), qualVals)
              }
          }
          else {
              useRow <- nonNArows[1]
          }
          retDF <- rbind(retDF, tmpDF[useRow, ])
      }
  }
  message(nrow(retDF), " out of ", nrow(inDF), 
          " rows kept for further analysis.")
  return(retDF)
}

.TPP_replaceZeros <- function(x){
  # internal function copied from TPP package to avoid 
  # import of non-exported package functions
  x[which(x == 0)] <- NA
  return(x)
}

.TPP_importFct_rmZeroSias <- function(data.list, 
                                      intensityStr){
  # internal function copied from TPP package to avoid 
  # import of non-exported package functions
  out <- lapply(names(data.list), function(l.name) {
    datTmp <- data.list[[l.name]]
    colsTmp <- colnames(datTmp)
    intensity.cols <- grep(intensityStr, colsTmp, value = TRUE)
    intensity.df <- subset(datTmp, select = intensity.cols) %>% 
      mutate_all(as.character) %>% mutate_all(as.numeric)
    new.intensity.df <- intensity.df %>% mutate_all(.TPP_replaceZeros)
    datTmp[, intensity.cols] <- new.intensity.df
    return(datTmp)
  })
  names(out) <- names(data.list)
  return(out)
}

.TPP_importFct_checkExperimentCol <- function(expCol){
  # internal function copied from TPP package to avoid 
  # import of non-exported package functions
  if (is.null(expCol)) {
    m <- paste("Config table needs an 'Experiment'", 
               "column with unique experiment IDs.")
    stop(m, "\n")
  }
  oldExpNames <- expCol
  newExpNames <- gsub("([^[:alnum:]])", "_", expCol)
  iChanged <- oldExpNames != newExpNames
  if (any(iChanged)) {
    m1 <- paste("Replaced non-alphanumeric characters", 
                "in the 'Experiment' column entries:")
    m2 <- paste("'", paste(oldExpNames[iChanged], collapse = "', '"), 
                "'\nby\n'", paste(newExpNames[iChanged], collapse = "', '"), 
                sep = "")
    message(m1, "\n", m2, "\n")
  }
  return(newExpNames)
}

.TPP_importFct_checkComparisons <- function(confgTable){
  # internal function copied from TPP package to avoid 
  # import of non-exported package functions
  expConds <- confgTable$Condition
  expNames <- confgTable$Experiment
  compCols <- grep("Comparison", colnames(confgTable), ignore.case = TRUE, 
                   value = TRUE)
  compChars <- apply(confgTable[compCols], 2, function(x) {
    length(grep("[[:alnum:]]", x, value = TRUE))
  })
  comp_unequal_two <- compChars != 2
  if (any(comp_unequal_two)) {
    warning(paste("\nThe following comparison columns could not be evaluated", 
                  "because they did not contain exactly two entries:\n\t\t"), 
            paste(compCols[comp_unequal_two], collapse = ",\n\t\t"))
  }
  validCompCols <- compCols[!comp_unequal_two]
  allCompStrs <- c()
  if (length(validCompCols)) {
    message("Comparisons will be performed between the following experiments:")
    for (colName in validCompCols) {
      current_compEntries <- confgTable[[colName]]
      current_compRows <- grep("[[:alnum:]]", current_compEntries)
      current_compExps <- expNames[current_compRows]
      compRef <- current_compExps[1]
      compTreatm <- current_compExps[2]
      if ("Condition" %in% names(confgTable)) {
        current_compConds <- expConds[current_compRows]
        if ("Vehicle" %in% current_compConds && "Treatment" %in% 
            current_compConds) {
          compRef <- current_compExps[current_compConds == 
                                        "Vehicle"]
          compTreatm <- current_compExps[current_compConds == 
                                           "Treatment"]
        }
      }
      compStr <- paste(compTreatm, "_vs_", compRef, sep = "")
      names(compStr) <- colName
      message(compStr)
      allCompStrs <- c(allCompStrs, compStr)
    }
    message("\n")
  }
  return(allCompStrs)
}

#' @importFrom stringr str_to_title
.TPP_importFct_checkConditions <- function(condInfo, 
                                          expectedLength){
  # internal function copied from TPP package to avoid 
  # import of non-exported package functions
  flagGenerateConds <- FALSE
  if (is.null(condInfo)) {
    message("No information about experimental conditions given.", 
            "Assigning NA instead.\n",
            "Reminder: recognition of Vehicle and Treatment groups", 
            "during pairwise \n",
            "comparisons is only possible when they are specified ",
            "in the config table.\n")
    condInfo <- rep(NA_character_, expectedLength)
  }
  else {
    condInfo <- as.character(condInfo) %>% 
      stringr::str_to_title()
    condLevels <- unique(condInfo)
    invalidLevels = 
      setdiff(condLevels, c("Treatment", "Vehicle"))
    if (length(invalidLevels) > 0) {
      stop("The entry '", invalidLevels, 
           paste("' in the condition column is invalid.", 
                 "Only the values 'Treatment' and", 
                 "'Vehicle' are allowed. Please correct", 
                 "this and start again."))
    }
  }
  return(condInfo)
}

.TPP_checkFunctionArgs <- 
  function(functionCall, expectedArguments){
  # internal function copied from TPP package to avoid 
  # import of non-exported package functions
  myArgs <- names(functionCall)
  lapply(expectedArguments, function(arg) {
    if (!arg %in% myArgs) {
      stop("Error in ", paste(functionCall)[1], 
           ": argument '", 
           arg, "' is missing, with no default", 
           call. = FALSE)
    }
  })
}

.TPP_nonLabelColumns <- function(){
  # internal function copied from TPP package to avoid 
  # import of non-exported package functions
  out <- data.frame(
    column = c("Experiment", "Experiment",
               "Experiment", "Path", "Path", 
               "Path", "Condition", "Replicate", 
               "Compound", "Temperature", "RefCol"), 
    type = c("TR", "CCR", "2D", "TR", "CCR", "2D", 
             "TR", "TR", "2D", "2D", "2D"), 
    obligatory = c(TRUE, TRUE, TRUE, FALSE, FALSE, 
                   FALSE, TRUE, FALSE, TRUE, TRUE, TRUE), 
    exclusive = c(FALSE, FALSE, FALSE, FALSE, FALSE, 
                  FALSE, TRUE, TRUE, TRUE, TRUE, TRUE))
  return(out)
}

.TPP_detectLabelColumnsInConfigTable <- 
  function(allColumns){
  # internal function copied from TPP package to avoid 
  # import of non-exported package functions
  .TPP_checkFunctionArgs(match.call(), c("allColumns"))
  noLabelCols <- .TPP_nonLabelColumns()$column %>% 
    as.character %>% 
    unique
  compCols <- grep("comparison", allColumns, value = TRUE, 
                   ignore.case = TRUE)
  noLabelCols <- c(noLabelCols, compCols)
  labelCols <- setdiff(allColumns, noLabelCols)
  return(labelCols)
}

.TPP_importCheckTemperatures <- function(temp){
  # internal function copied from TPP package to avoid 
  # import of non-exported package functions
  tempMatrix <- as.matrix(temp)
  rownames(tempMatrix) <- NULL
  naRows <- apply(is.na(tempMatrix), 1, all)
  if (any(naRows)) {
    stop("Row(s) ", paste(which(naRows), collapse = ", "), 
         " in the configuration table contain", 
         " only missing temperature values.")
  }
  return(tempMatrix)
}

#' @importFrom openxlsx read.xlsx
#' @importFrom utils read.table
.TPP_importFct_readConfigTable <- function(cfg){
  # internal function copied from TPP package to avoid 
  # import of non-exported package functions
  if (is.character(cfg)) {
    if (file.exists(cfg)) {
      strChunks <- strsplit(cfg, "\\.")[[1]]
      fileExtension <- strChunks[length(strChunks)]
      if (fileExtension == "txt") {
        tab <- read.table(
          file = cfg, header = TRUE, 
          check.names = FALSE, stringsAsFactors = FALSE, 
          sep = "\t")
      }
      else if (fileExtension == "csv") {
        tab <- read.table(
          file = cfg, header = TRUE, 
          check.names = FALSE, stringsAsFactors = FALSE, 
          sep = ",")
      }
      else if (fileExtension == "xlsx") {
        tab <- openxlsx::read.xlsx(cfg)
      }
      else {
        stop("Error during data import: ", cfg, 
             " does not belong to a valid configuration file.")
      }
    }
    else {
      stop("Error during data import: ", cfg, 
           " does not belong to a valid configuration file.")
    }
    cfg <- tab
  }
  return(cfg)
}

#' Import and chech configuration table
#' 
#' @param infoTable character string of a file path to
#' a config table (excel,txt or csv file) or data frame
#' containing a config table
#' @param type charater string indicating dataset type
#' default is 2D
#' 
#' @return data frame with config table
#' 
#' @examples 
#' data("config_tab")
#' TPP_importCheckConfigTable(config_tab, type = "2D")
#' @export
TPP_importCheckConfigTable <- function(infoTable, type = "2D"){
  .TPP_checkFunctionArgs(match.call(), c("infoTable", "type"))
  Experiment <- Path <- Compound <- NULL
  isValidDF <- FALSE
  if (is.data.frame(infoTable)) {
    if ((ncol(infoTable) > 1) & 
        ("Experiment" %in% colnames(infoTable))) {
      isValidDF <- TRUE
    }
  }
  if (!is.character(infoTable) & !isValidDF) {
    stop("'infoTable' must either be a data frame", 
         " with an 'Experiment' column \n",
         "and at least one isobaric label column,", 
         "or a filename pointing at a \n",
         "table that fulfills the same criteria")
  }
  isValidType <- type %in% c("2D")
  if (!isValidType) {
    stop("'type' must have this value: '2D'")
  }
  infoTable <- .TPP_importFct_readConfigTable(cfg = infoTable)
  infoTable$Experiment <- 
    .TPP_importFct_checkExperimentCol(infoTable$Experiment)
  infoTable <- subset(infoTable, Experiment != "")
  givenPaths <- NULL
  if (any("Path" %in% colnames(infoTable))) {
    if (all(infoTable$Path == "") || all(is.na(infoTable$Path))) {
      message("Removing empty 'Path' column from config table")
      infoTable <- infoTable %>% select(-Path)
    }
    else {
      givenPaths <- infoTable$Path
    }
  }
  compStrs <- NA
  infoTable$Condition <- NULL
  allCols <- colnames(infoTable)
  labelCols <- .TPP_detectLabelColumnsInConfigTable(allColumns = allCols)
  labelValues <- infoTable[, labelCols]
  labelValuesNum <- suppressWarnings(labelValues %>% apply(2, 
                                                           as.numeric))
  if (is.matrix(labelValuesNum)) {
    isInvalid <- labelValuesNum %>% apply(2, is.na) %>% apply(2, 
                                                              all)
  }
  else if (is.vector(labelValuesNum)) {
    isInvalid <- is.na(labelValuesNum)
  }
  invalidLabels <- labelCols[isInvalid]
  infoTable[, invalidLabels] <- NULL
  labelColsNew <- labelCols[!isInvalid]
  labelStr <- paste(labelColsNew, collapse = ", ")
  message("The following valid label columns were detected:\n", 
          labelStr, ".")
  if (type == "2D") {
    temperatures <- infoTable$Temperature
    if (is.null(temperatures) | length(temperatures) < 2) {
      m1 <- paste("Insufficient temperatures (<2)", 
                  "specified in config file.")
      m2 <- paste("Does your configuration table", 
                  "have the correct column names?")
      stop(m1, "\n", m2)
    }
    else if (length(which(!infoTable$RefCol %in% labelColsNew)) != 
             0) {
      stop(paste("Labels in reference column not found", 
                 "in any of the label columns."))
    }
    hasCompoundCol <- any(allCols == "Compound")
    if (!hasCompoundCol) {
      m <- paste("Config table of a 2D-TPP experiment", 
                 "needs a 'Compound' column.")
      stop(m, "\n")
    }
    else {
      infoTable <- infoTable %>% 
        mutate(Compound = 
                 gsub("([^[:alnum:]])", "_", Compound))
    }
    out <- infoTable
  }
  else {
    temperatures <- subset(infoTable, select = labelColsNew)
    tempMatrix <- .TPP_importCheckTemperatures(temp = temperatures)
    infoList <- list(
      expNames = as.character(infoTable$Experiment), 
      expCond = infoTable$Condition, files = givenPaths, 
      compStrs = compStrs, labels = labelColsNew, 
      tempMatrix = tempMatrix)
    out <- infoList
  }
  return(out)
}

#' Import 2D-TPP dataset main function
#' 
#' @param configTable character string of a file path to a config table
#' @param data possible list of datasets from different MS runs 
#' corresponding to a 2D-TPP dataset, circumvents loading datasets 
#' referencend in config table, default is NULL
#' @param idVar character string indicating which data column provides the 
#' unique identifiers for each protein.
#' @param intensityStr character string indicating which columns contain 
#' raw intensities measurements
#' @param fcStr character string indicating which columns contain the actual 
#' fold change values. Those column names containing the suffix \code{fcStr} 
#' will be regarded as containing fold change values.
#' @param naStrs character vector indicating missing values in the data table. 
#' When reading data from file, this value will be passed on to the argument 
#' \code{na.strings} in function \code{read.delim}.
#' @param addCol character string indicating additional column to import
#' @param nonZeroCols column like default qssm that should be imported and
#' requested to be non-zero in analyzed data
#' @param qualColName character string indicating which column can be used for 
#' additional quality criteria when deciding between different non-unique 
#' protein identifiers.
#' 
#' @return list of data frames containing different
#' datasets
#' 
#' @examples 
#' data("config_tab")
#' data("raw_dat_list")
#' dataList <- import2dMain(configTable = config_tab,
#'                          data = raw_dat_list,
#'                          idVar = "protein_id",
#'                          fcStr = "rel_fc_",
#'                          addCol = "gene_name",
#'                          naStrs = NA,
#'                          intensityStr = "signal_sum_",
#'                          nonZeroCols = "qusm",
#'                          qualColName = "qupm")
#' @export
import2dMain <- function(configTable, data, idVar, fcStr,
                         addCol, naStrs, intensityStr,
                         qualColName, nonZeroCols){
  
  # internal import main function, adapted from TPP package
  files <- configTable$Path
  if (!is.null(files)) {
    if (any(files == "")) {
      files <- NULL
    }
  }
  Experiment <- Compound <- Temperature <- RefCol <- NULL
  expNames <- configTable$Experiment
  argList <- .TPP_importFct_CheckDataFormat(dataframes = data, 
                                           files = files,
                                           expNames = expNames)
  data <- argList[["dataframes"]]
  files <- argList[["files"]]
  if (!is.null(files)) {
    files2 <- files[!duplicated(names(files))]
    data <- .TPP_importFct_readFiles(files = files2, 
                                    naStrs = naStrs)
  }
  iVec <- seq_len(nrow(configTable))
  dataList <- lapply(iVec, function(iTmp) {
    rowTmp <- configTable[iTmp, ]
    expTmp <- rowTmp$Experiment
    message("Importing 2D-TPP dataset: ", expTmp)
    tTmp <- rowTmp$Temperature
    dataTmp <- data[[expTmp]]
    noFCCols <- c("Compound", "Experiment", "Temperature",
                  "RefCol", "Path", "Condition")
    allCols <- colnames(rowTmp)
    labelCols <- setdiff(allCols, noFCCols)
    labelValues <- suppressWarnings(rowTmp[, labelCols] %>%
                                      as.numeric)
    labelColsNum <- labelCols[!is.na(labelValues)]
    signalCols <- paste(intensityStr, labelColsNum, sep = "")
    relevant.cols <- c(idVar, qualColName, nonZeroCols, addCol,
                       signalCols) %>% unique
    if (!is.null(fcStr)) {
      fcCols <- paste(fcStr, labelColsNum, sep = "")
      relevant.cols <- c(relevant.cols, fcCols)
      dataCols <- fcCols
    }
    else {
      dataCols <- signalCols
    }
    if (!all(relevant.cols %in% colnames(dataTmp))) {
      notFound <- paste(setdiff(relevant.cols, colnames(dataTmp)),
                        collapse = "', '")
      stop("The following columns could not be found: '",
           notFound, paste("'. Please check the suffices and the", 
                           "additional column names you have specified."))
    }
    dataFiltered <- .TPP_importFct_removeDuplicates(
      inDF = dataTmp,refColName = idVar, 
      nonNAColNames = dataCols, 
      qualColName = qualColName[1])
    idsTmp <- as.character(dataFiltered[, idVar])
    idsAnnotated <- paste(expTmp, tTmp, idsTmp, sep = "_")
    dataFinal <- dataFiltered %>% subset(select = relevant.cols) %>%
      mutate(temperature = tTmp, experiment = expTmp, unique_ID = idsAnnotated)
    return(dataFinal)
  })
  newNames <- vapply(seq(nrow(configTable)), function(iTmp) {
    rowTmp <- configTable[iTmp, ]
    tTmp <- rowTmp$Temperature
    expTmp <- rowTmp$Experiment
    newName <- paste(expTmp, tTmp, sep = "_")
    return(newName)
  }, "")
  names(dataList) <- newNames
  out <- .TPP_importFct_rmZeroSias(data.list = dataList,
                                   intensityStr = intensityStr)
  return(out)
}

#' Tranform configuration table from wide to long
#' 
#' @param configWide data frame containing a config table
#' @return data frame containing config table in long format 
#' 
#' @importFrom tidyr gather
#' @examples 
#' data("config_tab")
#' configWide2Long(configWide = config_tab)
#' 
#' @export
configWide2Long <- function(configWide){
 Path <- label <- conc <- Compound <- Experiment <- 
    Temperature <- RefCol <- NULL
  
  if(any(grepl("Path", colnames(configWide)))){
    configLong <- configWide %>%
      dplyr::select(-Path) %>%
      gather(label, conc, -Compound, 
             -Experiment, -Temperature, -RefCol) %>%
      filter(conc != "-")
  }else{
    configLong <- configWide %>%
      gather(label, conc, -Compound, 
             -Experiment, -Temperature, -RefCol) %>%
      filter(conc != "-")
  }
}

#' Annotate imported data list using a config table
#' @param dataList list of datasets from different MS runs 
#' corresponding to a 2D-TPP dataset
#' @param geneNameVar character string of the column name that describes
#' the gene name of a given protein in the raw data files
#' @param configLong long formatted data frame of a corresponding
#' config table
#' @param intensityStr character string indicating which columns contain 
#' raw intensities measurements
#' @param fcStr character string indicating which columns contain the actual 
#' fold change values. Those column names containing the suffix \code{fcStr} 
#' will be regarded as containing fold change values.
#' 
#' @return data frame containing all data annotated
#' by information supplied in the config table
#'   
#' @importFrom tidyr spread
#' 
#' @examples 
#' data("config_tab")
#' data("raw_dat_list")
#' dataList <- import2dMain(configTable = config_tab,
#'                          data = raw_dat_list,
#'                          idVar = "protein_id",
#'                          fcStr = "rel_fc_",
#'                          addCol = "gene_name",
#'                          naStrs = NA,
#'                          intensityStr = "signal_sum_",
#'                          nonZeroCols = "qusm",
#'                          qualColName = "qupm")
#' configLong <- configWide2Long(configWide = config_tab)
#' annotateDataList(dataList = dataList,
#'                  geneNameVar = "gene_name",
#'                  configLong = configLong,
#'                  intensityStr = "signal_sum_",
#'                  fcStr = "rel_fc_")
#' @export
annotateDataList <- function(dataList, geneNameVar, configLong,
                             intensityStr, fcStr){
  channel <- signal <- Temperature <- RefCol <- label <- 
    conc <- unique_ID <- spread_var <- NULL
  
  combinedTab <- bind_rows(lapply(dataList, function(dat){
    datLong <- dat %>% as_tibble() %>%
      gather(channel, signal, matches(intensityStr), matches(fcStr)) %>%
      mutate(label = gsub(fcStr, "", gsub(intensityStr, "", channel))) %>%
      left_join(configLong %>% 
                  dplyr::select(Temperature, RefCol, label, conc),
                by = c("temperature" = "Temperature", "label")) %>%
      mutate(spread_var = 
               ifelse(grepl(fcStr, channel), "rel_value", "raw_value")) %>%
      dplyr::select(-channel, -unique_ID) %>%
      spread(spread_var, signal)
  }))
  return(combinedTab)
}

#' Filter out contaminants
#' 
#' @param dataLong long format data frame of imported dataset
#' 
#' @return data frame containing full dataset filtered to 
#' contain no contaminants
#' 
#' @examples 
#' data("simulated_cell_extract_df")
#' filterOutContaminants(simulated_cell_extract_df)
#' 
#' @export
filterOutContaminants <- function(dataLong){
  # internal function to filter out contaminants
  representative <- NULL
  filter(dataLong, !grepl("##", representative))
}

.checkRatioRef <- function(dataLong, idVar, concFactor = 1e6){
  # internal function to check that protein 
  # fold changes are computed
  # relative to the correct TMT channel
  label <- RefCol <- rel_value <- raw_value <- conc <- NULL
  
  if(!all(filter(dataLong, label == RefCol)$rel_value == 1, 
          na.rm = TRUE)){
    message("Recomputing ratios!")
    dataOut <- dataLong %>%
      group_by(.dots = c(idVar, "temperature")) %>%
      mutate(rel_value = rel_value/rel_value[label == RefCol]) %>%
      ungroup %>%
      filter(!is.na(raw_value)) %>%
      mutate(conc = as.numeric(conc)) %>%
      mutate(log_conc = log10(conc/concFactor))
    
    return(dataOut)
    
  }else{
    message("Ratios were correctly computed!")
    return(dataLong %>%
             filter(!is.na(raw_value)) %>%
             mutate(conc = as.numeric(conc)) %>%
             mutate(log_conc = log10(conc/concFactor)))
  }
}

#' @importFrom stats median
.medianNormalizeRatios <- function(dataLong){
  # internal function to perform median normalization 
  # of ratios
  rel_value <- temperature <- conc <- 
    raw_rel_value <- NULL
  
  dataOut <- dataLong %>%
    rename(raw_rel_value = rel_value) %>%
    group_by(temperature, conc) %>%
    mutate(rel_value = raw_rel_value / 
             median(raw_rel_value, na.rm = TRUE)) %>%
    ungroup()
  
  return(dataOut)
}

#' Rename columns of imported data frame
#' 
#' @param dataLong long format data frame of imported dataset
#' @param idVar character string indicating which data column provides the 
#' unique identifiers for each protein.
#' @param geneNameVar character string of the column name that describes
#' the gene name of a given protein in the raw data files
#' 
#' @return data frame containing imported data with renamed
#' columns
#' 
#' @examples 
#' data("config_tab")
#' data("raw_dat_list")
#' 
#' dataList <- import2dMain(configTable = config_tab,
#'                          data = raw_dat_list,
#'                          idVar = "protein_id",
#'                          fcStr = "rel_fc_",
#'                          addCol = "gene_name",
#'                          naStrs = NA,
#'                          intensityStr = "signal_sum_",
#'                          nonZeroCols = "qusm",
#'                          qualColName = "qupm")
#' configLong <- configWide2Long(configWide = config_tab)
#' annoDat <- annotateDataList(dataList = dataList,
#'                             geneNameVar = "gene_name",
#'                             configLong = configLong,
#'                             intensityStr = "signal_sum_",
#'                             fcStr = "rel_fc_")
#' renameColumns(annoDat, 
#'               idVar = "protein_id", 
#'               geneNameVar = "gene_name")
#' @export
renameColumns <- function(dataLong, idVar, geneNameVar){
  clustername <- representative <- NULL
  
  dplyr::rename(dataLong, "representative" = all_of(idVar), 
                "clustername" = all_of(geneNameVar)) %>%
    group_by(clustername) %>%
    mutate(representative =
             .paste_rmNA(unique(unlist(strsplit(representative, 
                                               split = "\\|"))), 
                        sep = "|")) %>%
    ungroup()
}

#' Import 2D-TPP dataset using a config table
#' 
#' @param configTable character string of a file path to a config table
#' @param data possible list of datasets from different MS runs 
#' corresponding to a 2D-TPP dataset, circumvents loading datasets 
#' referencend in config table, default is NULL
#' @param idVar character string indicating which data column provides the 
#'   unique identifiers for each protein.
#' @param intensityStr character string indicating which columns contain 
#'   raw intensities measurements
#' @param fcStr character string indicating which columns contain the actual 
#'   fold change values. Those column names containing the suffix \code{fcStr} 
#'   will be regarded as containing fold change values.
#' @param naStrs character vector indicating missing values in the data table. 
#'   When reading data from file, this value will be passed on to the argument 
#'   \code{na.strings} in function \code{read.delim}.
#' @param qualColName character string indicating which column can be used for 
#'   additional quality criteria when deciding between different non-unique 
#'   protein identifiers.
#' @param medianNormalizeFC perform median normalization (default: TRUE).
#' @param addCol character string indicating additional column to import
#' @param filterContaminants boolean variable indicating whether data 
#' should be filtered to exclude contaminants (default: TRUE).
#' @param nonZeroCols column like default qssm that should be imported and
#' requested to be non-zero in analyzed data
#' @param geneNameVar character string of the column name that describes
#' the gene name of a given protein in the raw data files
#' @param concFactor numeric value that indicates how concentrations need to 
#' be adjusted to yield total unit e.g. default mmol - 1e6
#' 
#' @return tidy data frame representing a 2D-TPP dataset
#' 
#' @examples 
#' data("config_tab")
#' data("raw_dat_list")
#' import_df <- import2dDataset(configTable = config_tab, 
#'                              data = raw_dat_list,
#'                              idVar = "protein_id",
#'                              intensityStr = "signal_sum_",
#'                              fcStr = "rel_fc_",
#'                              nonZeroCols = "qusm",
#'                              geneNameVar = "gene_name",
#'                              addCol = NULL,
#'                              qualColName = "qupm",
#'                              naStrs = c("NA", "n/d", "NaN"),
#'                              concFactor = 1e6,
#'                              medianNormalizeFC = TRUE,
#'                              filterContaminants = TRUE)
#' 
#' @export
import2dDataset <- function(configTable, data,
                            idVar = "representative",
                            intensityStr = "sumionarea_protein_",
                            fcStr = "rel_fc_protein_",
                            nonZeroCols = "qssm",
                            geneNameVar = "clustername",
                            addCol = NULL,
                            qualColName = "qupm",
                            naStrs = c("NA", "n/d", "NaN"),
                            concFactor = 1e6,
                            medianNormalizeFC = TRUE,
                            filterContaminants = TRUE){
  
  raw_value <- rel_value <- NULL
  
  configWide <- TPP_importCheckConfigTable(
    infoTable = configTable, type = "2D")
  configLong <- configWide2Long(configWide = configWide)
  
  dataList <- import2dMain(configTable = configWide,
                           data = data,
                           idVar = idVar,
                           fcStr = fcStr,
                           addCol = c(geneNameVar, addCol),
                           naStrs = naStrs,
                           intensityStr = intensityStr,
                           nonZeroCols = nonZeroCols,
                           qualColName = qualColName)
  
  dataLong <- annotateDataList(dataList = dataList,
                               geneNameVar = geneNameVar,
                               configLong = configLong,
                               intensityStr = intensityStr,
                               fcStr = fcStr)
  
  dataRatioChecked <- .checkRatioRef(dataLong, idVar = idVar,
                                     concFactor = concFactor)
  
  if(medianNormalizeFC){
    message("Median normalizing fold changes...")
    dataNorm <- .medianNormalizeRatios(dataRatioChecked)
  }else{
    dataNorm <- dataRatioChecked
  }
  
  dataOut <- renameColumns(dataNorm,
                           idVar = idVar,
                           geneNameVar = geneNameVar)
  
  if(filterContaminants){
    dataOut <- filterOutContaminants(dataOut)
  }
  
  dataOut <- filter(dataOut, !is.na(raw_value), !is.na(rel_value))
  
  return(dataOut)
}

## Internal functions for optimization of fits

.trim_sum <- function(x, na.rm = TRUE)
  # Trimmed sum (sum up all but the highest values)
{
  sum(x[-which(x == max(x, na.rm = TRUE))],
      na.rm = TRUE)
}

.min_RSS_h0 <- function(data, par, len_temp) {
  # Optimization function for fitting an intercept model to a protein's 2D
  # thermal profile by minimizing the sum of squared errors
  temp_i <- log2_value <- NULL
  
  beta_0 <- par[seq_len(len_temp)]
  
  sum(with(data, (beta_0[temp_i] - log2_value) ^ 2))
}

.min_RSS_h0_trim <- function(data, par, len_temp) {
  # Optimization function for fitting an intercept model to a protein's 2D
  # thermal profile by minimizing the trimmed sum of squared errors
  temp_i <- log2_value <- NULL
  
  beta_0 <- par[seq_len(len_temp)]
  
  .trim_sum(with(data, (beta_0[temp_i] - log2_value) ^ 2))
}

.min_RSS_h1 <- function(data, par, len_temp) {
  # Optimization function for fitting an dose-response model to a
  # protein's 2D thermal profile by minimizing the sum of squared errors
  temp_i <- log2_value <- log_conc <- NULL
  
  zeta <- par[1]
  slope <- par[2]
  beta_max <- par[3]
  beta_0 <- par[4:(len_temp + 3)]
  alpha <- par[(4 + len_temp):(3 + len_temp * 2)]
  
  sum(with(data, (
    beta_0[temp_i] + (alpha[temp_i] * beta_max) /
      (1 + exp(-slope * (log_conc - zeta))) -
      log2_value
  ) ^ 2))
}

.min_RSS_h1_slope_pEC50 <- function(data, par, len_temp) {
  # Optimization function for fitting an dose-response model to a
  # protein's 2D thermal profile by minimizing the sum of squared errors
  temp_i <- log2_value <- log_conc <- temperature <- NULL
  
  zeta <- par[1]
  zeta_slope <- par[2]
  slope <- par[3]
  beta_max <- par[4]
  beta_0 <- par[5:(len_temp + 4)]
  alpha <- par[(5 + len_temp):(4 + len_temp * 2)]
  
  sum(with(data,
           (
             beta_0[temp_i] + (alpha[temp_i] * beta_max) /
               (1 + exp(-slope * (
                 log_conc - (zeta + zeta_slope * temperature)
               ))) -
               log2_value
           )^ 2))
}


.min_RSS_h1_trim <- function(data, par, len_temp) {
  # Optimization function for fitting an dose-response model to a
  # protein's 2D thermal profile by minimizing the trimmed sum of
  # squared errors
  temp_i <- log2_value <- log_conc <- NULL
  
  zeta <- par[1]
  slope <- par[2]
  beta_max <- par[3]
  beta_0 <- par[4:(len_temp + 3)]
  alpha <- par[(4 + len_temp):(3 + len_temp * 2)]
  
  .trim_sum(with(data, (
    beta_0[temp_i] + (alpha[temp_i] * beta_max) /
      (1 + exp(-slope * (log_conc - zeta))) -
      log2_value
  ) ^ 2))
}

.min_RSS_h0_gradient <- function(data, par, len_temp) {
  # Analytically solved gradient function for .min_RSS_h1
  temp_i <- log2_value <- NULL
  
  beta_0 <- par[seq_len(len_temp)]
  
  data <-
    full_join(data,
              expand.grid(
                log_conc = unique(data$log_conc),
                temp_i = seq_len(max(data$temp_i))
              ),
              by = c("log_conc", "temp_i"))
  
  outer_dev <-
    with(data, 2 * (beta_0[temp_i] - log2_value))
  
  d_beta_0 <- apply(matrix(outer_dev, nrow = 5, byrow = TRUE),
                    2, sum, na.rm = TRUE)
  
  return(d_beta_0)
  
}

.min_RSS_h0_gradient_trim <- function(data, par, len_temp) {
  # Analytically solved gradient function for .min_RSS_h1
  temp_i <- log2_value <- NULL
  
  beta_0 <- par[seq_len(len_temp)]
  
  data <-
    full_join(data,
              expand.grid(
                log_conc = unique(data$log_conc),
                temp_i = seq_len(max(data$temp_i))
              ),
              by = c("log_conc", "temp_i"))
  
  outer_dev <-
    with(data, 2 * (beta_0[temp_i] - log2_value))
  
  d_beta_0 <- apply(matrix(outer_dev, nrow = 5, byrow = TRUE),
                    2, .trim_sum, na.rm = TRUE)
  
  return(d_beta_0)
  
}

.min_RSS_h1_gradient <- function(data, par, len_temp) {
  # Analytically solved gradient function for .min_RSS_h1
  temp_i <- log2_value <- log_conc <- NULL
  
  zeta <- par[1]
  slope <- par[2]
  beta_max <- par[3]
  beta_0 <- par[4:(len_temp + 3)]
  alpha <- par[(4 + len_temp):(3 + len_temp * 2)]
  
  data <-
    full_join(data,
              expand.grid(
                log_conc = unique(data$log_conc),
                temp_i = seq_len(max(data$temp_i))
              ),
              by = c("log_conc", "temp_i"))
  outer_dev <-
    with(data, 2 * (beta_0[temp_i] + (alpha[temp_i] * beta_max) /
                      (1 + exp(
                        -slope * (log_conc - zeta)
                      )) -
                      log2_value))
  d_zeta <- sum(outer_dev * with(data,-(alpha * beta_max) / (1 + exp(
    -slope * (log_conc - zeta)
  )) ^ 2 *
    exp(-slope * (log_conc - zeta)) * slope), na.rm = TRUE)
  d_slope <- sum(outer_dev * with(data,-(alpha * beta_max) / (1 + exp(
    -slope * (log_conc - zeta)
  )) ^ 2 *
    exp(-slope * (log_conc - zeta)) * (-(log_conc - zeta))), na.rm = TRUE)
  d_beta_max <- sum(outer_dev * with(data, alpha / (1 + exp(
    -slope * (log_conc - zeta)
  ))), na.rm = TRUE)
  d_beta_0 <-
    apply(matrix(outer_dev, nrow = 5, byrow = TRUE), 2, sum, na.rm = TRUE)
  d_alpha <- apply(matrix(
    outer_dev * with(data, beta_max / (1 + exp(
      -slope * (log_conc - zeta)
    ))),
    nrow = 5,
    byrow = TRUE
  ), 2, sum, na.rm = TRUE)
  return(c(d_zeta, d_slope, d_beta_max, d_beta_0, d_alpha))
  
}

.min_RSS_h1_gradient_trim <- function(data, par, len_temp) {
  # Analytically solved gradient function for .min_RSS_h1_trim
  temp_i <- log2_value <- log_conc <- NULL
  
  zeta <- par[1]
  slope <- par[2]
  beta_max <- par[3]
  beta_0 <- par[4:(len_temp + 3)]
  alpha <- par[(4 + len_temp):(3 + len_temp * 2)]
  
  data <-
    full_join(data,
              expand.grid(
                log_conc = unique(data$log_conc),
                temp_i = seq_len(max(data$temp_i))
              ),
              by = c("log_conc", "temp_i"))
  outer_dev <-
    with(data, 2 * (beta_0[temp_i] + (alpha[temp_i] * beta_max) /
                      (1 + exp(
                        -slope * (log_conc - zeta)
                      )) -
                      log2_value))
  d_zeta <- .trim_sum(outer_dev * with(data,-(alpha * beta_max) / (1 + exp(
    -slope * (log_conc - zeta)
  )) ^ 2 *
    exp(-slope * (log_conc - zeta)) * slope), na.rm = TRUE)
  d_slope <- .trim_sum(outer_dev * with(data,-(alpha * beta_max) / (1 + exp(
    -slope * (log_conc - zeta)
  )) ^ 2 *
    exp(-slope * (log_conc - zeta)) * (-(log_conc - zeta))), na.rm = TRUE)
  d_beta_max <- .trim_sum(outer_dev * with(data, alpha / (1 + exp(
    -slope * (log_conc - zeta)
  ))), na.rm = TRUE)
  d_beta_0 <-
    apply(matrix(outer_dev, nrow = 5, byrow = TRUE), 2, .trim_sum, na.rm = TRUE)
  d_alpha <- apply(matrix(
    outer_dev * with(data, beta_max / (1 + exp(
      -slope * (log_conc - zeta)
    ))),
    nrow = 5,
    byrow = TRUE
  ),
  2,
  .trim_sum,
  na.rm = TRUE)
  return(c(d_zeta, d_slope, d_beta_max, d_beta_0, d_alpha))
  
}

.min_RSS_h1_slope50_gradient <- function(data, par, len_temp) {
  # Analytically solved gradient function for .min_RSS_h1_slope_pEC50
  temp_i <- log2_value <- log_conc <- temperature <- NULL
  
  zeta <- par[1]
  zeta_slope <- par[2]
  slope <- par[3]
  beta_max <- par[4]
  beta_0 <- par[5:(len_temp + 4)]
  alpha <- par[(5 + len_temp):(4 + len_temp * 2)]
  
  data <-
    full_join(data,
              expand.grid(
                log_conc = unique(data$log_conc),
                temp_i = seq_len(max(data$temp_i))
              ),
              by = c("log_conc", "temp_i"))
  outer_dev <-
    with(data,
         2 * (beta_0[temp_i] + (alpha[temp_i] * beta_max) /
                (1 + exp(
                  -slope * (log_conc - (zeta + zeta_slope * temperature))
                )) -
                log2_value))
  
  d_zeta <- sum(outer_dev *
                  with(data,-(alpha * beta_max) /
                         (1 + exp(
                           -slope * (log_conc - (zeta + zeta_slope * temperature))
                         )) ^ 2 *
                         exp(-slope * (
                           log_conc - (zeta + zeta_slope * temperature)
                         )) * slope),
                na.rm = TRUE)
  
  d_zeta_slope <- sum(outer_dev * with(
    data,
    -(alpha * beta_max) / (1 + exp(-slope * (
      log_conc - (zeta + zeta_slope * temperature)
    ))) ^ 2 *
      exp(-slope * (
        log_conc - (zeta + zeta_slope * temperature)
      )) * (slope * temperature)
  ), na.rm = TRUE)
  
  d_slope <- sum(outer_dev * with(data,-(alpha * beta_max) / (1 + exp(
    -slope * (log_conc - (zeta + zeta_slope * temperature))
  )) ^ 2 *
    exp(-slope * (
      log_conc - (zeta + zeta_slope * temperature)
    )) * (-(
      log_conc - (zeta + zeta_slope * temperature)
    ))), na.rm = TRUE)
  d_beta_max <- sum(outer_dev * with(data, alpha / (1 + exp(
    -slope * (log_conc - (zeta + zeta_slope * temperature))
  ))), na.rm = TRUE)
  d_beta_0 <-
    apply(matrix(outer_dev, nrow = 5, byrow = TRUE), 2, sum, na.rm = TRUE)
  d_alpha <- apply(matrix(
    outer_dev * with(data, beta_max / (1 + exp(
      -slope * (log_conc - (zeta + zeta_slope * temperature))
    ))),
    nrow = 5,
    byrow = TRUE
  ), 2, sum, na.rm = TRUE)
  return(c(d_zeta, d_zeta_slope, d_slope, d_beta_max, d_beta_0, d_alpha))
  
}

#' TPP2D defunct functions
#' 
#' @title Defunct functions in package \pkg{TPP2D}.
#' @description The functions listed below are defunct and will be removed in
#'   the near future. When possible, alternative functions with similar
#'   functionality are also mentioned. Help pages for deprecated functions are
#'   available at \code{help("-deprecated")}.
#' @name TPP2D-defunct
#' @keywords internal
NULL

#' S4 TPP2D Experiment Class
#'
#' @slot configTable data.frame.
#' @slot idVar character.
#' @slot intensityStr character.
#' @slot fcStr character.
#' @slot nonZeroCols character.
#' @slot geneNameVar character.
#' @slot qualColName character.
#' @slot naStrs character.
#' @slot concFactor numeric.
#' @slot medianNormalizeFC logical.
#' @slot filterContaminants logical.
#' @slot minObs numeric.
#' @slot independentFiltering logical.
#' @slot fcThres numeric.
#' @slot optim_fun_h0 function.
#' @slot optim_fun_h1 function.
#' @slot slopEC50 logical.
#' @slot maxit numeric.
#' @slot BPPARAM character.
#' @slot B numeric
#' @slot byMsExp logical.
#' @slot alpha numeric.
#' @slot tidyDataTable data.frame.
#' @slot modelParamsDf data.frame
#' @slot resultTable data.frame
#' @slot bootstrapNullDf data.frame
#' @slot hitTable data.frame
#'
#' @return an object of class tpp2dExperiment
#'
#' @export
#'
#' @examples
#' tpp2dObj <- new("tpp2dExperiment")
tpcaResult <- setClass("tpp2dExperiment",
                       slots = list(
                           configTable = "data.frame",
                           idVar = "character",
                           intensityStr = "character",
                           fcStr = "character",
                           nonZeroCols = "character",
                           geneNameVar = "character",
                           qualColName = "character",
                           naStrs = "character",
                           concFactor = "numeric",
                           medianNormalizeFC = "logical",
                           filterContaminants = "logical",
                           minObs = "numeric",
                           independentFiltering = "logical",
                           fcThres = "numeric",
                           optim_fun_h0 = "function",
                           optim_fun_h1 = "function",
                           slopEC50 = "logical",
                           maxit = "numeric",
                           BPPARAM = "character",
                           B = "numeric",
                           byMsExp = "logical",
                           alpha = "numeric",
                           tidyDataTable = "data.frame",
                           modelParamsDf = "data.frame",
                           resultTable = "data.frame",
                           bootstrapNullDf = "data.frame",
                           hitTable = "data.frame"
                           ))

#' Get FDR for given F statistics based on true and
#' null dataset 
#' 
#' @param df_out data frame containing results from analysis by
#' fitAndEvalDataset
#' @param df_null data frame containing results from analysis by
#' bootstrapNull
#' @param squeezeDenominator logical indicating whether F statistic
#' denominator should be shrinked using limma::squeezeVar
#' 
#' @return data frame annotating each protein with a FDR based on 
#' it's F statistic and number of observations
#' 
#' @examples 
#' data("simulated_cell_extract_df")
#' temp_df <- simulated_cell_extract_df %>% 
#'   filter(clustername %in% paste0("protein", 1:5)) %>% 
#'   group_by(representative) %>% 
#'   mutate(nObs = n()) %>% 
#'   ungroup 
#' example_out <- fitAndEvalDataset(temp_df)
#' example_null <- bootstrapNull(temp_df, B = 1)
#' getFDR(example_out, example_null)
#'  
#' @export
#'
#' @import dplyr
getFDR <- function(df_out, df_null, squeezeDenominator = TRUE){
    dataset <- nObs <- nObsRound <- F_statistic <- 
        is_decoy <- max_rank <- true_cumsum <- 
        null_cumsum <- representative <- clustername <- 
        dataset <- FDR <- all_true <- all_null <- 
        remaining_null <- remaining_true <- NULL
    
    if(squeezeDenominator){
        df_out <- .shrinkFstat(df_out, trueOrNull = "true")
        df_null <- .shrinkFstat(df_null, trueOrNull = "null")
    }
    
    B <- max(as.numeric(
        gsub("bootstrap_", "", unique(df_null$dataset))))
    
    out_df <- bind_rows(df_out %>% mutate(dataset = "true"),
                        df_null) %>%
        mutate(nObsRound = round(nObs, digits = -1)) %>%
        group_by(nObsRound) %>%
        arrange(desc(F_statistic)) %>%
        mutate(max_rank = n(),
               rank = dense_rank(desc(F_statistic)),
               is_decoy = ifelse(dataset != "true", 1, 0)) %>%
        mutate(all_null = sum(is_decoy),
               all_true = sum(!is_decoy),
               true_cumsum = cumsum(!is_decoy),
               null_cumsum = cumsum(is_decoy)) %>% 
        mutate(remaining_null = all_null - null_cumsum,
               remaining_true = all_true - true_cumsum) %>% 
        mutate(pi = remaining_true/(remaining_null/B)) %>% 
        mutate(FDR = pi * (null_cumsum/B)/true_cumsum) %>% 
        ungroup() %>% 
        mutate(FDR = ifelse(is.na(F_statistic), NA, FDR))
    
    return(out_df)
}

#' @importFrom limma squeezeVar 
.shrinkFstat <- function(inDf, trueOrNull = "true"){
  . <- rssH1 <- df2 <- rssH0 <- rssH1Squeezed <- 
    df1 <- df0 <- nObs <- nObsRound <- dataset <- NULL
  
  outDf <- inDf %>% 
    mutate(nObsRound = round(nObs, digits = -1)) %>% 
    { 
      if(trueOrNull == "true")
        group_by(., nObsRound)    
      else 
        group_by(., dataset, nObsRound) 
    } %>%
    do({
      squeezeResult <- limma::squeezeVar(.$rssH1/.$df2, df = .$df2)
      mutate(., rssH1Squeezed = squeezeResult$var.post) #,
             #df0 = squeezeResult$df.prior)
    }) %>%
    #mutate(df0 = ifelse(is.finite(df0), df0, 0)) %>% 
    ungroup %>% 
    mutate(F_statistic = (rssH0 - rssH1)/(rssH1Squeezed) * 1/df1)
             #(rssH1Squeezed) * (df0 + df2)/df1)
             
  return(outDf)
}
#' Compute FDR for given F statistics based on true and
#' null dataset (old function)
#' @name computeFdr-defunct
#' @seealso \code{\link{TPP2D-defunct}}
#' @keywords internal
NULL

#' @rdname TPP2D-defunct
#' @section \code{computeFdr}:
#' For \code{computeFdr}, use \code{\link{getFDR}}.
computeFdr <- function(df_out, df_null){
    .Defunct("getFDR")
}

#' Compute p-values for given F statistics based on true and
#' null dataset 
#' 
#' @param df_out data frame containing results from analysis by
#' fitAndEvalDataset
#' @param df_null data frame containing results from analysis by
#' bootstrapNull
#' @param squeezeDenominator logical indicating whether F statistic
#' denominator should be shrinked using limma::squeezeVar
#' @param pseudo_count numeric larger or equal to 0 added to both
#' counts of protein with an F-statistic higher than a threshold
#' theta of the true and bootstrapped datasets
#' 
#' @return data frame annotating each protein with a FDR based on 
#' it's F statistic and number of observations
#' 
#' @examples 
#' data("simulated_cell_extract_df")
#' temp_df <- simulated_cell_extract_df %>% 
#'   filter(clustername %in% paste0("protein", 1:3)) %>% 
#'   group_by(representative) %>% 
#'   mutate(nObs = n()) %>% 
#'   ungroup 
#' example_out <- fitAndEvalDataset(temp_df)
#' example_null <- bootstrapNull(temp_df, B = 2)
#' getPvalues(
#'   example_out, 
#'   example_null)
#'  
#' @export
#'
#' @import dplyr
#' @importFrom stats density
#' @importFrom stats p.adjust
getPvalues <- function(df_out, df_null, pseudo_count = 1,
                       squeezeDenominator = FALSE){
  dataset <- nObs <- nObsRound <- F_statistic <- 
    is_decoy <- max_rank <- true_cumsum <- p_value <- 
    null_cumsum <- representative <- clustername <- 
    dataset <- FDR <- all_true <- all_null <- NULL
  
  stopifnot(pseudo_count >= 0)
  
  if(squeezeDenominator){
    df_out <- .shrinkFstat(df_out, trueOrNull = "true")
    df_null <- .shrinkFstat(df_null, trueOrNull = "null")
  }
  
  out_df <- bind_rows(df_out %>% mutate(dataset = "true"),
                      df_null) %>%
    mutate(nObsRound = round(nObs, digits = -1)) %>%
    group_by(nObsRound) %>%
    arrange(desc(F_statistic)) %>%
    mutate(max_rank = n(),
           rank = dense_rank(dplyr::desc(F_statistic)),
           is_decoy = ifelse(dataset != "true", 1, 0)) %>%
    mutate(all_null = sum(is_decoy),
           null_cumsum = cumsum(is_decoy)) %>% 
    mutate(p_value = (null_cumsum + pseudo_count)/
             (all_null + pseudo_count)) %>% 
    ungroup() %>% 
    filter(dataset == "true") %>% 
    mutate(p_adj = p.adjust(p_value, method = "BH"))
  
  return(out_df)
}

#' Find hits according to FDR threshold
#' 
#' @param fdr_df data frame obtained from computeFdr
#' @param alpha significance threshold, default is set to 0.1
#' 
#' @return data frame of significant hits at FDR <= alpha
#' 
#' @examples 
#' data("simulated_cell_extract_df")
#' temp_df <- simulated_cell_extract_df %>% 
#'   filter(clustername %in% paste0("protein", 1:5)) %>% 
#'   group_by(representative) %>% 
#'   mutate(nObs = n()) %>% 
#'   ungroup 
#' example_out <- fitAndEvalDataset(temp_df)
#' example_null <- bootstrapNull(temp_df, B = 1)
#' fdr_df <- getFDR(example_out, example_null)
#' findHits(fdr_df, 0.1)
#' 
#' @export
#' 
#' @import dplyr
findHits <- function(fdr_df, alpha){
  
  nObsRound <- FDR <- max_rank_fdr <- dataset <- min_rank_true <- 
      representative <- clustername <- nObs <- nCoeffsH0 <- 
      nCoeffsH1 <- rssH0 <- rssH1 <- df1 <- df2 <- F_statistic <- 
      pEC50H1 <- slopeH1 <- pEC50_slopeH1 <- detected_effectH1 <- 
      df_fil <- NULL
  
  hits_df <- fdr_df %>% 
    filter(!is.na(F_statistic)) %>% 
    group_by(nObsRound) %>% 
    mutate(min_rank_true = min(rank[dataset == "true"])) %>% 
    filter(rank >= min_rank_true) %>% 
    mutate(max_rank_fdr = min(c(Inf, rank[FDR > alpha]), na.rm = TRUE)) %>% 
    filter(rank < max_rank_fdr) %>% 
    filter(dataset == "true") %>% 
    ungroup() %>% 
    dplyr::select(representative, clustername, nObs, matches("qupm"),
                  nCoeffsH0, nCoeffsH1, rssH0, rssH1, 
                  df1, df2, F_statistic, pEC50H1, slopeH1, pEC50_slopeH1, 
                  detected_effectH1, nObsRound, pi, FDR)
  
  return(hits_df)
}

#' @title Example subset of a simulated 2D-TPP cell 
#' extract dataset 
#' @name simulated_cell_extract_df
#' @docType data
#' @description Simulated example dataset obtained by 2D-TPP 
#' experiments for analysis by the TPP2D-package. It contains 
#' a tidy data frame after import and recomputing of robust 
#' signal intensities with 200 simulated protein profiles 
#' (protein1-200) and 3 spiked-in true positives (TP1-3)
#' @format data frame with columns representative (protein id), 
#' clustername (gene name), temperature, log_conc, raw_value, 
#' rel_value, value and log2_value
#' @usage data("simulated_cell_extract_df")
"simulated_cell_extract_df"

#' @title Example raw data for a subset of a simulated 
#' 2D-TPP cell extract dataset 
#' @name raw_dat_list
#' @docType data
#' @description Simulated example dataset obtained by 2D-TPP 
#' experiments for analysis by the TPP2D-package. It contains 
#' a list of data frames resembling raw data files returned 
#' from a MS database search with 200 simulated protein profiles 
#' (protein1-200) and 3 spiked-in true positives (TP1-3).
#' @format list of data frames with columns representative 
#' (protein id), clustername (gene name), temperature, log_conc, 
#' raw_value, rel_value, value and log2_value
#' @usage data("raw_dat_list")
"raw_dat_list"

#' @title Example config table for a import of a simulated 
#' 2D-TPP cell extract dataset 
#' @name config_tab
#' @docType data
#' @description Config table fot import of simulated example 
#' dataset obtained by 2D-TPP experiments for analysis by 
#' the TPP2D-package. It's a data frame with the columns
#' "Compound" describing the compound used for the assay, 
#' "Experiment" listing MS experiment ids of the separate runs
#' (typically comprising two multiplexed adjacent temperature), 
#' "Temperature": the temperature used for a given sub-experimet, 
#' the respective TMT labels "126"-"131L", RefCol referring to
#' the label used as a reference label for computing relative
#' fold changes (usually the label used for the control treatment).
#' Please note that when the data is not supplied as a list of
#' already imported data frames the config table for the import 
#' function should be a path to an txt, csv or xlsx file containing
#' an additional column "Path" listing for each row the respective
#' path to a searched protein output file.
#' @format "Compound" describing the compound used for the assay, 
#' "Experiment" listing MS experiment ids of the separate runs
#' (typically comprising two multiplexed adjacent temperature), 
#' "Temperature": the temperature used for a given sub-experimet, 
#' the respective TMT labels "126"-"131L", RefCol referring to
#' the label used as a reference label for computing relative
#' fold changes (usually the label used for the control treatment).
#' @usage data("config_tab")
"config_tab"

#' Bootstrap null distribution of F statistics for FDR estimation
#'
#' @param df tidy data_frame retrieved after import of a 2D-TPP 
#' dataset, potential filtering and addition of a column "nObs"
#' containing the number of observations per protein
#' @param fcThres numeric value of minimal fold change 
#' (or inverse fold change) a protein has to show to be kept 
#' upon independent filtering
#' @param minObs numeric value of minimal number of observations
#' that should be required per protein
#' @param independentFiltering boolean flag indicating whether
#' independent filtering should be performed based on minimal
#' fold changes per protein profile
#' @param maxit maximal number of iterations the optimization
#' should be given, default is set to 500
#' @param optim_fun_h0 optimization function that should be used
#' for fitting the H0 model
#' @param optim_fun_h1 optimization function that should be used
#' for fitting the H1 model
#' @param optim_fun_h1_2 optional additional optimization function 
#' that will be run with paramters retrieved from optim_fun_h1 and 
#' should be used for fitting the H1 model with the trimmed sum
#' model, default is NULL
#' @param gr_fun_h0 optional gradient function for optim_fun_h0,
#' default is NULL
#' @param gr_fun_h1 optional gradient function for optim_fun_h1,
#' default is NULL
#' @param gr_fun_h1_2 optional gradient function for optim_fun_h1_2,
#' default is NULL
#' @param ncores numeric value of numbers of cores that the function 
#' should use to parallelize
#' @param B numeric value of rounds of bootstrap, default: 20
#' @param byMsExp boolean flag indicating whether resampling of 
#' residuals should be performed separately for data generated by 
#' different MS experiments, default TRUE, recommended 
#' 
#' @return data frame containing F statistics of proteins with 
#' permuted 2D thermal profiles that are informative on the Null
#' distribution of F statistics
#' 
#' @examples
#' data("simulated_cell_extract_df")
#' temp_df <- simulated_cell_extract_df %>% 
#'   filter(clustername %in% paste0("protein", 1:3)) %>% 
#'   group_by(representative) %>% 
#'   mutate(nObs = n()) %>% 
#'   ungroup 
#' boot_df <- bootstrapNull(temp_df, B = 2/10)  
#' 
#' @export
#'
#' @importFrom stats lm
#' @importFrom stats residuals
#' @importFrom stats predict
#' @importFrom foreach foreach
#' @importFrom foreach %dopar%
#' @importFrom doParallel registerDoParallel
#' @importFrom parallel makeCluster
#' @importFrom parallel stopCluster
#' @import dplyr
bootstrapNull <- function(df, maxit = 500,
                          independentFiltering = FALSE,
                          fcThres = 1.5, minObs = 20,
                          optim_fun_h0 = .min_RSS_h0,
                          optim_fun_h1 = .min_RSS_h1_slope_pEC50,
                          optim_fun_h1_2 = NULL,
                          gr_fun_h0 = NULL,
                          gr_fun_h1 = NULL,
                          gr_fun_h1_2 = NULL,
                          ncores = 1,
                          B = 20,
                          byMsExp = TRUE){
  
  clustername <- prot <- log2_value <- experiment <- NULL
  
  .checkDfColumns(df)
  
  if(identical(optim_fun_h1, .min_RSS_h1_slope_pEC50)){
    slopEC50 = TRUE
  }else{
    slopEC50 = FALSE
  }
  
  ec50_limits <- .getEC50Limits(df)
  
  df_fil <- .minObsFilter(df, minObs = minObs)
  
  if(independentFiltering){
    message("Independent Filtering: removing proteins without 
            any values crossing the threshold.")
    df_fil <- .independentFilter(df_fil, fcThres = fcThres) 
  }
  # if(identical(optim_fun_h1, .min_RSS_h1_slope_pEC50_qupm_weight)){
  #     df_fil <- df_fil %>% 
  #         mutate(weight = ifelse(qupm > 1, 1, 0.25)) %>% 
  #         group_by(representative) %>% 
  #         mutate(weight = weight * length(weight)/sum(weight)) %>% 
  #         ungroup
  # }
  cl <- makeCluster(ncores)
  registerDoParallel(cl)
  unique_names <- unique(df_fil$clustername)
  null_list <- foreach(prot = unique_names) %dopar% {
    df_prot <- filter(df_fil, clustername == prot)
    prot_h0 <- lm(log2_value ~ 1 + as.factor(temperature),
                  data = df_prot)
    len_res <- length(residuals(prot_h0))
    
    out_list <- lapply(seq_len(B), function(boot){
      if(!byMsExp){
          df_resample_prot <- df_prot %>%
              mutate(log2_value = log2_value - residuals(prot_h0) +
                         sample(residuals(prot_h0), size = len_res, replace = TRUE))   
      }else{
          exp_res_df <- data.frame(
              experiment = df_prot$experiment,
              res_h0 = residuals(prot_h0),
              pred_h0 = predict(prot_h0)
          )
          df_resample_prot <- bind_rows(
              lapply(unique(df_prot$experiment), function(ms_exp){
                exp_res_tmp = filter(exp_res_df, 
                                     experiment == ms_exp)
                filter(df_prot, experiment == ms_exp) %>% 
                    mutate(log2_value = exp_res_tmp$pred_h0 +
                               sample(exp_res_tmp$res_h0, replace = TRUE))
          }))
          
      }
      
      sum_df <- fitAndEvalDataset(df_resample_prot, 
                                  optim_fun_h0 = optim_fun_h0,
                                  optim_fun_h1 = optim_fun_h1,
                                  optim_fun_h1_2 = optim_fun_h1_2,
                                  gr_fun_h0 = gr_fun_h0,
                                  gr_fun_h1 = gr_fun_h1,
                                  gr_fun_h1_2 = gr_fun_h1_2,
                                  ec50_lower_limit = ec50_limits[1],
                                  ec50_upper_limit = ec50_limits[2],
                                  slopEC50 = slopEC50)
      
      return(sum_df)
    })
  }
  stopCluster(cl)
  null_df <- bind_rows(lapply(null_list, function(x){
    bind_rows(lapply(seq_len(length(x)), function(i){
      x[[i]] %>%
        mutate(dataset = paste("bootstrap", 
                               as.character(i), sep = "_"))
      }))
  }))
  
  return(null_df)
}

#' Bootstrap null distribution of F statistics for FDR estimation
#' based on resampling alternative model residuals
#'
#' @param df tidy data frame retrieved after import of a 2D-TPP 
#' dataset, potential filtering and addition of a column "nObs"
#' containing the number of observations per protein
#' @param params_df data frame listing all null and alternative
#' model parameters as obtained by 'getModelParamsDf'
#' @param fcThres numeric value of minimal fold change 
#' (or inverse fold change) a protein has to show to be kept 
#' upon independent filtering
#' @param minObs numeric value of minimal number of observations
#' that should be required per protein
#' @param independentFiltering boolean flag indicating whether
#' independent filtering should be performed based on minimal
#' fold changes per protein profile
#' @param maxit maximal number of iterations the optimization
#' should be given, default is set to 500
#' @param optim_fun_h0 optimization function that should be used
#' for fitting the H0 model
#' @param optim_fun_h1 optimization function that should be used
#' for fitting the H1 model
#' @param optim_fun_h1_2 optional additional optimization function 
#' that will be run with paramters retrieved from optim_fun_h1 and 
#' should be used for fitting the H1 model with the trimmed sum
#' model, default is NULL
#' @param gr_fun_h0 optional gradient function for optim_fun_h0,
#' default is NULL
#' @param gr_fun_h1 optional gradient function for optim_fun_h1,
#' default is NULL
#' @param gr_fun_h1_2 optional gradient function for optim_fun_h1_2,
#' default is NULL
#' @param BPPARAM BiocParallel parameter for optional parallelization
#' of null distribution generation through bootstrapping, 
#' default: BiocParallel::SerialParam()
#' @param B numeric value of rounds of bootstrap, default: 20
#' @param byMsExp boolean flag indicating whether resampling of 
#' residuals should be performed separately for data generated by 
#' different MS experiments, default TRUE, recommended 
#' @param verbose logical indicating whether to print each 
#' protein while its profile is boostrapped
#' 
#' @return data frame containing F statistics of proteins with 
#' permuted 2D thermal profiles that are informative on the Null
#' distribution of F statistics
#' 
#' @examples
#' data("simulated_cell_extract_df")
#' temp_df <- simulated_cell_extract_df %>% 
#'   filter(clustername %in% paste0("protein", 1:3)) %>% 
#'   group_by(representative) %>% 
#'   mutate(nObs = n()) %>% 
#'   ungroup 
#' temp_params_df <- getModelParamsDf(temp_df)
#' boot_df <- bootstrapNullAlternativeModel(
#'   temp_df, params_df = temp_params_df, B = 2)  
#' 
#' @export
#'
#' @importFrom stats lm
#' @importFrom stats residuals
#' @importFrom stats predict
#' @importFrom utils head
#' @import BiocParallel
#' @import dplyr
bootstrapNullAlternativeModel <- 
    function(df, 
             params_df,
             maxit = 500,
             independentFiltering = FALSE,
             fcThres = 1.5, minObs = 20,
             optim_fun_h0 = TPP2D:::.min_RSS_h0,
             optim_fun_h1 = TPP2D:::.min_RSS_h1_slope_pEC50,
             optim_fun_h1_2 = NULL,
             gr_fun_h0 = NULL,
             gr_fun_h1 = NULL,
             gr_fun_h1_2 = NULL,
             BPPARAM = BiocParallel::SerialParam(progressbar = TRUE),
             B = 20,
             byMsExp = TRUE,
             verbose = FALSE){
      
        clustername <- prot <- log2_value <- experiment <- 
            temperature <- temp_i <- log_conc <- nObs <- NULL 
        
        if(B < 20){
          print(paste("Warning: You have specificed B < 20, it is",
                      "recommended to use at least B = 20 in order", 
                      "to obtain reliable results."))
        }
        
        .checkDfColumns(df)
        
        if(identical(optim_fun_h1, .min_RSS_h1_slope_pEC50)){
            slopEC50 = TRUE
        }else{
            slopEC50 = FALSE
        }
        
        ec50_limits <- .getEC50Limits(df)
        
        df_fil <- .minObsFilter(df, minObs = minObs) %>% 
            mutate(temp_i = dense_rank(temperature)) %>% 
            arrange(clustername, temp_i, log_conc)
        
        if(independentFiltering){
            message("Independent Filtering: removing proteins without 
                    any values crossing the threshold.")
            df_fil <- .independentFilter(df_fil, fcThres = fcThres) 
        }
        unique_names <- unique(df_fil$clustername)
        null_list <- BiocParallel::bplapply(unique_names, BPPARAM = BPPARAM, function(prot){
            if(verbose){
                print(prot)
            }
            df_prot <- filter(df_fil, clustername == prot) 
            params_prot <- head(filter(params_df, clustername == prot) %>% 
                filter(nObs == max(nObs)), 1)
            len_res <- length(params_prot$residualsH0[[1]])
            h0_predicted <- params_prot$estimateH0[[1]]
            unique_temp_prot <- unique(df_prot$temperature)
            len_temp_prot <- length(unique_temp_prot)
            
            out_list <- lapply(seq_len(B), function(boot){
                if(!byMsExp){
                    df_resample_prot <- df_prot %>%
                        mutate(log2_value = h0_predicted +
                                   sample(params_prot$residualsH1[[1]], 
                                          size = len_res, replace = TRUE))   
                }else{
                    exp_res_df <- data.frame(
                        experiment = df_prot$experiment,
                        res_h1 = params_prot$residualsH1[[1]],
                        pred_h0 = h0_predicted
                    )
                    df_resample_prot <- bind_rows(
                        lapply(unique(df_prot$experiment), function(ms_exp){
                            exp_res_tmp = filter(exp_res_df, 
                                                 experiment == ms_exp)
                            filter(df_prot, experiment == ms_exp) %>% 
                                mutate(log2_value = exp_res_tmp$pred_h0 +
                                           sample(exp_res_tmp$res_h1, replace = TRUE))
                        }))
                    
                }
                
                sum_df <- fitAndEvalDataset(df_resample_prot, 
                                            optim_fun_h0 = optim_fun_h0,
                                            optim_fun_h1 = optim_fun_h1,
                                            optim_fun_h1_2 = optim_fun_h1_2,
                                            gr_fun_h0 = gr_fun_h0,
                                            gr_fun_h1 = gr_fun_h1,
                                            gr_fun_h1_2 = gr_fun_h1_2,
                                            ec50_lower_limit = ec50_limits[1],
                                            ec50_upper_limit = ec50_limits[2],
                                            slopEC50 = slopEC50)
                
                return(sum_df)
            })
        })
        null_df <- bind_rows(lapply(null_list, function(x){
            bind_rows(lapply(seq_len(length(x)), function(i){
                if("data.frame" %in% class(x[[i]])){
                    x[[i]] %>%
                        mutate(dataset = paste("bootstrap", 
                                               as.character(i), sep = "_"))
                }else{
                    tibble()
                }
            }))
        }))
        
        return(null_df)
    }

#' Bootstrap null distribution of F statistics for FDR estimation
#' based on resampling alternative model residuals with only
#' one round of model fitting on resampled data and subsequent
#' resampling of thereby obtained residuals
#'
#' @param df tidy data frame retrieved after import of a 2D-TPP 
#' dataset, potential filtering and addition of a column "nObs"
#' containing the number of observations per protein
#' @param params_df data frame listing all null and alternative
#' model parameters as obtained by 'getModelParamsDf'
#' @param fcThres numeric value of minimal fold change 
#' (or inverse fold change) a protein has to show to be kept 
#' upon independent filtering
#' @param minObs numeric value of minimal number of observations
#' that should be required per protein
#' @param independentFiltering boolean flag indicating whether
#' independent filtering should be performed based on minimal
#' fold changes per protein profile
#' @param maxit maximal number of iterations the optimization
#' should be given, default is set to 500
#' @param optim_fun_h0 optimization function that should be used
#' for fitting the H0 model
#' @param optim_fun_h1 optimization function that should be used
#' for fitting the H1 model
#' @param optim_fun_h1_2 optional additional optimization function 
#' that will be run with paramters retrieved from optim_fun_h1 and 
#' should be used for fitting the H1 model with the trimmed sum
#' model, default is NULL
#' @param gr_fun_h0 optional gradient function for optim_fun_h0,
#' default is NULL
#' @param gr_fun_h1 optional gradient function for optim_fun_h1,
#' default is NULL
#' @param gr_fun_h1_2 optional gradient function for optim_fun_h1_2,
#' default is NULL
#' @param BPPARAM BiocParallel parameter for optional parallelization
#' of null distribution generation through bootstrapping, 
#' default: BiocParallel::SerialParam()
#' @param B numeric value of rounds of bootstrap, default: 20
#' @param byMsExp boolean flag indicating whether resampling of 
#' residuals should be performed separately for data generated by 
#' different MS experiments, default TRUE, recommended 
#' @param verbose logical indicating whether to print each 
#' protein while its profile is boostrapped
#' 
#' @return data frame containing F statistics of proteins with 
#' permuted 2D thermal profiles that are informative on the Null
#' distribution of F statistics
#' 
#' @examples
#' data("simulated_cell_extract_df")
#' temp_df <- simulated_cell_extract_df %>% 
#'   filter(clustername %in% paste0("protein", 1:3)) %>% 
#'   group_by(representative) %>% 
#'   mutate(nObs = n()) %>% 
#'   ungroup 
#' temp_params_df <- getModelParamsDf(temp_df)
#' boot_df <- bootstrapNullAlternativeModelFast(
#'   temp_df, params_df = temp_params_df, B = 20)  
#' 
#' @export
#'
#' @importFrom stats lm
#' @importFrom stats residuals
#' @importFrom stats predict
#' @importFrom utils head
#' @import BiocParallel
#' @import dplyr
bootstrapNullAlternativeModelFast <- 
  function(df, 
           params_df,
           maxit = 500,
           independentFiltering = FALSE,
           fcThres = 1.5, minObs = 20,
           optim_fun_h0 = TPP2D:::.min_RSS_h0,
           optim_fun_h1 = TPP2D:::.min_RSS_h1_slope_pEC50,
           optim_fun_h1_2 = NULL,
           gr_fun_h0 = NULL,
           gr_fun_h1 = NULL,
           gr_fun_h1_2 = NULL,
           BPPARAM = BiocParallel::SerialParam(progressbar = TRUE),
           B = 20,
           byMsExp = TRUE,
           verbose = FALSE){
    
    clustername <- prot <- log2_value <- experiment <- 
      temperature <- temp_i <- log_conc <- nObs <- NULL 
    
    if(B < 20){
      print(paste("Warning: You have specificed B < 20, it is",
                  "recommended to use at least B = 20 in order", 
                  "to obtain reliable results."))
    }
    
    .checkDfColumns(df)
    
    if(identical(optim_fun_h1, .min_RSS_h1_slope_pEC50)){
      slopEC50 = TRUE
    }else{
      slopEC50 = FALSE
    }
    
    ec50_limits <- .getEC50Limits(df)
    
    df_fil <- .minObsFilter(df, minObs = minObs) %>% 
      mutate(temp_i = dense_rank(temperature)) %>% 
      arrange(clustername, temp_i, log_conc)
    
    if(independentFiltering){
      message("Independent Filtering: removing proteins without 
                    any values crossing the threshold.")
      df_fil <- .independentFilter(df_fil, fcThres = fcThres) 
    }
    unique_names <- unique(df_fil$clustername)
    null_list <- BiocParallel::bplapply(unique_names, BPPARAM = BPPARAM, function(prot){
      if(verbose){
        print(prot)
      }
      df_prot <- filter(df_fil, clustername == prot) 
      params_prot <- head(filter(params_df, clustername == prot) %>% 
                            filter(nObs == max(nObs)), 1)
      len_res <- length(params_prot$residualsH0[[1]])
      h0_predicted <- params_prot$estimateH0[[1]]
      unique_temp_prot <- unique(df_prot$temperature)
      len_temp_prot <- length(unique_temp_prot)
      
      if(!byMsExp){
        df_resample_prot <- df_prot %>%
          mutate(log2_value = h0_predicted +
                   sample(params_prot$residualsH1[[1]], 
                          size = len_res, replace = TRUE))   
      }else{
        exp_res_df <- data.frame(
          experiment = df_prot$experiment,
          res_h1 = params_prot$residualsH1[[1]],
          pred_h0 = h0_predicted
        )
        df_resample_prot <- bind_rows(
          lapply(unique(df_prot$experiment), function(ms_exp){
            exp_res_tmp = filter(exp_res_df, 
                                 experiment == ms_exp)
            filter(df_prot, experiment == ms_exp) %>% 
              mutate(log2_value = exp_res_tmp$pred_h0 +
                       sample(exp_res_tmp$res_h1, replace = TRUE))
          }))
        
      }
      
      params_df <- getModelParamsDf(df_resample_prot, 
                                    minObs = minObs,
                                    optim_fun_h0 = optim_fun_h0,
                                    optim_fun_h1 = optim_fun_h1,
                                    optim_fun_h1_2 = optim_fun_h1_2,
                                    gr_fun_h0 = gr_fun_h0,
                                    gr_fun_h1 = gr_fun_h1,
                                    gr_fun_h1_2 = gr_fun_h1_2,
                                    slopEC50 = slopEC50)
      
      out_list <- lapply(seq_len(B), function(boot){
        b_nobs <- params_df$nObs
        ids_resampled <- sample(seq(b_nobs), replace = TRUE)
        params_df$residualsH0[[1]] <-  params_df$residualsH0[[1]][ids_resampled]
        params_df$residualsH1[[1]] <-  params_df$residualsH1[[1]][ids_resampled]
        params_df$rssH0 <- sum(params_df$residualsH0[[1]]^2)
        params_df$rssH1 <- sum(params_df$residualsH1[[1]]^2)
        fstat_df <- computeFStatFromParams(params_df)
        return(fstat_df)
      })
    })
    null_df <- bind_rows(lapply(null_list, function(x){
      bind_rows(lapply(seq_len(length(x)), function(i){
        if("data.frame" %in% class(x[[i]])){
          x[[i]] %>%
            mutate(dataset = paste("bootstrap", 
                                   as.character(i), sep = "_"))
        }else{
          tibble()
        }
      }))
    }))
    
    return(null_df)
  }

## Tests for recomputeSignalFromRatios()
## library(TPP2D); library(testthat); source("setup-2dtpp-dataset.R"); source("test-bootstrap-null.R")
context("bootstrap null")

# test_that("bootstrapNull works as expected", {
#   boot_df <- bootstrapNull(simulated_cell_extract_df %>% 
#                              filter(clustername == "tp1"),
#                            B = 3)
#   expect_equal(nrow(boot_df), 3)
# })

test_that("bootstrapNullAlternativeModel works as expected", {
    
    sub_df <- simulated_cell_extract_df %>% 
        filter(clustername == "tp1")
    params_df <- getModelParamsDf(sub_df)
    boot_df <- bootstrapNullAlternativeModel(
        df = sub_df,
        params_df = params_df,
        B = 3)
    expect_equal(nrow(boot_df), 3)
})

#' @importFrom stats lm
#' @importFrom stats coef
.getStartParameters <- function(df, unique_temp, len_temp,
                               slopEC50 = FALSE){
  # function to generate vector of start parameters
  # for h1 model optimization
  log_conc <- log2_value <- temperature <- NULL
  
  start_par <- c(mean(unique(df$log_conc)[
    which(is.finite(unique(df$log_conc)))]))
    if(slopEC50){
      start_par <- c(start_par, 0)
    }
    start_par <- c(
      start_par,
      coef(lm(log2_value ~ log_conc, 
              filter(df, is.finite(log_conc))))[[2]],
      max(unlist(lapply(unique_temp, function(temp){
        max(filter(df, temperature == temp)$log2_value) -
          min(filter(df, temperature == temp)$log2_value)
        }))),
      unlist(lapply(unique_temp, function(temp){
        mean(filter(df, temperature == temp)$log2_value)
        })),
      rep(0, len_temp))
    return(start_par)
}

.paste_rmNA <- function(x, sep = "|"){
  # function that pastes non-dedundant and na-filtered
  # vector elements into a string
  return(paste(x[!is.na(x)], collapse = sep))
}

.getOptLimits <- function(ec50Limits, len_temp, 
                         slopEC50 = FALSE){
  # function to generate list of vectors defining lower 
  # and upper limits of optimization parameters
  if(!slopEC50){
    out_list <- list(lower = c(ec50Limits[1],
                               rep(-Inf, 2 + len_temp), 
                               rep(0, len_temp)),
                     upper = c(ec50Limits[2],
                               rep(Inf, 2 + len_temp), 
                               rep(1, len_temp)))
  }else{
    out_list <- list(lower = c(ec50Limits[1],
                               rep(-Inf, 3 + len_temp), 
                               rep(0, len_temp)),
                     upper = c(ec50Limits[2],
                               rep(Inf, 3 + len_temp), 
                               rep(1, len_temp)))
  }
  return(out_list)
}

.checkDfColumns <- function(df){
  # internal function to check all needed columns
  # are presented in supplied input data frame
  req_coln <- c("representative",
                "clustername",
                "temperature",
                "log_conc",
                "log2_value")
  coln_df <- colnames(df)
  if(!all(req_coln %in% coln_df)){
    stop(paste(c("Input data frame, requires at least the", 
                 "following columns: representative,", 
                 "clustername temperature, log_conc,",
                 "log2_value!"), collapse = " "))
  }
}

.fitH0ParamDf <- function(df, 
                          optim_fun = .min_RSS_h0,
                          gr_fun = NULL,
                          slopEC50 = TRUE, 
                          maxit = 500,
                          qualColName = "qupm"){
    representative <- clustername <- nObs <- temperature <- 
        temp_i <- log_conc <- log2_value <- y_hat <- . <- NULL
    
    h0_param_df <- df %>% 
        group_by(representative, clustername, nObs) %>%
        mutate(temp_i = dense_rank(temperature)) %>%
        arrange(clustername, temp_i, log_conc) %>% 
        do({
            unique_temp <- unique(.$temperature)
            len_temp <- length(unique_temp)
            start_par = vapply(unique_temp, function(x)
                mean(filter(., temperature == x)$log2_value), 1)
            h0_model = try(optim(par = start_par,
                                 fn = optim_fun,
                                 len_temp = len_temp,
                                 data = .,
                                 method = "L-BFGS-B",
                                 control = list(maxit = maxit)))
            fit_df <- .getFitDf(
                df_fil = ., 
                conc_vec = unique(.$log_conc),
                model_type = "H0",
                optim_model = h0_model,
                slopEC50 = slopEC50)
            
            fit_est_df <- left_join(
                ., fit_df,
                by = c("log_conc", "temp_i")) %>%
                mutate(residuals = log2_value - y_hat)
            
            tibble(min_qupm = min(.[[qualColName]]),
                   max_qupm = max(.[[qualColName]]),
                   nCoeffs = length(h0_model$par),
                   rss = h0_model$value,
                   par = list(h0_model$par),
                   estimate = list(fit_est_df$y_hat),
                   residuals = list(fit_est_df$residuals))

        }) %>% 
        ungroup()
    return(h0_param_df)
}

.fitH1ParamDf <- function(df, optim_fun = .min_RSS_h1_slope_pEC50, 
                         optim_fun_2 = NULL,
                         gr_fun = NULL,
                         gr_fun_2 = NULL,
                         slopEC50 = TRUE, 
                         maxit = 500){
    representative <- clustername <- nObs <- temperature <- 
        temp_i <- log_conc <- log2_value <- y_hat <- . <- NULL
    
    h1_param_df <- df %>% 
        group_by(representative, clustername, nObs) %>%
        mutate(temp_i = dense_rank(temperature)) %>%
        arrange(clustername, temp_i, log_conc) %>% 
        do({
            unique_temp <- unique(.$temperature)
            len_temp <- length(unique_temp)
            h1_model <- .fitEvalH1(df_fil = .,
                                   unique_temp = unique_temp, 
                                   len_temp = len_temp,
                                   optim_fun = optim_fun, 
                                   optim_fun_2 = optim_fun_2,
                                   gr_fun = gr_fun,
                                   gr_fun_2 = gr_fun_2,
                                   slopEC50 = slopEC50, 
                                   maxit = maxit)
            
            key_params_df <- .eval_optim_result(
                h1_model, hypothesis = "H1",
                data = ., len_temp = len_temp,
                slopEC50 = slopEC50)
            
            fit_df <- .getFitDf(
                df_fil = ., 
                conc_vec = unique(.$log_conc),
                model_type = "H1",
                optim_model = h1_model,
                slopEC50 = TRUE) 
            
            fit_est_df <- left_join(
                ., fit_df,
                by = c("log_conc", "temp_i")) %>%
                mutate(residuals = log2_value - y_hat)
            
            tibble(nCoeffs = length(h1_model$par),
                   rss = h1_model$value,
                   par = list(h1_model$par),
                   estimate = list(fit_est_df$y_hat),
                   residuals = list(fit_est_df$residuals),
                   pEC50H1 = key_params_df$pEC50H1,
                   slopeH1 = key_params_df$slopeH1,
                   pEC50_slopeH1 = key_params_df$pEC50_slopeH1,
                   detected_effectH1 = key_params_df$detected_effectH1)
            
        }) %>% 
        ungroup()
    return(h1_param_df)
}

#' Get H0 and H1 model parameters
#'
#' @param df tidy data_frame retrieved after import of a 2D-TPP 
#' dataset, potential filtering and addition of a column "nObs"
#' containing the number of observations per protein
#' @param minObs numeric value of minimal number of observations
#' that should be required per protein
#' @param maxit maximal number of iterations the optimization
#' should be given, default is set to 500
#' @param optim_fun_h0 optimization function that should be used
#' for fitting the H0 model
#' @param optim_fun_h1 optimization function that should be used
#' for fitting the H1 model
#' @param optim_fun_h1_2 optional additional optimization function 
#' that will be run with paramters retrieved from optim_fun_h1 and 
#' should be used for fitting the H1 model with the trimmed sum
#' model, default is NULL
#' @param gr_fun_h0 optional gradient function for optim_fun_h0,
#' default is NULL
#' @param gr_fun_h1 optional gradient function for optim_fun_h1,
#' default is NULL
#' @param gr_fun_h1_2 optional gradient function for optim_fun_h1_2,
#' default is NULL
#' @param slopEC50 logical flag indicating whether the h1 model is
#' fitted with a linear model describing the shift od the pEC50 over 
#' temperatures
#' @param qualColName name of column indicating quantification
#' quality e.g. number of unique peptides used for quantification,
#' default: "qupm" 
#' 
#' @return a data.frame with fitted null and alternative model 
#' parameters
#' 
#' @export
#' 
#' @examples 
#' data("simulated_cell_extract_df")
#' getModelParamsDf(simulated_cell_extract_df)
getModelParamsDf <- function(df,
                             minObs = 20,
                             optim_fun_h0 = .min_RSS_h0,
                             optim_fun_h1 = .min_RSS_h1_slope_pEC50,
                             optim_fun_h1_2 = NULL,
                             gr_fun_h0 = NULL,
                             gr_fun_h1 = NULL,
                             gr_fun_h1_2 = NULL,
                             slopEC50 = TRUE,
                             maxit = 500,
                             qualColName = "qupm"){
    
    df_fil <- .minObsFilter(df, minObs = minObs)
    
    h0_param_df <- .fitH0ParamDf(
        df_fil,
        optim_fun = optim_fun_h0,
        gr_fun = gr_fun_h0,
        slopEC50 = slopEC50, 
        maxit = maxit)
    
    h1_param_df <- .fitH1ParamDf(
        df_fil,
        optim_fun = optim_fun_h1, 
        optim_fun_2 = optim_fun_h1_2,
        gr_fun = gr_fun_h1,
        gr_fun_2 = gr_fun_h1_2,
        slopEC50 = slopEC50, 
        maxit = maxit)
    
    combo_param_df <- left_join(
        h0_param_df, 
        h1_param_df, 
        by = c("representative", "clustername", "nObs"), 
        suffix = c("H0", "H1"))
    
    return(combo_param_df)
}

#' Plot qq-plot of true data and bootstrapped null with ggplot
#' 
#' @param x vector containing values of values of first 
#' distribution to compare
#' @param y vector containing values of values of secound 
#' distribution to compare
#' @param xlab x-axis label
#' @param ylab y-axis label
#' @param alpha transparency paramenter between 0 and 1
#' @param gg_theme ggplot theme, default is theme_classic()
#' @param offset offset for x and y axis on top of maximal 
#' values
#' @param plot_diagonal logical parameter indicating whether
#' an identity line should be plotted
#' 
#' @return A ggplot displaying the qq-plot of a true and a
#' a bootstrapped null distribution
#' 
#' @export
#'
#' @examples
#' 
#' data("simulated_cell_extract_df")
#' recomputeSignalFromRatios(simulated_cell_extract_df)
#'
#' @import ggplot2
#' @importFrom stats approx
gg_qq <- function(x, y, 
                  xlab = "F-statistics from sampled Null distr.",
                  ylab = "observed F-statistics", alpha = 0.25,
                  gg_theme = theme_classic(), offset = 1,
                  plot_diagonal = TRUE){
  sx <- sort(x)
  sy <- sort(y)
  lenx <- length(sx)
  leny <- length(sy)
  
  if (leny < lenx)
    sx <- approx(1L:lenx, sx, n = leny)$y
  if (leny > lenx)
    sy <- approx(1L:leny, sy, n = lenx)$y
  df <- data.frame(sx, sy)
  
  p <- ggplot(df, aes(sx, sy)) +
    geom_point(alpha = alpha) +
    coord_fixed(xlim = c(0, max(c(sx,sy)) + offset),
                ylim = c(0, max(c(sx,sy)) + offset),
                expand = FALSE) +
    xlab(xlab) +
    ylab(ylab) +
    gg_theme
  
  if(plot_diagonal){
      p <- p + 
          geom_line(aes(x, y),
                    linetype = "dashed",
                    color = "gray",
                    data = data.frame(
                        x = seq(0, max(c(sx,sy))),
                        y = seq(0, max(c(sx,sy))))) 
  }
  return(p)
}

#' Plot 2D thermal profile intensities of a protein 
#' of choice
#' 
#' @param df tidy data frame of a 2D-TPP dataset 
#' @param name gene name (clustername) of protein that 
#' should be visualized
#' 
#' @return A ggplot displaying the thermal profile of
#' a protein of choice in a datset of choice
#' 
#' @export
#'
#' @examples
#' 
#' data("simulated_cell_extract_df")
#' plot2dTppProfile(simulated_cell_extract_df, "protein1")
#'
#' @import ggplot2
plot2dTppProfile <- function(df, name){
  clustername <- log_conc <- log2_value <- 
    temperature <- NULL
  ggplot(filter(df, clustername == name),
         aes(log_conc, log2_value)) +
    geom_point() +
    facet_wrap(~temperature)
}

#' Plot 2D thermal profile ratios of a protein of choice
#' 
#' @param df tidy data frame of a 2D-TPP dataset 
#' @param name gene name (clustername) of protein that 
#' should be visualized
#' 
#' @return A ggplot displaying the thermal profile ratios of
#' a protein of choice in a datset of choice
#' 
#' @export
#'
#' @examples
#' 
#' data("simulated_cell_extract_df")
#' plot2dTppRelProfile(simulated_cell_extract_df, "protein1")
#'
#' @import ggplot2
plot2dTppRelProfile <- function(df, name){
  clustername <- log_conc <- rel_value <- 
    temperature <- NULL
  ggplot(filter(df, clustername == name),
         aes(log_conc, rel_value)) +
    geom_point() +
    facet_wrap(~temperature)
}

#' Plot H0 or H1 fit of 2D thermal profile intensities of 
#' a protein of choice
#' 
#' @param df tidy data frame of a 2D-TPP dataset 
#' @param name gene name (clustername) of protein that 
#' should be visualized
#' @param model_type character string indicating whether
#' the "H0" or the "H1" model should be fitted
#' @param optim_fun optimization function that should be used
#' for fitting either the H0 or H1 model
#' @param optim_fun_2 optional additional optimization function 
#' that will be run with paramters retrieved from optim_fun and 
#' should be used for fitting the H1 model with the trimmed sum
#' model, default is NULL
#' @param maxit maximal number of iterations the optimization
#' should be given, default is set to 500
#' @param xlab character string of x-axis label of plot
#' @param ylab character string of y-axis label of plot
#' @param dot_size numeric indicating the size of the data points
#' to plot
#' @param line_type character string defining the line type 
#' of the fitted curve, default "dashed"
#' @param fit_color character string defining the color
#' of the fitted curve
#' 
#' @return A ggplot displaying the thermal profile of
#' a protein of choice in a datset of choice
#' 
#' @export
#'
#' @examples
#' 
#' data("simulated_cell_extract_df")
#' plot2dTppProfile(simulated_cell_extract_df, "protein1")
#'
#' @import ggplot2
plot2dTppFit <- function(df, name,
                         model_type = "H0",
                         optim_fun = .min_RSS_h0,
                         optim_fun_2 = NULL,
                         maxit = 500,
                         xlab = "-log10(conc.)",
                         ylab = "log2(summed intensities)",
                         dot_size = 1,
                         line_type = "solid",
                         fit_color = "gray30"){
  
  clustername <- temperature <- temp_i <- 
    log_conc <- y_hat <- log2_value <- NULL
  
  .checkDfColumns(df)
  
  if(model_type == "H1"){
    optim_fun <- .min_RSS_h1_slope_pEC50
    slopEC50 <-  TRUE
  }else{
    slopEC50 <-  FALSE
  }
  
  df_fil <- filter(df, clustername == name) %>% 
    mutate(temp_i = dense_rank(temperature))
  unique_temp <- unique(df_fil$temperature)
  len_temp <- length(unique_temp)
  
  if(model_type == "H0"){
      start_par = vapply(unique_temp, function(x)
        mean(filter(df_fil, temperature == x)$log2_value), 1)
      h0_model = try(optim(par = start_par,
                           fn = optim_fun,
                           len_temp = len_temp,
                           data = df_fil,
                           method = "L-BFGS-B",
                           control = list(maxit = maxit)))
        
      fit_df <-
        .getFitDf(df_fil, model_type = model_type,
                 optim_model = h0_model,
                 slopEC50 = slopEC50)
  }else if(model_type == "H1"){
    h1_model <- .fitEvalH1(df_fil = df_fil,
                          unique_temp = unique_temp, 
                          len_temp = len_temp,
                          optim_fun = optim_fun, 
                          optim_fun_2 = optim_fun_2,
                          slopEC50 = slopEC50, 
                          maxit = maxit)
    fit_df <-
      .getFitDf(df_fil, model_type = model_type,
               optim_model = h1_model,
               slopEC50 = slopEC50)
  }else{
    stop("Please specify a valid model_type! Either H0 or H1!")
  }
  ggplot(fit_df, aes(log_conc, y_hat)) +
    geom_point(aes(log_conc, log2_value), 
               size = dot_size,
               shape = 21,
               data = df_fil) +
    geom_line(color = fit_color,
              linetype = line_type) +
    facet_wrap(~temperature) +
    ggtitle(name) +
    labs(x = xlab, y = ylab)
}


.getFitDf <- function(df_fil, conc_vec = seq(-9, -3, by = 0.1),
                     model_type = "H0", optim_model, 
                     slopEC50 = TRUE){
  # internal function to retrieve data frame with data
  # and predicted model values
  temp_i <- len_temp <- log_conc <- temperature <- NULL
  unique_temp <- unique(df_fil$temperature)
  unique_temp_len <- length(unique_temp)
  conc_len = length(conc_vec)
  
  if(model_type == "H0"){
    fit_df <-
      tibble(
        log_conc = rep(conc_vec, unique_temp_len),
        temperature = rep(unique_temp,
                          each = conc_len),
        temp_i = rep(seq_len(unique_temp_len), 
                     each = conc_len),
        len_temp = unique_temp_len) %>%
      mutate(y_hat = optim_model$par[temp_i])
    
  }else if(model_type == "H1"){
    if(slopEC50){
      fit_df <-
        tibble(
          log_conc = rep(conc_vec, unique_temp_len),
          temperature = rep(unique_temp, each = conc_len),
          temp_i = rep(seq_len(unique_temp_len), 
                       each = conc_len),
          len_temp = unique_temp_len) %>%
        mutate(y_hat = .evalH1SlopeEC50Model(
          optim_model = optim_model,
          temp_i = temp_i, len_temp = len_temp,
          log_conc = log_conc, temperature = temperature))
    }else{
      fit_df <-
        tibble(
          log_conc = rep(conc_vec, unique_temp_len),
          temperature = rep(unique_temp, each = conc_len),
          temp_i = rep(seq_len(unique_temp_len), 
                     each = conc_len),
          len_temp = unique_temp_len) %>%
        mutate(y_hat = .evalH1Model(
          optim_model = optim_model,
          temp_i = temp_i, len_temp = len_temp,
          log_conc = log_conc))
    }
  }else{
    stop("'model type' has to be either 'H0' or 'H1'!")
  }
  return(fit_df)
}

.evalH1Model <- function(optim_model, temp_i, log_conc, len_temp){
  optim_model$par[3 + temp_i] + 
    (optim_model$par[3 + len_temp + temp_i] * optim_model$par[3])/
    (1 + exp(-optim_model$par[2] * (log_conc - optim_model$par[1])))
}

.evalH1SlopeEC50Model <- function(optim_model, temp_i, len_temp,
                                 log_conc, temperature){
  optim_model$par[4 + temp_i] + 
    (optim_model$par[4 + len_temp + temp_i] * optim_model$par[4])/
    (1 + exp(-optim_model$par[3] * 
               (log_conc - (optim_model$par[1] + 
                  optim_model$par[2] * temperature))))
}

#' Plot heatmap of 2D thermal profile fold changes of 
#' a protein of choice
#' 
#' @param df tidy data frame of a 2D-TPP dataset 
#' @param name gene name (clustername) of protein that 
#' should be visualized
#' @param drug_name character string of profiled drug name
#' @param midpoint midpoint of fold changes for color
#' scaling, default: 1
#' 
#' @return A ggplot displaying the thermal profile 
#' as a heatmap of fold changes of
#' a protein of choice in a dataset of choice
#' 
#' @export
#' @examples
#' 
#' data("simulated_cell_extract_df")
#' plot2dTppFcHeatmap(simulated_cell_extract_df, 
#'  "tp2", drug_name = "drug1")
#'
#' @import ggplot2
#' @import dplyr
plot2dTppFcHeatmap <- function(df, name, 
                               drug_name = "",
                               midpoint = 1){
  clustername <- temperature <- conc <- 
    rel_value <- fc <- NULL
  
  heat_df <- df %>% 
    filter(clustername == name) %>% 
    dplyr::select(clustername, temperature, 
                  conc, rel_value) %>% 
    mutate(temperature = factor(
      temperature, levels = rev(sort(unique(temperature)))),
      conc = as.factor(conc)) %>% 
    group_by(clustername, temperature, conc) %>% 
    summarise(fc = mean(rel_value, na.rm = TRUE)) %>% 
    ungroup
  
  ggplot(heat_df, aes(conc, temperature)) +
    geom_tile(aes(fill = fc)) +
    scale_fill_gradient2(
      "Fold change", 
      low = "firebrick", mid = "khaki", 
      high = "darkgreen", midpoint = midpoint) +
    labs(x = paste(drug_name, "conc."),
         y = expression("Temperature ("*~degree*C*")")) +
    ggtitle(name) +
    theme_minimal()
}

#' Plot Volcano plot of TPP2D results
#' @param fdr_df data frame obtained from `getFDR`
#' @param hits_df hits_df data frame obtained from `findHits`
#' @param alpha transparency level of plotted points
#' @param title_string character argument handed over to ggtitle
#' @param x_lim vector with two numerics indicating the x axis limits
#' @param y_lim vector with two numerics indicating the y axis limits
#' @param facet_by_obs logical indicating whether plot should be facetted
#' by number of observations, default: FALSE
#'
#' @return a ggplot displaying a volcano plot of the results
#' obtained after a TPP2D analysis
#'
#' @import ggplot2
#' @import dplyr
#' 
#' @export
#' 
#' @examples 
#' data("simulated_cell_extract_df")
#' temp_df <- simulated_cell_extract_df %>%
#'   filter(clustername %in% paste0("protein", 1:5)) %>%
#'   group_by(representative) %>%
#'   mutate(nObs = n()) %>%
#'   ungroup
#' example_params <- getModelParamsDf(temp_df)
#' example_fstat <- computeFStatFromParams(example_params)
#' example_null <- bootstrapNullAlternativeModel(
#'    df = temp_df, params_df = example_params,
#'    B = 2)
#' fdr_df <- getFDR(example_fstat, example_null)
#' hits_df <- findHits(fdr_df, 0.1)
#' plot2dTppVolcano(fdr_df = fdr_df, hits_df = hits_df)
plot2dTppVolcano <- function(fdr_df, hits_df, 
                             alpha = 0.5,
                             title_string = "",
                             x_lim = NULL,
                             y_lim = NULL,
                             facet_by_obs = FALSE){
  dataset <- rssH0 <- rssH1 <- F_statistic <- group <- 
    clustername <- slopeH1 <- NULL
  
  stab_colors <- c("steelblue", "orange")
  names(stab_colors) <- c(
    "stabilized",
    "destabilized")
  
  fdr_true_df <- fdr_df %>% 
    filter(dataset == "true")
  
  if(is.null(x_lim)){
    x_lim <- c(
      floor(min(sign(fdr_true_df$slopeH1)*
            sqrt(fdr_true_df$rssH0 - fdr_true_df$rssH1)) - 0.5),
      round(max(sign(fdr_true_df$slopeH1)*
            sqrt(fdr_true_df$rssH0 - fdr_true_df$rssH1)) + 0.5)
    )
  }
  if(is.null(y_lim)){
    y_lim <- c(
      0,
      round(max(log2(fdr_true_df$F_statistic + 1)) + 0.5) 
    )
  }
  
  p <- ggplot(fdr_true_df %>% 
           mutate(group = case_when(slopeH1 > 0 ~ "stabilized",
                                    slopeH1 < 0 ~ "destabilized")), 
         aes(sign(slopeH1)*sqrt(rssH0 - rssH1), log2(F_statistic + 1))) +
    geom_point(color = "gray", alpha = alpha) + 
    geom_point(aes(color = group), alpha = alpha, 
               data = hits_df %>% 
                 mutate(group = case_when(
                   slopeH1 > 0 ~ "stabilized",
                   slopeH1 < 0 ~ "destabilized"))) + 
    geom_text(
      aes(label = clustername),
      data = hits_df, nudge_x = 0.5, nudge_y = 0.5) +
    scale_color_manual("", values = stab_colors) +
    labs(x = bquote(sign(kappa) %.% sqrt(~'RSS'^0~' - RSS'^1~'')),
         y = expression('log'[2]~'('*italic(F)*'-statistic + 1)')) +
    ggtitle(title_string) +
    coord_cartesian(xlim = x_lim,
                    ylim = y_lim) +
    theme(legend.position = "bottom")
  if(facet_by_obs){
    p <- p +
      facet_wrap(~nObsRound)
  }
  return(p)
}

## Tests for recomputeSignalFromRatios()
## library(TPP2D); library(testthat); source("setup-2dtpp-dataset.R"); source("test-recompute-signal.R")
context("plot functions")

test_that("plot2dTppProfile generates ggplot object", {
  gg_profile <- plot2dTppProfile(simulated_cell_extract_df, "protein1")
  expect_identical(class(gg_profile), c("ggplot2::ggplot", "ggplot", 
                                        "ggplot2::gg", "S7_object", "gg"))
})

test_that("plot2dTppProfile generates ggplot object", {
  gg_rel_profile <- plot2dTppRelProfile(simulated_cell_extract_df, "protein1")
  expect_identical(class(gg_rel_profile), c("ggplot2::ggplot", "ggplot", 
                                            "ggplot2::gg", "S7_object", "gg"))
})

test_that("plot2dTppFcHeatmap generates ggplot object", {
    gg_rel_heatmap <- plot2dTppFcHeatmap(
        simulated_cell_extract_df, "tp2", drug_name = "drug1")
    expect_identical(class(gg_rel_heatmap), c("ggplot2::ggplot", "ggplot", 
                                              "ggplot2::gg", "S7_object", "gg"))
})

---
title: "Introduction to TPP2D for 2D-TPP analysis"
author: "Nils Kurzawa"
date: "`r format(Sys.time(), '%d %B, %Y')`"
package: TPP2D
output:
    BiocStyle::html_document
vignette: >
    %\VignetteIndexEntry{Introduction to TPP2D for 2D-TPP analysis}
    %\VignetteEngine{knitr::rmarkdown}
    %VignetteEncoding{UTF-8}
bibliography: bibliography.bib
csl: cell.csl
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

# Abstract
Thermal proteome profiling (TPP) [@Savitski2014; @Mateus2020] is an unbiased 
mass spectrometry-based method to assess protein-ligand interactions. It works 
by employing the cellular thermal shift assay (CETSA) [@Molina2013] on a 
proteome-wide scale which monitors the profiles of proteins in cells 
over a temperature gradient and aims to detect shifts induced by ligand-protein 
interactions. 2D-TPP represents a refined version of the assay [@Becher2016]
which uses a concentration gradient of the ligand of interest over a temperature
gradient. This package aims to analyze data retrieved from 2D-TPP experiments by
a functional analysis approach.

# General information

This package implements a method to detect ligand-protein interactions from
datasets obtained with the 2D-TPP assay. Please note that methods for analyzing
convential TPP datasets (e.g. single dose, melting curve approach) can be found
at: [`TPP`](https://bioconductor.org/packages/release/bioc/html/TPP.html) and 
[`NPARC`](https://bioconductor.org/packages/release/bioc/html/NPARC.html) .

This vignette is not aiming to give an in-depth introduction to thermal 
proteome profiling, please refer to other sources for this purpose:     

- [Original TPP publication](http://science.sciencemag.org/content/346/6205/1255784)     
- [2D-TPP publication](https://www.nature.com/articles/nchembio.2185)
- [review article](https://www.embopress.org/doi/10.15252/msb.20199232)


**Note:** if you use `TPP2D` in published research, please cite:

> Kurzawa, N.\*, Becher, I.\* *et al.* (2020)
> A computational method for detection of ligand-binding proteins from dose range thermal proteome profiles.
> *Nat. Commun.*, [10.1038/s41467-020-19529-8](https://doi.org/10.1038/s41467-020-19529-8)

# Installation

1. Download the package from Bioconductor.

```{r getPackage, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
install.packages("BiocManager")
BiocManager::install("TPP2D")
```

Or install the development version of the package from Github.
```{r, eval = FALSE}
BiocManager::install("nkurzaw/TPP2D")
```

2. Load the package into R session.
```{r Load, message=FALSE}
library(TPP2D)
```

# Introduction

The 2D-TPP assay is usually set up in a way that for each temperature different 
ligand concentrations (including a vehicle condition) are used and two adjacent 
temperatures each are multiplexed in a single mass spectrometry (MS) run. 
Typically up to 10 or 12 temperatures are used in total that add up to 5 
or 6 MS runs respectively (Figure 1).  
```{r echo=FALSE, fig.cap="Experimental 2D-TPP workflow."}
knitr::include_graphics("tpp_2d_schematic.jpg")
```
This package aims to provide a tool for finding ligand-protein interactions, i.e. proteins affected in their thermal stability by the treatment used in the
experiment) at a given false disscovery rate (FDR). Please note that a change in
thermal stability of a protein is not a guarantee for it interacting with the molecule used as treatment. However, we try to give the user additional
information by specifying whether an observed effect is likely due to
stabilization or a change in expression or solubility of a given protein to make the interpretation of detected hits as easy as possible.

# Step-by-step workflow

```{r, message=FALSE, warning=FALSE}
library(dplyr)
library(TPP2D)
```

After having loaded `dplyr` and the `TPP2D` package itself we start by loading 
an example dataset which is supplied with the package. Therefore, we use 
the `import2dDataset` function.     
For this puporse we need to supply a config table that essentially describes 
which experimental conditions the different TMT labels used correspond to and 
supplies paths to the raw data files (note: since this example dataset is 
included in the package it does not contain a "Path" column, this is however 
mandatory if the data should be read in from external raw files).
```{r}
data("config_tab")
data("raw_dat_list")

config_tab
```


We then call the import function (note: we here supply a list of data frames 
for the "data" argument, replacing the raw data files that would be normally 
specified in the above mentioned column of the config table. If this is 
supplied the argument "data" can simply be ignored):
```{r, warning=FALSE}
import_df <- import2dDataset(
    configTable = config_tab,
    data = raw_dat_list,
    idVar = "protein_id",
    intensityStr = "signal_sum_",
    fcStr = "rel_fc_",
    nonZeroCols = "qusm",
    geneNameVar = "gene_name",
    addCol = NULL,
    qualColName = "qupm",
    naStrs = c("NA", "n/d", "NaN"),
    concFactor = 1e6,
    medianNormalizeFC = TRUE,
    filterContaminants = TRUE)

recomp_sig_df <- recomputeSignalFromRatios(import_df)

# resolve ambiguous protein names
preproc_df <- resolveAmbiguousProteinNames(recomp_sig_df)
# alternatively one could choose to run
# preproc_df <- resolveAmbiguousProteinNames(
#     recomp_sig_df, includeIsoforms = TRUE)
```
Please refer to the help page of the function to retrieve in-depth description 
of the different arguments. Essentially the function needs to know the names 
or prefixes of the columns in the raw data files, that contain different 
informations like protein id or the raw or relative signal intensities 
measured for the different TMT labels.    
The imported synthetic dataset consists of 17 simulated protein 2D thermal 
profiles (protein1-17) and 3 spiked-in true positives (tp1-3). It represents 
a data frame with the columns:
```{r, echo=FALSE}
knitr::kable(tibble(
    column = colnames(recomp_sig_df),
    description = 
            c("protein identifier",
            "number of unique quantified peptides",
            "number of unique spectra",
            "gene name",
            "temperature incubated at",
            "experiment identifier",
            "TMT label",
            "RefCol",
            "treatment concentration",
            "raw reporter ion intensity sum",
            paste("raw relative fold change compared to",
                    "vehicle condition at the same temperature"),
            "log10 treatment concentration",
            "median normalized fold change",
            "recomputed reporter ion intensity",
            "recomputed log2 reporter ion intensity"),
    required = 
            c("Yes",
            "No",
            "No",
            "Yes",
            "Yes",
            "No",
            "No",
            "No",
            "No",
            "No",
            "No",
            "Yes",
            "No",
            "No",
            "Yes"))
)
```

Here the column "required" indicates which of these columns is 
neccessary for usage of the downstream functions.    
    
    
We then begin our actual data analysis by fitting two nested models to 
each protein profile: A null model that is expected when a protein profile 
remains unaffected by a given treatment and an alternative model which is 
a constrained sigmoidal dose-response model across all temperatures. 

```{r}
model_params_df <- getModelParamsDf(
    df = preproc_df)
```

The goodness of fit of both models for each protein is then compared and 
a $F$-statistic is computed.

```{r}
fstat_df <- computeFStatFromParams(model_params_df)
```


Then we create a null model using our dataset to be able to estimate the 
FDR for a given $F$-statistic in the next step.

```{r}
set.seed(12, kind = "L'Ecuyer-CMRG")
null_model <- bootstrapNullAlternativeModel(
    df = preproc_df,
    params_df = model_params_df,
    B = 2)
```

Please note that setting $B = 2$ (corresponsing to $2$ 
permutations) is not enough to guarantee faithful FDR estimation, this 
has simply been set for fast demonstration purposes. We recommend to 
use at least $B = 20$ for applications in praxis.    
    
To estimate the FDR for all given $F$-statistics and retrieve all 
significant hits at a set FDR $\alpha$ we use the following functions:

```{r, warning=FALSE}
fdr_tab <- getFDR( 
    df_out = fstat_df,
    df_null = null_model)

hits <- findHits(
    fdr_df = fdr_tab,
    alpha = 0.1)

hits %>% 
    dplyr::select(clustername, nObs, F_statistic, FDR)
```

We can now plot our obtained result as a (one-sided) volcano plot:

```{r}
plot2dTppVolcano(fdr_df = fdr_tab, hits_df = hits)
```



Finally, we can fit and plot proteins that have come up as 
significant in our analysis by using:
```{r}
plot2dTppFit(recomp_sig_df, "tp1", 
             model_type = "H0",
             dot_size = 1,
             fit_color = "darkorange")
```
or respectively for the H1 model:
```{r}
plot2dTppFit(recomp_sig_df, "tp1", 
             model_type = "H1",
             dot_size = 1,
             fit_color = "darkgreen")
```

An alternative option to visualize thermal profiles of proteins of interest is to use `plot2dTppFcHeatmap`:

```{r}
plot2dTppFcHeatmap(recomp_sig_df, "tp1", 
                   drug_name = "drug X")
```
```{r}
plot2dTppFcHeatmap(recomp_sig_df, "tp3", 
                   drug_name = "drug X")
```

# Acknowledgements

We thank the following people for valuable feedback and help with the `TPP2D` package:               
    
- André Mateus
- Constantin Ahlmann-Eltze    
- Alexander Helmboldt               
- Stefan Gade
- Dorothee Childs
- Nikos Ignatiadis              
- Britta Velten             


```{r}
sessionInfo()
```


# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TPP2D-defunct.R, R/fdr_funcs.R
\name{TPP2D-defunct}
\alias{TPP2D-defunct}
\alias{computeFdr}
\title{Defunct functions in package \pkg{TPP2D}.}
\usage{
computeFdr(df_out, df_null)
}
\description{
The functions listed below are defunct and will be removed in
  the near future. When possible, alternative functions with similar
  functionality are also mentioned. Help pages for deprecated functions are
  available at \code{help("-deprecated")}.
}
\details{
TPP2D defunct functions
}
\section{\code{computeFdr}}{

For \code{computeFdr}, use \code{\link{getFDR}}.
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fdr_funcs.R
\name{getPvalues}
\alias{getPvalues}
\title{Compute p-values for given F statistics based on true and
null dataset}
\usage{
getPvalues(df_out, df_null, pseudo_count = 1, squeezeDenominator = FALSE)
}
\arguments{
\item{df_out}{data frame containing results from analysis by
fitAndEvalDataset}

\item{df_null}{data frame containing results from analysis by
bootstrapNull}

\item{pseudo_count}{numeric larger or equal to 0 added to both
counts of protein with an F-statistic higher than a threshold
theta of the true and bootstrapped datasets}

\item{squeezeDenominator}{logical indicating whether F statistic
denominator should be shrinked using limma::squeezeVar}
}
\value{
data frame annotating each protein with a FDR based on 
it's F statistic and number of observations
}
\description{
Compute p-values for given F statistics based on true and
null dataset
}
\examples{
data("simulated_cell_extract_df")
temp_df <- simulated_cell_extract_df \%>\% 
  filter(clustername \%in\% paste0("protein", 1:3)) \%>\% 
  group_by(representative) \%>\% 
  mutate(nObs = n()) \%>\% 
  ungroup 
example_out <- fitAndEvalDataset(temp_df)
example_null <- bootstrapNull(temp_df, B = 2)
getPvalues(
  example_out, 
  example_null)
 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_funcs.R
\name{plot2dTppProfile}
\alias{plot2dTppProfile}
\title{Plot 2D thermal profile intensities of a protein 
of choice}
\usage{
plot2dTppProfile(df, name)
}
\arguments{
\item{df}{tidy data frame of a 2D-TPP dataset}

\item{name}{gene name (clustername) of protein that 
should be visualized}
}
\value{
A ggplot displaying the thermal profile of
a protein of choice in a datset of choice
}
\description{
Plot 2D thermal profile intensities of a protein 
of choice
}
\examples{

data("simulated_cell_extract_df")
plot2dTppProfile(simulated_cell_extract_df, "protein1")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perform_analysis.R
\name{computeFStatFromParams}
\alias{computeFStatFromParams}
\title{Compute F statistics from paramter data frame}
\usage{
computeFStatFromParams(params_df)
}
\arguments{
\item{params_df}{data frame listing all null and alternative
model parameters as obtained by 'getModelParamsDf'}
}
\value{
data frame of all proteins and computed F statistics
and parameters that were used for the computation
}
\description{
Compute F statistics from paramter data frame
}
\examples{
data("simulated_cell_extract_df")
params_df <- getModelParamsDf(simulated_cell_extract_df)
computeFStatFromParams(params_df)
 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_funcs.R
\name{filterOutContaminants}
\alias{filterOutContaminants}
\title{Filter out contaminants}
\usage{
filterOutContaminants(dataLong)
}
\arguments{
\item{dataLong}{long format data frame of imported dataset}
}
\value{
data frame containing full dataset filtered to 
contain no contaminants
}
\description{
Filter out contaminants
}
\examples{
data("simulated_cell_extract_df")
filterOutContaminants(simulated_cell_extract_df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{config_tab}
\alias{config_tab}
\title{Example config table for a import of a simulated 
2D-TPP cell extract dataset}
\format{
"Compound" describing the compound used for the assay, 
"Experiment" listing MS experiment ids of the separate runs
(typically comprising two multiplexed adjacent temperature), 
"Temperature": the temperature used for a given sub-experimet, 
the respective TMT labels "126"-"131L", RefCol referring to
the label used as a reference label for computing relative
fold changes (usually the label used for the control treatment).
}
\usage{
data("config_tab")
}
\description{
Config table fot import of simulated example 
dataset obtained by 2D-TPP experiments for analysis by 
the TPP2D-package. It's a data frame with the columns
"Compound" describing the compound used for the assay, 
"Experiment" listing MS experiment ids of the separate runs
(typically comprising two multiplexed adjacent temperature), 
"Temperature": the temperature used for a given sub-experimet, 
the respective TMT labels "126"-"131L", RefCol referring to
the label used as a reference label for computing relative
fold changes (usually the label used for the control treatment).
Please note that when the data is not supplied as a list of
already imported data frames the config table for the import 
function should be a path to an txt, csv or xlsx file containing
an additional column "Path" listing for each row the respective
path to a searched protein output file.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_funcs.R
\name{import2dMain}
\alias{import2dMain}
\title{Import 2D-TPP dataset main function}
\usage{
import2dMain(
  configTable,
  data,
  idVar,
  fcStr,
  addCol,
  naStrs,
  intensityStr,
  qualColName,
  nonZeroCols
)
}
\arguments{
\item{configTable}{character string of a file path to a config table}

\item{data}{possible list of datasets from different MS runs 
corresponding to a 2D-TPP dataset, circumvents loading datasets 
referencend in config table, default is NULL}

\item{idVar}{character string indicating which data column provides the 
unique identifiers for each protein.}

\item{fcStr}{character string indicating which columns contain the actual 
fold change values. Those column names containing the suffix \code{fcStr} 
will be regarded as containing fold change values.}

\item{addCol}{character string indicating additional column to import}

\item{naStrs}{character vector indicating missing values in the data table. 
When reading data from file, this value will be passed on to the argument 
\code{na.strings} in function \code{read.delim}.}

\item{intensityStr}{character string indicating which columns contain 
raw intensities measurements}

\item{qualColName}{character string indicating which column can be used for 
additional quality criteria when deciding between different non-unique 
protein identifiers.}

\item{nonZeroCols}{column like default qssm that should be imported and
requested to be non-zero in analyzed data}
}
\value{
list of data frames containing different
datasets
}
\description{
Import 2D-TPP dataset main function
}
\examples{
data("config_tab")
data("raw_dat_list")
dataList <- import2dMain(configTable = config_tab,
                         data = raw_dat_list,
                         idVar = "protein_id",
                         fcStr = "rel_fc_",
                         addCol = "gene_name",
                         naStrs = NA,
                         intensityStr = "signal_sum_",
                         nonZeroCols = "qusm",
                         qualColName = "qupm")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{simulated_cell_extract_df}
\alias{simulated_cell_extract_df}
\title{Example subset of a simulated 2D-TPP cell 
extract dataset}
\format{
data frame with columns representative (protein id), 
clustername (gene name), temperature, log_conc, raw_value, 
rel_value, value and log2_value
}
\usage{
data("simulated_cell_extract_df")
}
\description{
Simulated example dataset obtained by 2D-TPP 
experiments for analysis by the TPP2D-package. It contains 
a tidy data frame after import and recomputing of robust 
signal intensities with 200 simulated protein profiles 
(protein1-200) and 3 spiked-in true positives (TP1-3)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_funcs.R
\name{configWide2Long}
\alias{configWide2Long}
\title{Tranform configuration table from wide to long}
\usage{
configWide2Long(configWide)
}
\arguments{
\item{configWide}{data frame containing a config table}
}
\value{
data frame containing config table in long format
}
\description{
Tranform configuration table from wide to long
}
\examples{
data("config_tab")
configWide2Long(configWide = config_tab)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_funcs.R
\name{import2dDataset}
\alias{import2dDataset}
\title{Import 2D-TPP dataset using a config table}
\usage{
import2dDataset(
  configTable,
  data,
  idVar = "representative",
  intensityStr = "sumionarea_protein_",
  fcStr = "rel_fc_protein_",
  nonZeroCols = "qssm",
  geneNameVar = "clustername",
  addCol = NULL,
  qualColName = "qupm",
  naStrs = c("NA", "n/d", "NaN"),
  concFactor = 1e+06,
  medianNormalizeFC = TRUE,
  filterContaminants = TRUE
)
}
\arguments{
\item{configTable}{character string of a file path to a config table}

\item{data}{possible list of datasets from different MS runs 
corresponding to a 2D-TPP dataset, circumvents loading datasets 
referencend in config table, default is NULL}

\item{idVar}{character string indicating which data column provides the 
unique identifiers for each protein.}

\item{intensityStr}{character string indicating which columns contain 
raw intensities measurements}

\item{fcStr}{character string indicating which columns contain the actual 
fold change values. Those column names containing the suffix \code{fcStr} 
will be regarded as containing fold change values.}

\item{nonZeroCols}{column like default qssm that should be imported and
requested to be non-zero in analyzed data}

\item{geneNameVar}{character string of the column name that describes
the gene name of a given protein in the raw data files}

\item{addCol}{character string indicating additional column to import}

\item{qualColName}{character string indicating which column can be used for 
additional quality criteria when deciding between different non-unique 
protein identifiers.}

\item{naStrs}{character vector indicating missing values in the data table. 
When reading data from file, this value will be passed on to the argument 
\code{na.strings} in function \code{read.delim}.}

\item{concFactor}{numeric value that indicates how concentrations need to 
be adjusted to yield total unit e.g. default mmol - 1e6}

\item{medianNormalizeFC}{perform median normalization (default: TRUE).}

\item{filterContaminants}{boolean variable indicating whether data 
should be filtered to exclude contaminants (default: TRUE).}
}
\value{
tidy data frame representing a 2D-TPP dataset
}
\description{
Import 2D-TPP dataset using a config table
}
\examples{
data("config_tab")
data("raw_dat_list")
import_df <- import2dDataset(configTable = config_tab, 
                             data = raw_dat_list,
                             idVar = "protein_id",
                             intensityStr = "signal_sum_",
                             fcStr = "rel_fc_",
                             nonZeroCols = "qusm",
                             geneNameVar = "gene_name",
                             addCol = NULL,
                             qualColName = "qupm",
                             naStrs = c("NA", "n/d", "NaN"),
                             concFactor = 1e6,
                             medianNormalizeFC = TRUE,
                             filterContaminants = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fdr_funcs.R
\name{getFDR}
\alias{getFDR}
\title{Get FDR for given F statistics based on true and
null dataset}
\usage{
getFDR(df_out, df_null, squeezeDenominator = TRUE)
}
\arguments{
\item{df_out}{data frame containing results from analysis by
fitAndEvalDataset}

\item{df_null}{data frame containing results from analysis by
bootstrapNull}

\item{squeezeDenominator}{logical indicating whether F statistic
denominator should be shrinked using limma::squeezeVar}
}
\value{
data frame annotating each protein with a FDR based on 
it's F statistic and number of observations
}
\description{
Get FDR for given F statistics based on true and
null dataset
}
\examples{
data("simulated_cell_extract_df")
temp_df <- simulated_cell_extract_df \%>\% 
  filter(clustername \%in\% paste0("protein", 1:5)) \%>\% 
  group_by(representative) \%>\% 
  mutate(nObs = n()) \%>\% 
  ungroup 
example_out <- fitAndEvalDataset(temp_df)
example_null <- bootstrapNull(temp_df, B = 1)
getFDR(example_out, example_null)
 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/null_bootstrap_funcs.R
\name{bootstrapNullAlternativeModelFast}
\alias{bootstrapNullAlternativeModelFast}
\title{Bootstrap null distribution of F statistics for FDR estimation
based on resampling alternative model residuals with only
one round of model fitting on resampled data and subsequent
resampling of thereby obtained residuals}
\usage{
bootstrapNullAlternativeModelFast(
  df,
  params_df,
  maxit = 500,
  independentFiltering = FALSE,
  fcThres = 1.5,
  minObs = 20,
  optim_fun_h0 = TPP2D:::.min_RSS_h0,
  optim_fun_h1 = TPP2D:::.min_RSS_h1_slope_pEC50,
  optim_fun_h1_2 = NULL,
  gr_fun_h0 = NULL,
  gr_fun_h1 = NULL,
  gr_fun_h1_2 = NULL,
  BPPARAM = BiocParallel::SerialParam(progressbar = TRUE),
  B = 20,
  byMsExp = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{df}{tidy data frame retrieved after import of a 2D-TPP 
dataset, potential filtering and addition of a column "nObs"
containing the number of observations per protein}

\item{params_df}{data frame listing all null and alternative
model parameters as obtained by 'getModelParamsDf'}

\item{maxit}{maximal number of iterations the optimization
should be given, default is set to 500}

\item{independentFiltering}{boolean flag indicating whether
independent filtering should be performed based on minimal
fold changes per protein profile}

\item{fcThres}{numeric value of minimal fold change 
(or inverse fold change) a protein has to show to be kept 
upon independent filtering}

\item{minObs}{numeric value of minimal number of observations
that should be required per protein}

\item{optim_fun_h0}{optimization function that should be used
for fitting the H0 model}

\item{optim_fun_h1}{optimization function that should be used
for fitting the H1 model}

\item{optim_fun_h1_2}{optional additional optimization function 
that will be run with paramters retrieved from optim_fun_h1 and 
should be used for fitting the H1 model with the trimmed sum
model, default is NULL}

\item{gr_fun_h0}{optional gradient function for optim_fun_h0,
default is NULL}

\item{gr_fun_h1}{optional gradient function for optim_fun_h1,
default is NULL}

\item{gr_fun_h1_2}{optional gradient function for optim_fun_h1_2,
default is NULL}

\item{BPPARAM}{BiocParallel parameter for optional parallelization
of null distribution generation through bootstrapping, 
default: BiocParallel::SerialParam()}

\item{B}{numeric value of rounds of bootstrap, default: 20}

\item{byMsExp}{boolean flag indicating whether resampling of 
residuals should be performed separately for data generated by 
different MS experiments, default TRUE, recommended}

\item{verbose}{logical indicating whether to print each 
protein while its profile is boostrapped}
}
\value{
data frame containing F statistics of proteins with 
permuted 2D thermal profiles that are informative on the Null
distribution of F statistics
}
\description{
Bootstrap null distribution of F statistics for FDR estimation
based on resampling alternative model residuals with only
one round of model fitting on resampled data and subsequent
resampling of thereby obtained residuals
}
\examples{
data("simulated_cell_extract_df")
temp_df <- simulated_cell_extract_df \%>\% 
  filter(clustername \%in\% paste0("protein", 1:3)) \%>\% 
  group_by(representative) \%>\% 
  mutate(nObs = n()) \%>\% 
  ungroup 
temp_params_df <- getModelParamsDf(temp_df)
boot_df <- bootstrapNullAlternativeModelFast(
  temp_df, params_df = temp_params_df, B = 20)  

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perform_analysis.R
\name{competeModels}
\alias{competeModels}
\title{Compete H0 and H1 models per protein and obtain F statistic}
\usage{
competeModels(
  df,
  fcThres = 1.5,
  independentFiltering = FALSE,
  minObs = 20,
  optim_fun_h0 = .min_RSS_h0,
  optim_fun_h1 = .min_RSS_h1_slope_pEC50,
  optim_fun_h1_2 = NULL,
  gr_fun_h0 = NULL,
  gr_fun_h1 = NULL,
  gr_fun_h1_2 = NULL,
  maxit = 750
)
}
\arguments{
\item{df}{tidy data frame retrieved after import of a 2D-TPP 
dataset, potential filtering and addition of a column "nObs"
containing the number of observations per protein}

\item{fcThres}{numeric value of minimal fold change 
(or inverse fold change) a protein has to show to be kept 
upon independent filtering}

\item{independentFiltering}{boolean flag indicating whether
independent filtering should be performed based on minimal
fold changes per protein profile}

\item{minObs}{numeric value of minimal number of observations
that should be required per protein}

\item{optim_fun_h0}{optimization function that should be used
for fitting the H0 model}

\item{optim_fun_h1}{optimization function that should be used
for fitting the H1 model}

\item{optim_fun_h1_2}{optional additional optimization function 
that will be run with paramters retrieved from optim_fun_h1 and 
should be used for fitting the H1 model with the trimmed sum
model, default is NULL}

\item{gr_fun_h0}{optional gradient function for optim_fun_h0,
default is NULL}

\item{gr_fun_h1}{optional gradient function for optim_fun_h1,
default is NULL}

\item{gr_fun_h1_2}{optional gradient function for optim_fun_h1_2,
default is NULL}

\item{maxit}{maximal number of iterations the optimization
should be given, default is set to 500}
}
\value{
data frame summarising the fit characteristics of H0 and
H1 models and therof resulting computed F statistics per protein
}
\description{
Compete H0 and H1 models per protein and obtain F statistic
}
\examples{
data("simulated_cell_extract_df")
temp_df <- simulated_cell_extract_df \%>\% 
  filter(clustername \%in\% paste0("protein", 1:10)) \%>\% 
  group_by(representative) \%>\% 
  mutate(nObs = n()) \%>\% 
  ungroup 
competeModels(temp_df)  

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perform_analysis.R
\name{computeFstat}
\alias{computeFstat}
\title{Compute F statistic from H1 and H0 model characteristics}
\usage{
computeFstat(h0_df, h1_df)
}
\arguments{
\item{h0_df}{data frame with H0 model characteristics for each
protein}

\item{h1_df}{data frame with H1 model characteristics for each
protein}
}
\value{
data frame with H0 and H1 model characteristics for each
protein and respectively computed F statistics
}
\description{
Compute F statistic from H1 and H0 model characteristics
}
\examples{
data("simulated_cell_extract_df")
temp_df <- simulated_cell_extract_df \%>\% 
  filter(clustername \%in\% paste0("protein", 1:20)) \%>\% 
  group_by(representative) \%>\% 
  mutate(nObs = n()) \%>\% 
  ungroup 
  
h0_df <- fitH0Model(temp_df)
h1_df <- fitH1Model(temp_df)
  
computeFstat(h0_df, h1_df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perform_analysis.R
\name{fitAndEvalDataset}
\alias{fitAndEvalDataset}
\title{Fit H0 and H1 model to 2D thermal profiles of proteins
and compute F statistic}
\usage{
fitAndEvalDataset(
  df,
  maxit = 500,
  optim_fun_h0 = .min_RSS_h0,
  optim_fun_h1 = .min_RSS_h1_slope_pEC50,
  optim_fun_h1_2 = NULL,
  gr_fun_h0 = NULL,
  gr_fun_h1 = NULL,
  gr_fun_h1_2 = NULL,
  ec50_lower_limit = NULL,
  ec50_upper_limit = NULL,
  slopEC50 = TRUE
)
}
\arguments{
\item{df}{tidy data_frame retrieved after import of a 2D-TPP 
dataset, potential filtering and addition of a column "nObs"
containing the number of observations per protein}

\item{maxit}{maximal number of iterations the optimization
should be given, default is set to 500}

\item{optim_fun_h0}{optimization function that should be used
for fitting the H0 model}

\item{optim_fun_h1}{optimization function that should be used
for fitting the H1 model}

\item{optim_fun_h1_2}{optional additional optimization function 
that will be run with paramters retrieved from optim_fun_h1 and 
should be used for fitting the H1 model with the trimmed sum
model, default is NULL}

\item{gr_fun_h0}{optional gradient function for optim_fun_h0,
default is NULL}

\item{gr_fun_h1}{optional gradient function for optim_fun_h1,
default is NULL}

\item{gr_fun_h1_2}{optional gradient function for optim_fun_h1_2,
default is NULL}

\item{ec50_lower_limit}{lower limit of ec50 parameter}

\item{ec50_upper_limit}{lower limit of ec50 parameter}

\item{slopEC50}{logical flag indicating whether the h1 model is
fitted with a linear model describing the shift od the pEC50 over 
temperatures}
}
\value{
data frame with H0 and H1 model characteristics for each
protein and respectively computed F statistics
}
\description{
Fit H0 and H1 model to 2D thermal profiles of proteins
and compute F statistic
}
\examples{
data("simulated_cell_extract_df")
temp_df <- simulated_cell_extract_df \%>\% 
  group_by(representative) \%>\% 
  mutate(nObs = n()) \%>\% 
  ungroup 
fitAndEvalDataset(temp_df)  

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_funcs.R
\name{annotateDataList}
\alias{annotateDataList}
\title{Annotate imported data list using a config table}
\usage{
annotateDataList(dataList, geneNameVar, configLong, intensityStr, fcStr)
}
\arguments{
\item{dataList}{list of datasets from different MS runs 
corresponding to a 2D-TPP dataset}

\item{geneNameVar}{character string of the column name that describes
the gene name of a given protein in the raw data files}

\item{configLong}{long formatted data frame of a corresponding
config table}

\item{intensityStr}{character string indicating which columns contain 
raw intensities measurements}

\item{fcStr}{character string indicating which columns contain the actual 
fold change values. Those column names containing the suffix \code{fcStr} 
will be regarded as containing fold change values.}
}
\value{
data frame containing all data annotated
by information supplied in the config table
}
\description{
Annotate imported data list using a config table
}
\examples{
data("config_tab")
data("raw_dat_list")
dataList <- import2dMain(configTable = config_tab,
                         data = raw_dat_list,
                         idVar = "protein_id",
                         fcStr = "rel_fc_",
                         addCol = "gene_name",
                         naStrs = NA,
                         intensityStr = "signal_sum_",
                         nonZeroCols = "qusm",
                         qualColName = "qupm")
configLong <- configWide2Long(configWide = config_tab)
annotateDataList(dataList = dataList,
                 geneNameVar = "gene_name",
                 configLong = configLong,
                 intensityStr = "signal_sum_",
                 fcStr = "rel_fc_")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_funcs.R
\name{plot2dTppFit}
\alias{plot2dTppFit}
\title{Plot H0 or H1 fit of 2D thermal profile intensities of 
a protein of choice}
\usage{
plot2dTppFit(
  df,
  name,
  model_type = "H0",
  optim_fun = .min_RSS_h0,
  optim_fun_2 = NULL,
  maxit = 500,
  xlab = "-log10(conc.)",
  ylab = "log2(summed intensities)",
  dot_size = 1,
  line_type = "solid",
  fit_color = "gray30"
)
}
\arguments{
\item{df}{tidy data frame of a 2D-TPP dataset}

\item{name}{gene name (clustername) of protein that 
should be visualized}

\item{model_type}{character string indicating whether
the "H0" or the "H1" model should be fitted}

\item{optim_fun}{optimization function that should be used
for fitting either the H0 or H1 model}

\item{optim_fun_2}{optional additional optimization function 
that will be run with paramters retrieved from optim_fun and 
should be used for fitting the H1 model with the trimmed sum
model, default is NULL}

\item{maxit}{maximal number of iterations the optimization
should be given, default is set to 500}

\item{xlab}{character string of x-axis label of plot}

\item{ylab}{character string of y-axis label of plot}

\item{dot_size}{numeric indicating the size of the data points
to plot}

\item{line_type}{character string defining the line type 
of the fitted curve, default "dashed"}

\item{fit_color}{character string defining the color
of the fitted curve}
}
\value{
A ggplot displaying the thermal profile of
a protein of choice in a datset of choice
}
\description{
Plot H0 or H1 fit of 2D thermal profile intensities of 
a protein of choice
}
\examples{

data("simulated_cell_extract_df")
plot2dTppProfile(simulated_cell_extract_df, "protein1")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/full_analysis.R
\name{runTPP2D}
\alias{runTPP2D}
\title{Run complete TPP2D analysis}
\usage{
runTPP2D(
  df = NULL,
  configTable = NULL,
  data = NULL,
  idVar = "protein_id",
  intensityStr = "signal_sum_",
  fcStr = "rel_fc_",
  nonZeroCols = "qusm",
  geneNameVar = "gene_name",
  addCol = NULL,
  qualColName = "qupm",
  naStrs = c("NA", "n/d", "NaN"),
  concFactor = 1e+06,
  medianNormalizeFC = TRUE,
  filterContaminants = TRUE,
  recomputeSignalRatios = FALSE,
  minObs = 20,
  independentFiltering = FALSE,
  fcThres = 1.5,
  optim_fun_h0 = .min_RSS_h0,
  optim_fun_h1 = .min_RSS_h1_slope_pEC50,
  optim_fun_h1_2 = NULL,
  gr_fun_h0 = NULL,
  gr_fun_h1 = NULL,
  gr_fun_h1_2 = NULL,
  slopEC50 = TRUE,
  maxit = 750,
  BPPARAM = BiocParallel::SerialParam(progressbar = TRUE),
  B = 20,
  byMsExp = TRUE,
  alpha = 0.1
)
}
\arguments{
\item{df}{tidy data_frame retrieved after import of a 2D-TPP 
dataset, potential filtering and addition of a column "nObs"
containing the number of observations per protein}

\item{configTable}{character string of a file path to a config table}

\item{data}{possible list of datasets from different MS runs 
corresponding to a 2D-TPP dataset, circumvents loading datasets 
referencend in config table, default is NULL}

\item{idVar}{character string indicating which data column provides the 
unique identifiers for each protein.}

\item{intensityStr}{character string indicating which columns contain 
raw intensities measurements}

\item{fcStr}{character string indicating which columns contain the actual 
fold change values. Those column names containing the suffix \code{fcStr} 
will be regarded as containing fold change values.}

\item{nonZeroCols}{column like default qssm that should be imported and
requested to be non-zero in analyzed data}

\item{geneNameVar}{character string of the column name that describes
the gene name of a given protein in the raw data files}

\item{addCol}{character string indicating additional column to import}

\item{qualColName}{character string indicating which column can be used for 
additional quality criteria when deciding between different non-unique 
protein identifiers.}

\item{naStrs}{character vector indicating missing values in the data table. 
When reading data from file, this value will be passed on to the argument 
\code{na.strings} in function \code{read.delim}.}

\item{concFactor}{numeric value that indicates how concentrations need to 
be adjusted to yield total unit e.g. default mmol - 1e6}

\item{medianNormalizeFC}{perform median normalization (default: TRUE).}

\item{filterContaminants}{logical variable indicating whether data 
should be filtered to exclude contaminants (default: TRUE).}

\item{recomputeSignalRatios}{logical variable indicaiting whether 
signals should be recomputed from relative fold changes, recommended
if Isobarquant was used for protein quantification}

\item{minObs}{number of minimal observations per protein to include it
in the analysis}

\item{independentFiltering}{logical variable indicating whether
independent filtering should be performed based on minimal
fold changes per protein profile}

\item{fcThres}{numeric value of minimal fold change 
(or inverse fold change) a protein has to show to be kept 
upon independent filtering}

\item{optim_fun_h0}{optimization function that should be used
for fitting the H0 model}

\item{optim_fun_h1}{optimization function that should be used
for fitting the H1 model}

\item{optim_fun_h1_2}{optional additional optimization function 
that will be run with paramters retrieved from optim_fun_h1 and 
should be used for fitting the H1 model with the trimmed sum
model, default is NULL}

\item{gr_fun_h0}{optional gradient function for optim_fun_h0,
default is NULL}

\item{gr_fun_h1}{optional gradient function for optim_fun_h1,
default is NULL}

\item{gr_fun_h1_2}{optional gradient function for optim_fun_h1_2,
default is NULL}

\item{slopEC50}{logical flag indicating whether the h1 model is
fitted with a linear model describing the shift od the pEC50 over 
temperatures}

\item{maxit}{maximal number of iterations the optimization
should be given, default is set to 500}

\item{BPPARAM}{= BiocParallel::SerialParam(progressbar = TRUE),}

\item{B}{numeric value indicating number of rounds of bootstraps
that should be performed to estimate the null distribution}

\item{byMsExp}{logical indicating whether bootstrapping should be
performed within MS experiments}

\item{alpha}{FDR level that should be controlled}
}
\value{
a tpp2dExperiment object
}
\description{
Run complete TPP2D analysis
}
\examples{
data("simulated_cell_extract_df")
runTPP2D(df = simulated_cell_extract_df \%>\% 
   filter(representative \%in\% 1:3),
   B = 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_funcs.R
\name{plot2dTppFcHeatmap}
\alias{plot2dTppFcHeatmap}
\title{Plot heatmap of 2D thermal profile fold changes of 
a protein of choice}
\usage{
plot2dTppFcHeatmap(df, name, drug_name = "", midpoint = 1)
}
\arguments{
\item{df}{tidy data frame of a 2D-TPP dataset}

\item{name}{gene name (clustername) of protein that 
should be visualized}

\item{drug_name}{character string of profiled drug name}

\item{midpoint}{midpoint of fold changes for color
scaling, default: 1}
}
\value{
A ggplot displaying the thermal profile 
as a heatmap of fold changes of
a protein of choice in a dataset of choice
}
\description{
Plot heatmap of 2D thermal profile fold changes of 
a protein of choice
}
\examples{

data("simulated_cell_extract_df")
plot2dTppFcHeatmap(simulated_cell_extract_df, 
 "tp2", drug_name = "drug1")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_funcs.R
\name{gg_qq}
\alias{gg_qq}
\title{Plot qq-plot of true data and bootstrapped null with ggplot}
\usage{
gg_qq(
  x,
  y,
  xlab = "F-statistics from sampled Null distr.",
  ylab = "observed F-statistics",
  alpha = 0.25,
  gg_theme = theme_classic(),
  offset = 1,
  plot_diagonal = TRUE
)
}
\arguments{
\item{x}{vector containing values of values of first 
distribution to compare}

\item{y}{vector containing values of values of secound 
distribution to compare}

\item{xlab}{x-axis label}

\item{ylab}{y-axis label}

\item{alpha}{transparency paramenter between 0 and 1}

\item{gg_theme}{ggplot theme, default is theme_classic()}

\item{offset}{offset for x and y axis on top of maximal 
values}

\item{plot_diagonal}{logical parameter indicating whether
an identity line should be plotted}
}
\value{
A ggplot displaying the qq-plot of a true and a
a bootstrapped null distribution
}
\description{
Plot qq-plot of true data and bootstrapped null with ggplot
}
\examples{

data("simulated_cell_extract_df")
recomputeSignalFromRatios(simulated_cell_extract_df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimate_params.R
\name{getModelParamsDf}
\alias{getModelParamsDf}
\title{Get H0 and H1 model parameters}
\usage{
getModelParamsDf(
  df,
  minObs = 20,
  optim_fun_h0 = .min_RSS_h0,
  optim_fun_h1 = .min_RSS_h1_slope_pEC50,
  optim_fun_h1_2 = NULL,
  gr_fun_h0 = NULL,
  gr_fun_h1 = NULL,
  gr_fun_h1_2 = NULL,
  slopEC50 = TRUE,
  maxit = 500,
  qualColName = "qupm"
)
}
\arguments{
\item{df}{tidy data_frame retrieved after import of a 2D-TPP 
dataset, potential filtering and addition of a column "nObs"
containing the number of observations per protein}

\item{minObs}{numeric value of minimal number of observations
that should be required per protein}

\item{optim_fun_h0}{optimization function that should be used
for fitting the H0 model}

\item{optim_fun_h1}{optimization function that should be used
for fitting the H1 model}

\item{optim_fun_h1_2}{optional additional optimization function 
that will be run with paramters retrieved from optim_fun_h1 and 
should be used for fitting the H1 model with the trimmed sum
model, default is NULL}

\item{gr_fun_h0}{optional gradient function for optim_fun_h0,
default is NULL}

\item{gr_fun_h1}{optional gradient function for optim_fun_h1,
default is NULL}

\item{gr_fun_h1_2}{optional gradient function for optim_fun_h1_2,
default is NULL}

\item{slopEC50}{logical flag indicating whether the h1 model is
fitted with a linear model describing the shift od the pEC50 over 
temperatures}

\item{maxit}{maximal number of iterations the optimization
should be given, default is set to 500}

\item{qualColName}{name of column indicating quantification
quality e.g. number of unique peptides used for quantification,
default: "qupm"}
}
\value{
a data.frame with fitted null and alternative model 
parameters
}
\description{
Get H0 and H1 model parameters
}
\examples{
data("simulated_cell_extract_df")
getModelParamsDf(simulated_cell_extract_df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fdr_funcs.R
\name{computeFdr-defunct}
\alias{computeFdr-defunct}
\title{Compute FDR for given F statistics based on true and
null dataset (old function)}
\description{
Compute FDR for given F statistics based on true and
null dataset (old function)
}
\seealso{
\code{\link{TPP2D-defunct}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_funcs.R
\name{renameColumns}
\alias{renameColumns}
\title{Rename columns of imported data frame}
\usage{
renameColumns(dataLong, idVar, geneNameVar)
}
\arguments{
\item{dataLong}{long format data frame of imported dataset}

\item{idVar}{character string indicating which data column provides the 
unique identifiers for each protein.}

\item{geneNameVar}{character string of the column name that describes
the gene name of a given protein in the raw data files}
}
\value{
data frame containing imported data with renamed
columns
}
\description{
Rename columns of imported data frame
}
\examples{
data("config_tab")
data("raw_dat_list")

dataList <- import2dMain(configTable = config_tab,
                         data = raw_dat_list,
                         idVar = "protein_id",
                         fcStr = "rel_fc_",
                         addCol = "gene_name",
                         naStrs = NA,
                         intensityStr = "signal_sum_",
                         nonZeroCols = "qusm",
                         qualColName = "qupm")
configLong <- configWide2Long(configWide = config_tab)
annoDat <- annotateDataList(dataList = dataList,
                            geneNameVar = "gene_name",
                            configLong = configLong,
                            intensityStr = "signal_sum_",
                            fcStr = "rel_fc_")
renameColumns(annoDat, 
              idVar = "protein_id", 
              geneNameVar = "gene_name")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perform_analysis.R
\name{getPEC504Temperature}
\alias{getPEC504Temperature}
\title{Get pEC50 for a protein of interest at a specific temperatures
(optimally the melting point of the protein)}
\usage{
getPEC504Temperature(fstat_df, protein, temperaturePEC50 = 60)
}
\arguments{
\item{fstat_df}{data frame as obtained after calling 
\code{getModelParamsDf}, containing fitted null and 
alternative model parameters for each protein}

\item{protein}{character string referring to the protein 
of interest}

\item{temperaturePEC50}{temperature (numeric) at which pEC50
should be inferred}
}
\value{
numeric value specifying the pEC50 for the 
indicated protein and temperature
}
\description{
Get pEC50 for a protein of interest at a specific temperatures
(optimally the melting point of the protein)
}
\examples{
data("simulated_cell_extract_df")

model_params_df <- getModelParamsDf(
   df = filter(simulated_cell_extract_df, 
           clustername == "tp1"))

getPEC504Temperature(
    fstat_df = model_params_df, 
    protein = "tp1", 
    temperaturePEC50 = 60)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_funcs.R
\name{TPP_importCheckConfigTable}
\alias{TPP_importCheckConfigTable}
\title{Import and chech configuration table}
\usage{
TPP_importCheckConfigTable(infoTable, type = "2D")
}
\arguments{
\item{infoTable}{character string of a file path to
a config table (excel,txt or csv file) or data frame
containing a config table}

\item{type}{charater string indicating dataset type
default is 2D}
}
\value{
data frame with config table
}
\description{
Import and chech configuration table
}
\examples{
data("config_tab")
TPP_importCheckConfigTable(config_tab, type = "2D")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_funcs.R
\name{plot2dTppRelProfile}
\alias{plot2dTppRelProfile}
\title{Plot 2D thermal profile ratios of a protein of choice}
\usage{
plot2dTppRelProfile(df, name)
}
\arguments{
\item{df}{tidy data frame of a 2D-TPP dataset}

\item{name}{gene name (clustername) of protein that 
should be visualized}
}
\value{
A ggplot displaying the thermal profile ratios of
a protein of choice in a datset of choice
}
\description{
Plot 2D thermal profile ratios of a protein of choice
}
\examples{

data("simulated_cell_extract_df")
plot2dTppRelProfile(simulated_cell_extract_df, "protein1")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fdr_funcs.R
\name{findHits}
\alias{findHits}
\title{Find hits according to FDR threshold}
\usage{
findHits(fdr_df, alpha)
}
\arguments{
\item{fdr_df}{data frame obtained from computeFdr}

\item{alpha}{significance threshold, default is set to 0.1}
}
\value{
data frame of significant hits at FDR <= alpha
}
\description{
Find hits according to FDR threshold
}
\examples{
data("simulated_cell_extract_df")
temp_df <- simulated_cell_extract_df \%>\% 
  filter(clustername \%in\% paste0("protein", 1:5)) \%>\% 
  group_by(representative) \%>\% 
  mutate(nObs = n()) \%>\% 
  ungroup 
example_out <- fitAndEvalDataset(temp_df)
example_null <- bootstrapNull(temp_df, B = 1)
fdr_df <- getFDR(example_out, example_null)
findHits(fdr_df, 0.1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perform_analysis.R
\name{fitH1Model}
\alias{fitH1Model}
\title{Fit H1 model and evaluate fit statistics}
\usage{
fitH1Model(
  df,
  maxit = 500,
  optim_fun = .min_RSS_h1_slope_pEC50,
  optim_fun_2 = NULL,
  gr_fun = NULL,
  gr_fun_2 = NULL,
  ec50_lower_limit = NULL,
  ec50_upper_limit = NULL,
  slopEC50 = TRUE
)
}
\arguments{
\item{df}{tidy data_frame retrieved after import of a 2D-TPP 
dataset, potential filtering and addition of a column "nObs"
containing the number of observations per protein}

\item{maxit}{maximal number of iterations the optimization
should be given, default is set to 500}

\item{optim_fun}{optimization function that should be used
for fitting the H0 model}

\item{optim_fun_2}{optional secound optimization function for
fitting the H1 model that should be used based on the fitted 
parameters of the optimizationfor based on optim_fun}

\item{gr_fun}{optional gradient function for optim_fun,
default is NULL}

\item{gr_fun_2}{optional gradient function for optim_fun_2,
default is NULL}

\item{ec50_lower_limit}{lower limit of ec50 parameter}

\item{ec50_upper_limit}{lower limit of ec50 parameter}

\item{slopEC50}{logical flag indicating whether the h1 model is
fitted with a linear model describing the shift od the pEC50 over 
temperatures}
}
\value{
data frame with H1 model characteristics for each
protein
}
\description{
Fit H1 model and evaluate fit statistics
}
\examples{

data("simulated_cell_extract_df")
temp_df <- simulated_cell_extract_df \%>\% 
  filter(clustername \%in\% paste0("protein", 1:5)) \%>\% 
  group_by(representative) \%>\% 
  mutate(nObs = n()) \%>\% 
  ungroup
  
fitH1Model(temp_df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perform_analysis.R
\name{fitH0Model}
\alias{fitH0Model}
\title{Fit H0 model and evaluate fit statistics}
\usage{
fitH0Model(df, maxit = 500, optim_fun = .min_RSS_h0, gr_fun = NULL)
}
\arguments{
\item{df}{tidy data_frame retrieved after import of a 2D-TPP 
dataset, potential filtering and addition of a column "nObs"
containing the number of observations per protein}

\item{maxit}{maximal number of iterations the optimization
should be given, default is set to 500}

\item{optim_fun}{optimization function that should be used
for fitting the H0 model}

\item{gr_fun}{optional gradient function for optim_fun,
default is NULL}
}
\value{
data frame with H0 model characteristics for each
protein
}
\description{
Fit H0 model and evaluate fit statistics
}
\examples{

data("simulated_cell_extract_df")
temp_df <- simulated_cell_extract_df \%>\% 
  filter(clustername \%in\% paste0("protein", 1:5)) \%>\% 
  group_by(representative) \%>\% 
  mutate(nObs = n()) \%>\% 
  ungroup 
  
fitH0Model(temp_df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{raw_dat_list}
\alias{raw_dat_list}
\title{Example raw data for a subset of a simulated 
2D-TPP cell extract dataset}
\format{
list of data frames with columns representative 
(protein id), clustername (gene name), temperature, log_conc, 
raw_value, rel_value, value and log2_value
}
\usage{
data("raw_dat_list")
}
\description{
Simulated example dataset obtained by 2D-TPP 
experiments for analysis by the TPP2D-package. It contains 
a list of data frames resembling raw data files returned 
from a MS database search with 200 simulated protein profiles 
(protein1-200) and 3 spiked-in true positives (TP1-3).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tpp2dExperiment-class.R
\docType{class}
\name{tpp2dExperiment-class}
\alias{tpp2dExperiment-class}
\alias{tpcaResult}
\title{S4 TPP2D Experiment Class}
\value{
an object of class tpp2dExperiment
}
\description{
S4 TPP2D Experiment Class
}
\section{Slots}{

\describe{
\item{\code{configTable}}{data.frame.}

\item{\code{idVar}}{character.}

\item{\code{intensityStr}}{character.}

\item{\code{fcStr}}{character.}

\item{\code{nonZeroCols}}{character.}

\item{\code{geneNameVar}}{character.}

\item{\code{qualColName}}{character.}

\item{\code{naStrs}}{character.}

\item{\code{concFactor}}{numeric.}

\item{\code{medianNormalizeFC}}{logical.}

\item{\code{filterContaminants}}{logical.}

\item{\code{minObs}}{numeric.}

\item{\code{independentFiltering}}{logical.}

\item{\code{fcThres}}{numeric.}

\item{\code{optim_fun_h0}}{function.}

\item{\code{optim_fun_h1}}{function.}

\item{\code{slopEC50}}{logical.}

\item{\code{maxit}}{numeric.}

\item{\code{BPPARAM}}{character.}

\item{\code{B}}{numeric}

\item{\code{byMsExp}}{logical.}

\item{\code{alpha}}{numeric.}

\item{\code{tidyDataTable}}{data.frame.}

\item{\code{modelParamsDf}}{data.frame}

\item{\code{resultTable}}{data.frame}

\item{\code{bootstrapNullDf}}{data.frame}

\item{\code{hitTable}}{data.frame}
}}

\examples{
tpp2dObj <- new("tpp2dExperiment")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/null_bootstrap_funcs.R
\name{bootstrapNull}
\alias{bootstrapNull}
\title{Bootstrap null distribution of F statistics for FDR estimation}
\usage{
bootstrapNull(
  df,
  maxit = 500,
  independentFiltering = FALSE,
  fcThres = 1.5,
  minObs = 20,
  optim_fun_h0 = .min_RSS_h0,
  optim_fun_h1 = .min_RSS_h1_slope_pEC50,
  optim_fun_h1_2 = NULL,
  gr_fun_h0 = NULL,
  gr_fun_h1 = NULL,
  gr_fun_h1_2 = NULL,
  ncores = 1,
  B = 20,
  byMsExp = TRUE
)
}
\arguments{
\item{df}{tidy data_frame retrieved after import of a 2D-TPP 
dataset, potential filtering and addition of a column "nObs"
containing the number of observations per protein}

\item{maxit}{maximal number of iterations the optimization
should be given, default is set to 500}

\item{independentFiltering}{boolean flag indicating whether
independent filtering should be performed based on minimal
fold changes per protein profile}

\item{fcThres}{numeric value of minimal fold change 
(or inverse fold change) a protein has to show to be kept 
upon independent filtering}

\item{minObs}{numeric value of minimal number of observations
that should be required per protein}

\item{optim_fun_h0}{optimization function that should be used
for fitting the H0 model}

\item{optim_fun_h1}{optimization function that should be used
for fitting the H1 model}

\item{optim_fun_h1_2}{optional additional optimization function 
that will be run with paramters retrieved from optim_fun_h1 and 
should be used for fitting the H1 model with the trimmed sum
model, default is NULL}

\item{gr_fun_h0}{optional gradient function for optim_fun_h0,
default is NULL}

\item{gr_fun_h1}{optional gradient function for optim_fun_h1,
default is NULL}

\item{gr_fun_h1_2}{optional gradient function for optim_fun_h1_2,
default is NULL}

\item{ncores}{numeric value of numbers of cores that the function 
should use to parallelize}

\item{B}{numeric value of rounds of bootstrap, default: 20}

\item{byMsExp}{boolean flag indicating whether resampling of 
residuals should be performed separately for data generated by 
different MS experiments, default TRUE, recommended}
}
\value{
data frame containing F statistics of proteins with 
permuted 2D thermal profiles that are informative on the Null
distribution of F statistics
}
\description{
Bootstrap null distribution of F statistics for FDR estimation
}
\examples{
data("simulated_cell_extract_df")
temp_df <- simulated_cell_extract_df \%>\% 
  filter(clustername \%in\% paste0("protein", 1:3)) \%>\% 
  group_by(representative) \%>\% 
  mutate(nObs = n()) \%>\% 
  ungroup 
boot_df <- bootstrapNull(temp_df, B = 2/10)  

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepare_data.R
\name{resolveAmbiguousProteinNames}
\alias{resolveAmbiguousProteinNames}
\title{Resolve ambiguous protein names}
\usage{
resolveAmbiguousProteinNames(df, includeIsoforms = FALSE)
}
\arguments{
\item{df}{tidy data_frame retrieved after 
import of a 2D-TPP dataset}

\item{includeIsoforms}{logical indicating
whether protein isoform should be kept for
analysis}
}
\value{
data frame with resolved protein name
ambiguity
}
\description{
Resolve ambiguous protein names
}
\examples{
tst_df <- bind_rows(tibble(representative = rep(1:3, each = 3), 
                           clustername = rep(letters[1:3], each = 3)), 
                    tibble(representative = rep(c(4, 5), c(3, 2)), 
                           clustername = rep(c("a", "b"), c(3, 2))))
                           
resolveAmbiguousProteinNames(tst_df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepare_data.R
\name{recomputeSignalFromRatios}
\alias{recomputeSignalFromRatios}
\title{Recompute robust signal intensities based on 
bootstrapped TMT channel ratios}
\usage{
recomputeSignalFromRatios(df)
}
\arguments{
\item{df}{tidy data_frame retrieved after 
import of a 2D-TPP dataset}
}
\value{
A data_frame with recomputed signal 
intensities (columname: value) and log2 
transformed signal intensities (columnanme: log2_value) 
that more reliably reflect relative ratios 
between the TMT channels
}
\description{
Recompute robust signal intensities based on 
bootstrapped TMT channel ratios
}
\examples{

data("simulated_cell_extract_df")
recomputeSignalFromRatios(simulated_cell_extract_df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_funcs.R
\name{plot2dTppVolcano}
\alias{plot2dTppVolcano}
\title{Plot Volcano plot of TPP2D results}
\usage{
plot2dTppVolcano(
  fdr_df,
  hits_df,
  alpha = 0.5,
  title_string = "",
  x_lim = NULL,
  y_lim = NULL,
  facet_by_obs = FALSE
)
}
\arguments{
\item{fdr_df}{data frame obtained from `getFDR`}

\item{hits_df}{hits_df data frame obtained from `findHits`}

\item{alpha}{transparency level of plotted points}

\item{title_string}{character argument handed over to ggtitle}

\item{x_lim}{vector with two numerics indicating the x axis limits}

\item{y_lim}{vector with two numerics indicating the y axis limits}

\item{facet_by_obs}{logical indicating whether plot should be facetted
by number of observations, default: FALSE}
}
\value{
a ggplot displaying a volcano plot of the results
obtained after a TPP2D analysis
}
\description{
Plot Volcano plot of TPP2D results
}
\examples{
data("simulated_cell_extract_df")
temp_df <- simulated_cell_extract_df \%>\%
  filter(clustername \%in\% paste0("protein", 1:5)) \%>\%
  group_by(representative) \%>\%
  mutate(nObs = n()) \%>\%
  ungroup
example_params <- getModelParamsDf(temp_df)
example_fstat <- computeFStatFromParams(example_params)
example_null <- bootstrapNullAlternativeModel(
   df = temp_df, params_df = example_params,
   B = 2)
fdr_df <- getFDR(example_fstat, example_null)
hits_df <- findHits(fdr_df, 0.1)
plot2dTppVolcano(fdr_df = fdr_df, hits_df = hits_df)
}
